@@ -3,10 +3,10 @@
  *
  *      Filename:  allocator.h
  *
- *      Description:  Header File allocator Module. 
+ *      Description:  Header File allocator Module.
  *
- *      Version:   4.1
- *      Released:  13.6.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  none
@@ -27,7 +27,6 @@
  *
  * =======================================================================================
  */
-
 #ifndef ALLOCATOR_H
 #define ALLOCATOR_H
 
@@ -39,12 +38,13 @@
 
 extern void allocator_init(int numVectors);
 extern void allocator_finalize();
+extern size_t allocator_dataTypeLength(DataType type);
 extern void allocator_allocateVector(void** ptr,
-        int alignment,
-        uint64_t size,
-        int offset,
-        DataType type,
-        bstring domain);
+                int alignment,
+                uint64_t size,
+                int offset,
+                DataType type,
+                int stride,
+                bstring domain);
 
 #endif /*ALLOCATOR_H*/
-
