@@ -5,8 +5,8 @@
  *
  *      Description:  A flexible and extensible benchmarking toolbox
  *
- *      Version:   4.1
- *      Released:  13.6.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
@@ -27,6 +27,9 @@
  *
  * =======================================================================================
  */
+
+/* #####   HEADER FILE INCLUDES   ######################################### */
+
 #include <stdlib.h>
 #include <errno.h>
 #include <stdio.h>
@@ -35,6 +38,7 @@
 #include <unistd.h>
 #include <ctype.h>
 #include <inttypes.h>
+#include <math.h>
 
 #include <bstrlib.h>
 #include <errno.h>
@@ -118,6 +122,7 @@ int main(int argc, char** argv)
     int tmp = 0;
     double time;
     double cycPerUp = 0.0;
+    double cycPerCL = 0.0;
     const TestCase* test = NULL;
     uint64_t realSize = 0;
     uint64_t realIter = 0;
@@ -188,10 +193,9 @@ int main(int argc, char** argv)
                 else
                 {
                     ownprintf("Name: %s\n",test->name);
+                    ownprintf("Description: %s\n",test->desc);
                     ownprintf("Number of streams: %d\n",test->streams);
                     ownprintf("Loop stride: %d\n",test->stride);
-                    ownprintf("Flops: %d\n",test->flops);
-                    ownprintf("Bytes: %d\n",test->bytes);
                     switch (test->type)
                     {
                         case INT:
@@ -204,6 +208,23 @@ int main(int argc, char** argv)
                             ownprintf("Data Type: Double precision float\n");
                             break;
                     }
+                    ownprintf("Flops per element: %d\n",test->flops);
+                    ownprintf("Bytes per element: %d\n",test->bytes);
+                    if (test->loads > 0 && test->stores > 0)
+                    {
+                        double ratio = (double)test->loads/(double)(test->stores+test->loads);
+                        double load_bytes = ((double)test->bytes) * ratio;
+                        ownprintf("Load bytes per element: %.0f\n", load_bytes);
+                        ownprintf("Store bytes per element: %.0f\n",((double)test->bytes) - load_bytes);
+                    }
+                    else if (test->loads >= 0 && test->stores == 0)
+                    {
+                        ownprintf("Load bytes per element: %d\n",test->bytes);
+                    }
+                    else if (test->loads == 0 && test->stores > 0)
+                    {
+                        ownprintf("Store bytes per element: %d\n",test->bytes);
+                    }
                     if (test->loads >= 0)
                     {
                         ownprintf("Load Ops: %d\n",test->loads);
@@ -224,6 +245,10 @@ int main(int argc, char** argv)
                     {
                         ownprintf("Loop instructions: %d\n",test->instr_loop);
                     }
+                    if (test->uops >= 0)
+                    {
+                        ownprintf("Loop micro Ops (\u03BCOPs): %d\n",test->uops);
+                    }
                 }
                 bdestroy(testcase);
                 exit (EXIT_SUCCESS);
@@ -326,8 +351,13 @@ int main(int argc, char** argv)
                 bstring groupstr = bfromcstr(optarg);
                 i = bstr_to_workgroup(currentWorkgroup, groupstr, test->type, test->streams);
                 bdestroy(groupstr);
+                size_t newsize = 0;
+                size_t stride = test->stride;
+                int nrThreads = currentWorkgroup->numberOfThreads;
+                size_t orig_size = currentWorkgroup->size;
                 if (i == 0)
                 {
+                    int warn_once = 1;
                     for (i=0; i<  test->streams; i++)
                     {
                         if (currentWorkgroup->streams[i].offset%test->stride)
@@ -335,12 +365,26 @@ int main(int argc, char** argv)
                             fprintf (stderr, "Error: Stream %d: offset is not a multiple of stride!\n",i);
                             return EXIT_FAILURE;
                         }
+                        if ((int)(floor(orig_size/currentWorkgroup->numberOfThreads)) % test->stride)
+                        {
+                            newsize = (((int)(floor(orig_size/nrThreads))/stride)*(stride))*nrThreads;
+                            if (warn_once)
+                            {
+                                fprintf (stderr, "Warning: Sanitizing vector length to a multiple of the loop stride %d and thread count %d from %d elements (%d bytes) to %d elements (%d bytes)\n",stride, nrThreads, orig_size, orig_size*test->bytes, newsize, newsize*test->bytes);
+                                warn_once = 0;
+                            }
+                        }
+                        else
+                        {
+                            newsize = orig_size;
+                        }
                         allocator_allocateVector(&(currentWorkgroup->streams[i].ptr),
-                                PAGE_ALIGNMENT,
-                                currentWorkgroup->size,
-                                currentWorkgroup->streams[i].offset,
-                                test->type,
-                                currentWorkgroup->streams[i].domain);
+                                                    PAGE_ALIGNMENT,
+                                                    newsize,
+                                                    currentWorkgroup->streams[i].offset,
+                                                    test->type,
+                                                    test->stride,
+                                                    currentWorkgroup->streams[i].domain);
                     }
                     tmp++;
                 }
@@ -348,6 +392,8 @@ int main(int argc, char** argv)
                 {
                     exit(EXIT_FAILURE);
                 }
+                if (newsize != currentWorkgroup->size)
+                    currentWorkgroup->size = newsize;
                 break;
             default:
                 continue;
@@ -453,8 +499,6 @@ int main(int argc, char** argv)
         }
     }
 
-
-
     time = (double) maxCycles / (double) cyclesClock;
     ownprintf(bdata(HLINE));
     ownprintf("Cycles:\t\t\t%" PRIu64 "\n", maxCycles);
@@ -463,43 +507,62 @@ int main(int argc, char** argv)
     ownprintf("Time:\t\t\t%e sec\n", time);
     ownprintf("Iterations:\t\t%" PRIu64 "\n", realIter);
     ownprintf("Iterations per thread:\t%" PRIu64 "\n",threads_data[0].data.iter);
-    ownprintf("Inner loop executions:\t%.0f\n", ((double)realSize)/((double)test->stride));
-    ownprintf("Size:\t\t\t%" PRIu64 "\n",  realSize*test->bytes );
+    ownprintf("Inner loop executions:\t%d\n", (int)(((double)realSize)/((double)test->stride*globalNumberOfThreads)));
+    ownprintf("Size (Byte):\t\t%" PRIu64 "\n",  realSize*test->bytes );
     ownprintf("Size per thread:\t%" PRIu64 "\n", threads_data[0].data.size*test->bytes);
     ownprintf("Number of Flops:\t%" PRIu64 "\n", (threads_data[0].data.iter * realSize *  test->flops));
     ownprintf("MFlops/s:\t\t%.2f\n",
             1.0E-06 * ((double) threads_data[0].data.iter * realSize *  test->flops/  time));
-    
-    ownprintf("Data volume (Byte):\t%llu\n", LLU_CAST (threads_data[0].data.iter * realSize *  test->bytes));
+    ownprintf("Data volume (Byte):\t%llu\n",
+            LLU_CAST (threads_data[0].data.iter * realSize *  test->bytes));
     ownprintf("MByte/s:\t\t%.2f\n",
             1.0E-06 * ( (double) threads_data[0].data.iter * realSize *  test->bytes/ time));
 
-    cycPerUp = ((double) maxCycles / (double) (threads_data[0].data.iter * realSize));
-    ownprintf("Cycles per update:\t%f\n", cycPerUp);
-
+    cycPerCL = ((double) maxCycles / (double) (threads_data[0].data.iter*(realSize/test->streams)* 8 / 64.0));
     switch ( test->type )
     {
         case INT:
         case SINGLE:
-            ownprintf("Cycles per cacheline:\t%f\n", (16.0 * cycPerUp));
+            cycPerUp = cycPerCL/(16.0*test->streams);
             break;
         case DOUBLE:
-            ownprintf("Cycles per cacheline:\t%f\n", (8.0 * cycPerUp));
+            cycPerUp = cycPerCL/(8.0*test->streams);
             break;
     }
+    ownprintf("Cycles per update:\t%f\n", cycPerUp);
+    ownprintf("Cycles per cacheline:\t%f\n", cycPerCL);
     ownprintf("Loads per update:\t%ld\n", test->loads );
     ownprintf("Stores per update:\t%ld\n", test->stores );
+    if (test->loads > 0 && test->stores > 0)
+    {
+        double ratio = (double)test->loads/(double)(test->stores+test->loads);
+        double load_bytes = ((double)test->bytes) * ratio;
+        ownprintf("Load bytes per element:\t%.0f\n", load_bytes);
+        ownprintf("Store bytes per elem.:\t%.0f\n",((double)test->bytes) - load_bytes);
+    }
+    else if (test->loads >= 0 && test->stores == 0)
+    {
+        ownprintf("Load bytes per element:\t%d\n",test->bytes);
+        ownprintf("Store bytes per elem.:\t0\n");
+    }
+    else if (test->loads == 0 && test->stores > 0)
+    {
+        ownprintf("Load bytes per element:\t0\n");
+        ownprintf("Store bytes per elem.:\t%d\n",test->bytes);
+    }
     if ((test->loads > 0) && (test->stores > 0))
     {
         ownprintf("Load/store ratio:\t%.2f\n", ((double)test->loads)/((double)test->stores) );
     }
     if ((test->instr_loop > 0) && (test->instr_const > 0))
     {
-        ownprintf("Instructions:\t\t%" PRIu64 "\n", LLU_CAST ((double)realSize/test->stride)*test->instr_loop*threads_data[0].data.iter + test->instr_const );
+        ownprintf("Instructions:\t\t%" PRIu64 "\n",
+                LLU_CAST ((double)realSize/test->stride)*test->instr_loop*threads_data[0].data.iter + test->instr_const );
     }
     if (test->uops > 0)
     {
-        ownprintf("UOPs:\t\t\t%" PRIu64 "\n", LLU_CAST ((double)realSize/test->stride)*test->uops*threads_data[0].data.iter);
+        ownprintf("UOPs:\t\t\t%" PRIu64 "\n",
+                LLU_CAST ((double)realSize/test->stride)*test->uops*threads_data[0].data.iter);
     }
 
     ownprintf(bdata(HLINE));
