@@ -5,8 +5,8 @@
  *
  *      Description:  Benchmarking framework for likwid-bench
  *
- *      Version:   4.1
- *      Released:  13.6.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:  Jan Treibig (jt), jan.treibig@gmail.com
  *               Thomas Roehl (tr), thomas.roehl@googlemail.com
@@ -28,6 +28,7 @@
  *
  * =======================================================================================
  */
+
 /* #####   HEADER FILE INCLUDES   ######################################### */
 
 #include <pthread.h>
@@ -43,14 +44,10 @@
 #include <barrier.h>
 #include <likwid.h>
 
-/* #####   EXPORTED VARIABLES   ########################################### */
-
-
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
 
 #define BARRIER   barrier_synchronize(&barr)
 
-
 #define EXECUTE(func)   \
     BARRIER; \
     LIKWID_MARKER_START("bench");  \
@@ -66,11 +63,10 @@
     BARRIER
 
 
-
-
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
-void* runTest(void* arg)
+void*
+runTest(void* arg)
 {
     int threadId;
     int offset;
@@ -90,12 +86,15 @@ void* runTest(void* arg)
     barrier_registerThread(&barr, 0, data->globalThreadId);
 
     /* Prepare ptrs for thread */
-    vecsize = myData->size;
+    vecsize = myData->size / data->numberOfThreads;
     size = myData->size / data->numberOfThreads;
-    myData->size = size;
-    size -= (size % myData->test->stride);
-    offset = data->threadId * size;
     
+    size -= (size % myData->test->stride);
+    myData->size = size;
+    offset = data->threadId * size;
+    //printf("Orig size %lu Size %lu\n", myData->size / data->numberOfThreads, size);
+    if (size != vecsize && data->threadId == 0)
+        printf("Sanitizing vector length to a multiple of the loop stride from %d elements (%d bytes) to %d elements (%d bytes)\n", vecsize, vecsize*myData->test->bytes, size, size*myData->test->bytes);
 
     switch ( myData->test->type )
     {
@@ -134,7 +133,6 @@ void* runTest(void* arg)
             break;
     }
 
-
     /* pin the thread */
     likwid_pinThread(myData->processors[threadId]);
     printf("Group: %d Thread %d Global Thread %d running on core %d - Vector length %llu Offset %d\n",
@@ -142,7 +140,7 @@ void* runTest(void* arg)
             threadId,
             data->globalThreadId,
             affinity_threadGetProcessorId(),
-            LLU_CAST vecsize,
+            LLU_CAST size,
             offset);
     BARRIER;
 
@@ -438,7 +436,6 @@ void* runTest(void* arg)
     pthread_exit(NULL);
 }
 
-
 #define MEASURE(func) \
     iterations = 8; \
     while (1) \
@@ -455,12 +452,12 @@ void* runTest(void* arg)
             break; \
     } \
 
-
-void* getIterSingle(void* arg)
+void*
+getIterSingle(void* arg)
 {
     int threadId = 0;
     int offset = 0;
-    size_t size = 0;
+    size_t size = 0, vecsize = 0;
     size_t i;
     ThreadData* data;
     ThreadUserData* myData;
@@ -473,7 +470,13 @@ void* getIterSingle(void* arg)
     func = myData->test->kernel;
     threadId = data->threadId;
 
-    size = myData->size - (myData->size % myData->test->stride);
+    //size = myData->size - (myData->size % myData->test->stride);
+    vecsize = myData->size;
+    size = myData->size / data->numberOfThreads;
+    
+    size -= (size % myData->test->stride);
+    offset = data->threadId * size;
+
     likwid_pinThread(myData->processors[threadId]);
 
 #ifdef DEBUG_LIKWID
@@ -768,3 +771,4 @@ void* getIterSingle(void* arg)
 #endif
     return NULL;
 }
+
