@@ -5,8 +5,8 @@
  *
  *      Description:  Implementation of access daemon.
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Authors:  Michael Meier, michael.meier@rrze.fau.de
  *                Jan Treibig (jt), jan.treibig@gmail.com,
@@ -29,7 +29,9 @@
  *
  * =======================================================================================
  */
+
 /* #####   HEADER FILE INCLUDES   ######################################### */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdint.h>
@@ -54,22 +56,24 @@
 #include <perfmon_sandybridgeEP_counters.h>
 #include <perfmon_broadwelld_counters.h>
 #include <perfmon_broadwellEP_counters.h>
+#include <perfmon_knl_counters.h>
 #include <topology.h>
 #include <cpuid.h>
 #include <lock.h>
 
-
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
+
 #define SA struct sockaddr
 #define str(x) #x
 
 #define CHECK_FILE_ERROR(func, msg)  \
     if ((func) == 0) { syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); }
 
-
-
-
-
+#define LOG_AND_EXIT_IF_ERROR(func, msg)  \
+    if ((func) < 0) {  \
+        syslog(LOG_ERR, "ERROR - [%s:%d] " str(msg) " - %s \n", __FILE__, __LINE__, strerror(errno)); \
+        exit(EXIT_FAILURE); \
+    }
 
 #define PCI_ROOT_PATH    "/proc/bus/pci/"
 #define MAX_PATH_LENGTH   80
@@ -80,10 +84,12 @@
  * with an external monitoring system. */
 
 /* #####   TYPE DEFINITIONS   ########### */
+
 typedef int (*AllowedPrototype)(uint32_t);
 typedef int (*AllowedPciPrototype)(PciDeviceType, uint32_t);
 
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
+
 static int sockfd = -1;
 static int connfd = -1; /* temporary in to make it compile */
 static char* filepath;
@@ -95,6 +101,7 @@ static int FD_PCI[MAX_NUM_NODES][MAX_NUM_PCI_DEVICES];
 static int isPCIUncore = 0;
 static PciDevice* pci_devices_daemon = NULL;
 static char pci_filepath[MAX_PATH_LENGTH];
+static int num_pmc_counters = 0;
 
 /* Socket to bus mapping -- will be determined at runtime;
  * typical mappings are:
@@ -106,13 +113,14 @@ static char pci_filepath[MAX_PATH_LENGTH];
  */
 static char* socket_bus[MAX_NUM_NODES] = { [0 ... (MAX_NUM_NODES-1)] = NULL};
 
-
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 
-static int allowed_intel(uint32_t reg)
+static int
+allowed_intel(uint32_t reg)
 {
-    if ( ((reg & 0x0F8U) == 0x0C0U) ||
-            ((reg & 0xFF0U) == 0x180U) ||
+    if ( ((reg & 0x0F0U) == 0x0C0U) ||
+            ((reg & 0x190U) == 0x180U) ||
+            ((reg & 0x190U) == 0x190U && num_pmc_counters > 4) ||
             ((reg & 0xF00U) == 0x300U) ||
             ((reg & 0xF00U) == 0xC00U) ||
             ((reg & 0xF00U) == 0xD00U) ||
@@ -124,8 +132,15 @@ static int allowed_intel(uint32_t reg)
             (reg == 0x19C)  ||
             (reg == 0x1A2)  ||
             (reg == 0x1AD)  ||
+            (reg == 0x1AE)  ||
+            (reg == 0x1AF)  ||
+            (reg == 0x1AC)  ||
             (reg == 0x1A6)  ||
-            (reg == 0x1A7))
+            (reg == 0x1A7)  ||
+            (reg == 0x620)  ||
+            (reg == 0xCD)   ||
+            (reg == 0x1B0)  ||
+            (reg == 0x1B1))
     {
         return 1;
     }
@@ -135,17 +150,21 @@ static int allowed_intel(uint32_t reg)
     }
 }
 
-static int allowed_sandybridge(uint32_t reg)
+static int
+allowed_sandybridge(uint32_t reg)
 {
     if ((allowed_intel(reg)) ||
-        (((reg & 0xF00U) == 0x600U)))
+        (((reg & 0xF00U) == 0x600U)) ||
+        (((reg & 0xF00U) == 0x700U)) ||
+        (reg == MSR_ALT_PEBS))
     {
         return 1;
     }
     return 0;
 }
 
-static int allowed_pci_sandybridge(PciDeviceType type, uint32_t reg)
+static int
+allowed_pci_sandybridge(PciDeviceType type, uint32_t reg)
 {
     switch (type)
     {
@@ -283,7 +302,8 @@ static int allowed_pci_sandybridge(PciDeviceType type, uint32_t reg)
     return 0;
 }
 
-static int allowed_haswell(uint32_t reg)
+static int
+allowed_haswell(uint32_t reg)
 {
     if ((allowed_intel(reg)) ||
         (allowed_sandybridge(reg)) ||
@@ -297,7 +317,8 @@ static int allowed_haswell(uint32_t reg)
     }
 }
 
-static int allowed_pci_haswell(PciDeviceType type, uint32_t reg)
+static int
+allowed_pci_haswell(PciDeviceType type, uint32_t reg)
 {
     switch (type)
     {
@@ -425,7 +446,8 @@ static int allowed_pci_haswell(PciDeviceType type, uint32_t reg)
     return 0;
 }
 
-static int allowed_silvermont(uint32_t reg)
+static int
+allowed_silvermont(uint32_t reg)
 {
 
     if ( ((reg & 0x0F8U) == 0x0C0U) ||
@@ -450,6 +472,139 @@ static int allowed_silvermont(uint32_t reg)
     }
 }
 
+static int allowed_knl(uint32_t reg)
+{
+    if (allowed_silvermont(reg))
+        return 1;
+    else
+    {
+        if (((reg & 0xF00U) == 0x700U) ||
+            ((reg & 0xF00U) == 0xE00U) ||
+            ((reg & 0xF00U) == 0xF00U))
+            return 1;
+    }
+    return 0;
+}
+
+static int allowed_pci_knl(PciDeviceType type, uint32_t reg)
+{
+    switch(type)
+    {
+	case EDC:
+	    if ((reg == PCI_MIC2_EDC_U_CTR0_A) ||
+		(reg == PCI_MIC2_EDC_U_CTR0_B) ||
+	        (reg == PCI_MIC2_EDC_U_CTR1_A) ||
+		(reg == PCI_MIC2_EDC_U_CTR1_B) ||
+	        (reg == PCI_MIC2_EDC_U_CTR2_A) ||
+		(reg == PCI_MIC2_EDC_U_CTR2_B) ||
+	        (reg == PCI_MIC2_EDC_U_CTR3_A) ||
+		(reg == PCI_MIC2_EDC_U_CTR3_B) ||
+		(reg == PCI_MIC2_EDC_U_CTRL0) ||
+		(reg == PCI_MIC2_EDC_U_CTRL1) ||
+		(reg == PCI_MIC2_EDC_U_CTRL2) ||
+		(reg == PCI_MIC2_EDC_U_CTRL3) ||
+		(reg == PCI_MIC2_EDC_U_BOX_CTRL) ||
+		(reg == PCI_MIC2_EDC_U_BOX_STATUS) ||
+		(reg == PCI_MIC2_EDC_U_FIXED_CTR_A) ||
+		(reg == PCI_MIC2_EDC_U_FIXED_CTR_B) ||
+		(reg == PCI_MIC2_EDC_U_FIXED_CTRL) ||
+	        (reg == PCI_MIC2_EDC_D_CTR0_A) ||
+		(reg == PCI_MIC2_EDC_D_CTR0_B) ||
+	        (reg == PCI_MIC2_EDC_D_CTR1_A) ||
+		(reg == PCI_MIC2_EDC_D_CTR1_B) ||
+	        (reg == PCI_MIC2_EDC_D_CTR2_A) ||
+		(reg == PCI_MIC2_EDC_D_CTR2_B) ||
+	        (reg == PCI_MIC2_EDC_D_CTR3_A) ||
+		(reg == PCI_MIC2_EDC_D_CTR3_B) ||
+		(reg == PCI_MIC2_EDC_D_CTRL0) ||
+		(reg == PCI_MIC2_EDC_D_CTRL1) ||
+		(reg == PCI_MIC2_EDC_D_CTRL2) ||
+		(reg == PCI_MIC2_EDC_D_CTRL3) ||
+		(reg == PCI_MIC2_EDC_D_BOX_CTRL) ||
+		(reg == PCI_MIC2_EDC_D_BOX_STATUS) ||
+		(reg == PCI_MIC2_EDC_D_FIXED_CTR_A) ||
+		(reg == PCI_MIC2_EDC_D_FIXED_CTR_B) ||
+		(reg == PCI_MIC2_EDC_D_FIXED_CTRL))
+	    {
+		return 1;
+	    }
+	    break;
+	case IMC:
+	    if ((reg == PCI_MIC2_MC_U_CTR0_A) ||
+		(reg == PCI_MIC2_MC_U_CTR0_B) ||
+	        (reg == PCI_MIC2_MC_U_CTR1_A) ||
+		(reg == PCI_MIC2_MC_U_CTR1_B) ||
+	        (reg == PCI_MIC2_MC_U_CTR2_A) ||
+		(reg == PCI_MIC2_MC_U_CTR2_B) ||
+	        (reg == PCI_MIC2_MC_U_CTR3_A) ||
+		(reg == PCI_MIC2_MC_U_CTR3_B) ||
+		(reg == PCI_MIC2_MC_U_CTRL0) ||
+		(reg == PCI_MIC2_MC_U_CTRL1) ||
+		(reg == PCI_MIC2_MC_U_CTRL2) ||
+		(reg == PCI_MIC2_MC_U_CTRL3) ||
+		(reg == PCI_MIC2_MC_U_BOX_CTRL) ||
+		(reg == PCI_MIC2_MC_U_BOX_STATUS) ||
+		(reg == PCI_MIC2_MC_U_FIXED_CTR_A) ||
+		(reg == PCI_MIC2_MC_U_FIXED_CTR_B) ||
+		(reg == PCI_MIC2_MC_U_FIXED_CTRL) ||
+	        (reg == PCI_MIC2_MC_D_CTR0_A) ||
+		(reg == PCI_MIC2_MC_D_CTR0_B) ||
+	        (reg == PCI_MIC2_MC_D_CTR1_A) ||
+		(reg == PCI_MIC2_MC_D_CTR1_B) ||
+	        (reg == PCI_MIC2_MC_D_CTR2_A) ||
+		(reg == PCI_MIC2_MC_D_CTR2_B) ||
+	        (reg == PCI_MIC2_MC_D_CTR3_A) ||
+		(reg == PCI_MIC2_MC_D_CTR3_B) ||
+		(reg == PCI_MIC2_MC_D_CTRL0) ||
+		(reg == PCI_MIC2_MC_D_CTRL1) ||
+		(reg == PCI_MIC2_MC_D_CTRL2) ||
+		(reg == PCI_MIC2_MC_D_CTRL3) ||
+		(reg == PCI_MIC2_MC_D_BOX_CTRL) ||
+		(reg == PCI_MIC2_MC_D_BOX_STATUS) ||
+		(reg == PCI_MIC2_MC_D_FIXED_CTR_A) ||
+		(reg == PCI_MIC2_MC_D_FIXED_CTR_B) ||
+		(reg == PCI_MIC2_MC_D_FIXED_CTRL))
+	    {
+		return 1;
+	    }
+	    break;
+	case R2PCIE:
+	    if ((reg == PCI_MIC2_M2PCIE_CTR0_A) ||
+		(reg == PCI_MIC2_M2PCIE_CTR0_B) ||
+		(reg == PCI_MIC2_M2PCIE_CTR1_A) ||
+                (reg == PCI_MIC2_M2PCIE_CTR1_B) ||
+		(reg == PCI_MIC2_M2PCIE_CTR2_A) ||
+                (reg == PCI_MIC2_M2PCIE_CTR2_B) ||
+		(reg == PCI_MIC2_M2PCIE_CTR3_A) ||
+                (reg == PCI_MIC2_M2PCIE_CTR3_B) ||
+		(reg == PCI_MIC2_M2PCIE_CTRL0) ||
+		(reg == PCI_MIC2_M2PCIE_CTRL1) ||
+		(reg == PCI_MIC2_M2PCIE_CTRL2) ||
+		(reg == PCI_MIC2_M2PCIE_CTRL3) ||
+		(reg == PCI_MIC2_M2PCIE_BOX_CTRL) ||
+		(reg == PCI_MIC2_M2PCIE_BOX_STATUS))
+	    {
+		return 1;
+	    }
+	    break;
+	case IRP:
+	    if ((reg == PCI_MIC2_IRP_CTR0) ||
+		(reg == PCI_MIC2_IRP_CTR1) ||
+		(reg == PCI_MIC2_IRP_CTRL0) ||
+		(reg == PCI_MIC2_IRP_CTRL1) ||
+		(reg == PCI_MIC2_IRP_BOX_CTRL) ||
+		(reg == PCI_MIC2_IRP_BOX_STATUS))
+	    {
+		return 1;
+	    }
+	    break;
+	default:
+	    break;
+
+    }
+    return 0;
+}
+
 static int allowed_amd(uint32_t reg)
 {
     if ( (reg & 0xFFFFFFF0U) == 0xC0010000U)
@@ -462,7 +617,8 @@ static int allowed_amd(uint32_t reg)
     }
 }
 
-static int allowed_amd15(uint32_t reg)
+static int
+allowed_amd15(uint32_t reg)
 {
     if ( ((reg & 0xFFFFFFF0U) == 0xC0010000U) ||
             ((reg & 0xFFFFFFF0U) == 0xC0010200U) ||
@@ -476,7 +632,8 @@ static int allowed_amd15(uint32_t reg)
     }
 }
 
-static int allowed_amd16(uint32_t reg)
+static int
+allowed_amd16(uint32_t reg)
 {
     if ( ((reg & 0xFFFFFFF0U) == 0xC0010000U) ||
             ((reg & 0xFFFFFFF8U) == 0xC0010240U))
@@ -489,7 +646,8 @@ static int allowed_amd16(uint32_t reg)
     }
 }
 
-static void msr_read(AccessDataRecord * dRecord)
+static void
+msr_read(AccessDataRecord * dRecord)
 {
     uint64_t data;
     uint32_t cpu = dRecord->cpu;
@@ -498,6 +656,13 @@ static void msr_read(AccessDataRecord * dRecord)
     dRecord->errorcode = ERR_NOERROR;
     dRecord->data = 0;
 
+    if (!lock_check())
+    {
+        syslog(LOG_ERR,"Access to performance counters is locked.\n");
+        dRecord->errorcode = ERR_LOCKED;
+        return;
+    }
+
     if (FD_MSR[cpu] <= 0)
     {
         dRecord->errorcode = ERR_NODEV;
@@ -506,28 +671,39 @@ static void msr_read(AccessDataRecord * dRecord)
 
     if (!allowed(reg))
     {
+        syslog(LOG_ERR, "Access to register 0x%X not allowed\n", reg);
         dRecord->errorcode = ERR_RESTREG;
         return;
     }
 
     if (pread(FD_MSR[cpu], &data, sizeof(data), reg) != sizeof(data))
     {
-        syslog(LOG_ERR, "Failed to read data to register 0x%x on core %u", reg, cpu);
+        syslog(LOG_ERR, "Failed to read data from register 0x%x on core %u", reg, cpu);
+#ifdef DEBUG_LIKWID
         syslog(LOG_ERR, "%s", strerror(errno));
+#endif
         dRecord->errorcode = ERR_RWFAIL;
         return;
     }
     dRecord->data = data;
 }
 
-static void msr_write(AccessDataRecord * dRecord)
+static void
+msr_write(AccessDataRecord * dRecord)
 {
     uint32_t cpu = dRecord->cpu;
     uint32_t reg = dRecord->reg;
     uint64_t data = dRecord->data;
 
     dRecord->errorcode = ERR_NOERROR;
-    
+
+    if (!lock_check())
+    {
+        syslog(LOG_ERR,"Access to performance counters is locked.\n");
+        dRecord->errorcode = ERR_LOCKED;
+        return;
+    }
+
     if (FD_MSR[cpu] <= 0)
     {
         dRecord->errorcode = ERR_NODEV;
@@ -544,13 +720,16 @@ static void msr_write(AccessDataRecord * dRecord)
     if (pwrite(FD_MSR[cpu], &data, sizeof(data), reg) != sizeof(data))
     {
         syslog(LOG_ERR, "Failed to write data to register 0x%x on core %u", reg, cpu);
+#ifdef DEBUG_LIKWID
         syslog(LOG_ERR, "%s", strerror(errno));
+#endif
         dRecord->errorcode = ERR_RWFAIL;
         return;
     }
 }
 
-static void msr_check(AccessDataRecord * dRecord)
+static void
+msr_check(AccessDataRecord * dRecord)
 {
     uint32_t cpu = dRecord->cpu;
     dRecord->errorcode = ERR_NOERROR;
@@ -563,7 +742,8 @@ static void msr_check(AccessDataRecord * dRecord)
     return;
 }
 
-static void pci_read(AccessDataRecord* dRecord)
+static void
+pci_read(AccessDataRecord* dRecord)
 {
     uint32_t socketId = dRecord->cpu;
     uint32_t reg = dRecord->reg;
@@ -573,6 +753,13 @@ static void pci_read(AccessDataRecord* dRecord)
     dRecord->errorcode = ERR_NOERROR;
     dRecord->data = 0;
 
+    if (!lock_check())
+    {
+        syslog(LOG_ERR,"Access to performance counters is locked.\n");
+        dRecord->errorcode = ERR_LOCKED;
+        return;
+    }
+
     if (FD_PCI[socketId][device] == -2)
     {
         dRecord->errorcode = ERR_NODEV;
@@ -599,8 +786,10 @@ static void pci_read(AccessDataRecord* dRecord)
             dRecord->errorcode = ERR_OPENFAIL;
             return;
         }
+#ifdef DEBUG_LIKWID
         syslog(LOG_ERR, "Open device file %s for device %s (%s) on socket %u", pci_filepath,
                     pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
+#endif
     }
 
     if (FD_PCI[socketId][device] > 0 && pread(FD_PCI[socketId][device], &data, sizeof(data), reg) != sizeof(data))
@@ -614,9 +803,8 @@ static void pci_read(AccessDataRecord* dRecord)
     dRecord->data = (uint64_t) data;
 }
 
-
-
-static void pci_write(AccessDataRecord* dRecord)
+static void
+pci_write(AccessDataRecord* dRecord)
 {
     uint32_t socketId = dRecord->cpu;
     uint32_t reg = dRecord->reg;
@@ -625,6 +813,13 @@ static void pci_write(AccessDataRecord* dRecord)
 
     dRecord->errorcode = ERR_NOERROR;
 
+    if (!lock_check())
+    {
+        syslog(LOG_ERR,"Access to performance counters is locked.\n");
+        dRecord->errorcode = ERR_LOCKED;
+        return;
+    }
+
     if (FD_PCI[socketId][device] == -2)
     {
         dRecord->errorcode = ERR_NODEV;
@@ -653,8 +848,10 @@ static void pci_write(AccessDataRecord* dRecord)
             dRecord->errorcode = ERR_OPENFAIL;
             return;
         }
+#ifdef DEBUG_LIKWID
         syslog(LOG_ERR, "Open device file %s for device %s (%s) on socket %u", pci_filepath,
                     pci_types[pci_devices_daemon[device].type].name, pci_devices_daemon[device].name, socketId);
+#endif
     }
 
     if (FD_PCI[socketId][device] > 0 && pwrite(FD_PCI[socketId][device], &data, sizeof data, reg) != sizeof data)
@@ -666,8 +863,8 @@ static void pci_write(AccessDataRecord* dRecord)
     }
 }
 
-
-static void pci_check(AccessDataRecord* dRecord)
+static void
+pci_check(AccessDataRecord* dRecord)
 {
     uint32_t socketId = dRecord->cpu;
     uint32_t device = dRecord->device;
@@ -681,7 +878,8 @@ static void pci_check(AccessDataRecord* dRecord)
     return;
 }
 
-static void kill_client(void)
+static void
+kill_client(void)
 {
     if (connfd != -1)
     {
@@ -691,7 +889,8 @@ static void kill_client(void)
     connfd = -1;
 }
 
-static void stop_daemon(void)
+static void
+stop_daemon(void)
 {
     kill_client();
     for (int i=0;i<MAX_NUM_NODES;i++)
@@ -712,7 +911,8 @@ static void stop_daemon(void)
     exit(EXIT_SUCCESS);
 }
 
-int getBusFromSocket(const uint32_t socket)
+static int
+getBusFromSocket(const uint32_t socket)
 {
     int cur_bus = 0;
     uint32_t cur_socket = 0;
@@ -747,7 +947,8 @@ int getBusFromSocket(const uint32_t socket)
     return -1;
 }
 
-static void Signal_Handler(int sig)
+static void
+Signal_Handler(int sig)
 {
     if (sig == SIGPIPE)
     {
@@ -762,7 +963,8 @@ static void Signal_Handler(int sig)
     }
 }
 
-static void daemonize(int* parentPid)
+static void
+daemonize(int* parentPid)
 {
     pid_t pid, sid;
 
@@ -839,6 +1041,7 @@ int main(void)
     }
 
     daemonize(&pid);
+    syslog(LOG_INFO, "AccessDaemon runs with UID %d, eUID %d\n", getuid(), geteuid());
 
     {
         uint32_t  eax = 0x00;
@@ -856,6 +1059,9 @@ int main(void)
         CPUID(eax, ebx, ecx, edx);
         uint32_t family = ((eax >> 8) & 0xFU) + ((eax >> 20) & 0xFFU);
         model  = (((eax >> 16) & 0xFU) << 4) + ((eax >> 4) & 0xFU);
+        eax = 0x0A;
+        CPUID(eax, ebx, ecx, edx);
+        num_pmc_counters = (int)((eax>>8)&0xFFU);
 
         switch (family)
         {
@@ -877,26 +1083,28 @@ int main(void)
                          (model == HASWELL_M2) ||
                          (model == BROADWELL) ||
                          (model == SKYLAKE1) ||
-                         (model == SKYLAKE2))
+                         (model == SKYLAKE2) ||
+                         (model == KABYLAKE1) ||
+                         (model == KABYLAKE2))
                 {
-                    allowed = allowed_haswell;
+                    allowed = allowed_sandybridge;
                 }
                 else if (model == BROADWELL_D)
                 {
-                    allowed = allowed_haswell;
+                    allowed = allowed_sandybridge;
                     isPCIUncore = 1;
                     allowedPci = allowed_pci_haswell;
                 }
                 else if (model == HASWELL_EP)
                 {
                     isPCIUncore = 1;
-                    allowed = allowed_haswell;
+                    allowed = allowed_sandybridge;
                     allowedPci = allowed_pci_haswell;
                 }
                 else if (model == BROADWELL_E)
                 {
                     isPCIUncore = 1;
-                    allowed = allowed_haswell;
+                    allowed = allowed_sandybridge;
                     allowedPci = allowed_pci_haswell;
                 }
                 else if ((model == ATOM_SILVERMONT_C) ||
@@ -908,6 +1116,12 @@ int main(void)
                 {
                     allowed = allowed_silvermont;
                 }
+                else if (model == XEON_PHI_KNL)
+                {
+                    allowed = allowed_knl;
+                    isPCIUncore = 1;
+                    allowedPci = allowed_pci_knl;
+                }
                 break;
             case K8_FAMILY:
             case K10_FAMILY:
@@ -928,10 +1142,10 @@ int main(void)
 
     /* setup filename for socket */
     filepath = (char*) calloc(sizeof(addr1.sun_path), 1);
-    snprintf(filepath, sizeof(addr1.sun_path), "/tmp/likwid-%d", pid);
+    snprintf(filepath, sizeof(addr1.sun_path), TOSTRING(LIKWIDSOCKETBASE) "-%d", pid);
 
     /* get a socket */
-    EXIT_IF_ERROR(sockfd = socket(AF_LOCAL, SOCK_STREAM, 0), socket failed);
+    LOG_AND_EXIT_IF_ERROR(sockfd = socket(AF_LOCAL, SOCK_STREAM, 0), socket failed);
 
     /* initialize socket data structure */
     bzero(&addr1, sizeof(addr1));
@@ -944,9 +1158,9 @@ int main(void)
     CHECK_ERROR(setfsuid(getuid()), setfsuid failed);
 
     /* bind and listen on socket */
-    EXIT_IF_ERROR(bind(sockfd, (SA*) &addr1, sizeof(addr1)), bind failed);
-    EXIT_IF_ERROR(listen(sockfd, 1), listen failed);
-    EXIT_IF_ERROR(chmod(filepath, S_IRUSR|S_IWUSR), chmod failed);
+    LOG_AND_EXIT_IF_ERROR(bind(sockfd, (SA*) &addr1, sizeof(addr1)), bind failed);
+    LOG_AND_EXIT_IF_ERROR(listen(sockfd, 1), listen failed);
+    LOG_AND_EXIT_IF_ERROR(chmod(filepath, S_IRUSR|S_IWUSR), chmod failed);
 
     socklen = sizeof(addr1);
 
@@ -1036,6 +1250,10 @@ int main(void)
                 //testDevice = 0x80862f30;
                 pci_devices_daemon = broadwellEP_pci_devices;
             }
+            else if (model == XEON_PHI_KNL)
+            {
+                pci_devices_daemon = knl_pci_devices;
+            }
             else
             {
                 //testDevice = 0;
@@ -1086,10 +1304,12 @@ int main(void)
                                 pci_devices_daemon[i].online = 1;
                                 close(fd);
                             }
+#ifdef DEBUG_LIKWID
                             else if (j==0)
                             {
                                 syslog(LOG_NOTICE, "Device %s for socket %d not found at path %s, excluded it from device list: %s\n",pci_devices_daemon[i].name,j, pci_filepath, strerror(errno));
                             }
+#endif
                         }
                     }
                 }
@@ -1162,9 +1382,10 @@ LOOP:
             dRecord.errorcode = ERR_UNKNOWN;
         }
 
-        EXIT_IF_ERROR(write(connfd, (void*) &dRecord, sizeof(AccessDataRecord)), write failed);
+        LOG_AND_EXIT_IF_ERROR(write(connfd, (void*) &dRecord, sizeof(AccessDataRecord)), write failed);
     }
 
     /* never reached */
     return EXIT_SUCCESS;
 }
+
