@@ -5,8 +5,8 @@
  *
  *      Description:  Interface to the access daemon for the access module.
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
@@ -27,6 +27,9 @@
  *
  * =======================================================================================
  */
+
+/* #####   HEADER FILE INCLUDES   ######################################### */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdint.h>
@@ -52,6 +55,7 @@
 #define gettid() syscall(SYS_gettid)
 
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
+
 static int globalSocket = -1;
 static pid_t masterPid = 0;
 static int cpuSockets_open = 0;
@@ -60,6 +64,7 @@ static pthread_mutex_t globalLock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t cpuLocks[MAX_NUM_THREADS] = { [0 ... MAX_NUM_THREADS-1] = PTHREAD_MUTEX_INITIALIZER };
 
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
+
 static char*
 access_client_strerror(AccessErrorType det)
 {
@@ -72,6 +77,7 @@ access_client_strerror(AccessErrorType det)
         case ERR_RWFAIL:     return "failed to read/write register";
         case ERR_DAEMONBUSY: return "daemon already has a same/higher priority client";
         case ERR_NODEV:      return "no such pci device";
+        case ERR_LOCKED:     return "access to registers is locked";
         default:             return "UNKNOWN errorcode";
     }
 }
@@ -122,7 +128,7 @@ access_client_startDaemon(int cpu_id)
         ERROR_PRINT(Failed to find the daemon '%s'\n, exeprog);
         exit(EXIT_FAILURE);
     }
-
+    DEBUG_PRINT(DEBUGLEV_INFO, Starting daemon %s, exeprog);
     pid = fork();
 
     if (pid == 0)
@@ -145,14 +151,15 @@ access_client_startDaemon(int cpu_id)
     }
     else if (pid < 0)
     {
-        ERROR_PLAIN_PRINT(Failed to fork);
+        ERROR_PRINT(Failed to fork access daemon for CPU %d, cpu_id);
+        return pid;
     }
 
     EXIT_IF_ERROR(socket_fd = socket(AF_LOCAL, SOCK_STREAM, 0), socket() failed);
 
     address.sun_family = AF_LOCAL;
     address_length = sizeof(address);
-    snprintf(address.sun_path, sizeof(address.sun_path), "/tmp/likwid-%d", pid);
+    snprintf(address.sun_path, sizeof(address.sun_path), TOSTRING(LIKWIDSOCKETBASE) "-%d", pid);
     filepath = strdup(address.sun_path);
 
     while (timeout > 0)
@@ -167,17 +174,16 @@ access_client_startDaemon(int cpu_id)
         }
 
         timeout--;
-        DEBUG_PRINT(DEBUGLEV_INFO, Still waiting for socket %s ..., filepath);
+        DEBUG_PRINT(DEBUGLEV_INFO, Still waiting for socket %s for CPU %d..., filepath, cpu_id);
     }
-    
+
     if (timeout <= 0)
     {
         ERRNO_PRINT;  /* should hopefully still work, as we make no syscalls in between. */
-        fprintf(stderr, "Exiting due to timeout: The socket file at '%s' \
-                could not be opened within 10 seconds.\n", filepath);
-        fprintf(stderr, "Consult the error message above this to find out why.\n");
-        fprintf(stderr, "If the error is 'no such file or directoy', \
-                it usually means that likwid-accessD just failed to start.\n");
+        fprintf(stderr, "Exiting due to timeout: The socket file at '%s' could not be\n", filepath);
+        fprintf(stderr, "opened within 10 seconds. Consult the error message above\n");
+        fprintf(stderr, "this to find out why. If the error is 'no such file or directoy',\n");
+        fprintf(stderr, "it usually means that likwid-accessD just failed to start.\n");
         exit(EXIT_FAILURE);
     }
     DEBUG_PRINT(DEBUGLEV_INFO, Successfully opened socket %s to daemon for CPU %d, filepath, cpu_id);
@@ -188,9 +194,9 @@ access_client_startDaemon(int cpu_id)
 
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
-int access_client_init(int cpu_id)
+int
+access_client_init(int cpu_id)
 {
-    int ret = 0;
     if (masterPid != 0 && gettid() == masterPid)
     {
         return 0;
@@ -199,6 +205,12 @@ int access_client_init(int cpu_id)
     {
         pthread_mutex_lock(&cpuLocks[cpu_id]);
         cpuSockets[cpu_id] = access_client_startDaemon(cpu_id);
+        if (cpuSockets[cpu_id] < 0)
+        {
+            //ERROR_PRINT(Start of access daemon failed for CPU %d, cpu_id);
+            pthread_mutex_unlock(&cpuLocks[cpu_id]);
+            return cpuSockets[cpu_id];
+        }
         cpuSockets_open++;
         pthread_mutex_unlock(&cpuLocks[cpu_id]);
         if (globalSocket == -1)
@@ -208,11 +220,13 @@ int access_client_init(int cpu_id)
             masterPid = gettid();
             pthread_mutex_unlock(&globalLock);
         }
+        return 0;
     }
-    return ret;
+    return -1;
 }
 
-int access_client_read(PciDeviceIndex dev, const int cpu_id, uint32_t reg, uint64_t *data)
+int
+access_client_read(PciDeviceIndex dev, const int cpu_id, uint32_t reg, uint64_t *data)
 {
     int ret;
     int socket = globalSocket;
@@ -220,6 +234,7 @@ int access_client_read(PciDeviceIndex dev, const int cpu_id, uint32_t reg, uint6
     AccessDataRecord record;
     record.cpu = cpu_id;
     record.device = MSR_DEV;
+    record.errorcode = ERR_OPENFAIL;
 
     if (cpuSockets_open == 0)
     {
@@ -281,7 +296,8 @@ int access_client_read(PciDeviceIndex dev, const int cpu_id, uint32_t reg, uint6
     return 0;
 }
 
-int access_client_write(PciDeviceIndex dev, const int cpu_id, uint32_t reg, uint64_t data)
+int
+access_client_write(PciDeviceIndex dev, const int cpu_id, uint32_t reg, uint64_t data)
 {
     int socket = globalSocket;
     int ret;
@@ -289,6 +305,7 @@ int access_client_write(PciDeviceIndex dev, const int cpu_id, uint32_t reg, uint
     record.cpu = cpu_id;
     record.device = MSR_DEV;
     pthread_mutex_t* lockptr = &globalLock;
+    record.errorcode = ERR_OPENFAIL;
 
     if (cpuSockets_open == 0)
     {
@@ -347,7 +364,8 @@ int access_client_write(PciDeviceIndex dev, const int cpu_id, uint32_t reg, uint
     return 0;
 }
 
-void access_client_finalize(int cpu_id)
+void
+access_client_finalize(int cpu_id)
 {
     AccessDataRecord record;
     if (cpuSockets[cpu_id] > 0)
@@ -362,9 +380,11 @@ void access_client_finalize(int cpu_id)
     {
         globalSocket = -1;
     }
+    masterPid = 0;
 }
 
-int access_client_check(PciDeviceIndex dev, int cpu_id)
+int
+access_client_check(PciDeviceIndex dev, int cpu_id)
 {
     int socket = globalSocket;
     pthread_mutex_t* lockptr = &globalLock;
@@ -373,6 +393,7 @@ int access_client_check(PciDeviceIndex dev, int cpu_id)
     record.cpu = cpu_id;
     record.device = dev;
     record.type = DAEMON_CHECK;
+    record.errorcode = ERR_OPENFAIL;
     if (dev != MSR_DEV)
     {
         record.cpu = affinity_core2node_lookup[cpu_id];
@@ -395,3 +416,4 @@ int access_client_check(PciDeviceIndex dev, int cpu_id)
     }
     return 0;
 }
+
