@@ -5,8 +5,8 @@
  *
  *      Description:  Implementation of affinity module.
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
@@ -55,12 +55,12 @@
 /* #####   EXPORTED VARIABLES   ########################################### */
 
 int affinity_core2node_lookup[MAX_NUM_THREADS];
+int affinity_thread2core_lookup[MAX_NUM_THREADS];
 
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
 
 #define gettid() syscall(SYS_gettid)
 
-
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
 static int  affinity_numberOfDomains = 0;
@@ -284,6 +284,7 @@ affinity_init()
                                       domains[currentDomain + subCounter].processorList,
                                       i, offset,
                                       domains[currentDomain + subCounter].numberOfProcessors);
+
             domains[currentDomain + subCounter].numberOfProcessors = tmp;
             offset += (tmp < numberOfCoresPerCache ? tmp : numberOfCoresPerCache);
             subCounter++;
@@ -301,12 +302,20 @@ affinity_init()
             {
                 domains[currentDomain + subCounter].numberOfProcessors =
                                 numa_info.nodes[subCounter].numberOfProcessors;
+
                 domains[currentDomain + subCounter].numberOfCores =
                                 numa_info.nodes[subCounter].numberOfProcessors/cpuid_topology.numThreadsPerCore;
+
                 domains[currentDomain + subCounter].tag = bformat("M%d", subCounter);
-                DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity domain M%d: %d HW threads on %d cores, subCounter, domains[currentDomain + subCounter].numberOfProcessors, domains[currentDomain + subCounter].numberOfCores);
+
+                DEBUG_PRINT(DEBUGLEV_DEVELOP,
+                        Affinity domain M%d: %d HW threads on %d cores,
+                        subCounter, domains[currentDomain + subCounter].numberOfProcessors,
+                        domains[currentDomain + subCounter].numberOfCores);
+
                 domains[currentDomain + subCounter].processorList =
                                 (int*) malloc(numa_info.nodes[subCounter].numberOfProcessors*sizeof(int));
+
                 if (!domains[currentDomain + subCounter].processorList)
                 {
                     fprintf(stderr,"No more memory for %ld bytes for processor list of affinity domain %s\n",
@@ -332,12 +341,18 @@ affinity_init()
         domains[currentDomain + subCounter].numberOfProcessors = NUMAthreads;
         domains[currentDomain + subCounter].numberOfCores =  NUMAthreads/cpuid_topology.numThreadsPerCore;
         domains[currentDomain + subCounter].tag = bformat("M%d", subCounter);
-        DEBUG_PRINT(DEBUGLEV_DEVELOP, Affinity domain M%d: %d HW threads on %d cores, subCounter, domains[currentDomain + subCounter].numberOfProcessors, domains[currentDomain + subCounter].numberOfCores);
+
+        DEBUG_PRINT(DEBUGLEV_DEVELOP,
+                Affinity domain M%d: %d HW threads on %d cores,
+                subCounter, domains[currentDomain + subCounter].numberOfProcessors,
+                domains[currentDomain + subCounter].numberOfCores);
+
         domains[currentDomain + subCounter].processorList = (int*) malloc(NUMAthreads*sizeof(int));
+
         if (!domains[currentDomain + subCounter].processorList)
         {
             fprintf(stderr,"No more memory for %ld bytes for processor list of affinity domain %s\n",
-                    NUMAthreads*sizeof(int), 
+                    NUMAthreads*sizeof(int),
                     bdata(domains[currentDomain + subCounter].tag));
             return;
         }
@@ -365,7 +380,6 @@ affinity_init()
     affinity_initialized = 1;
 }
 
-
 void
 affinity_finalize()
 {
@@ -402,7 +416,6 @@ affinity_finalize()
     affinity_initialized = 0;
 }
 
-
 int
 affinity_processGetProcessorId()
 {
@@ -419,7 +432,6 @@ affinity_processGetProcessorId()
     return getProcessorID(&cpu_set);
 }
 
-
 int
 affinity_threadGetProcessorId()
 {
@@ -449,7 +461,6 @@ affinity_pinThread(int processorId)
 }
 #endif
 
-
 void
 affinity_pinProcess(int processorId)
 {
@@ -461,7 +472,7 @@ affinity_pinProcess(int processorId)
 }
 
 void
-affinity_pinProcesses(int cpu_count, int* processorIds)
+affinity_pinProcesses(int cpu_count, const int* processorIds)
 {
     int i;
     cpu_set_t cpuset;
@@ -474,7 +485,6 @@ affinity_pinProcesses(int cpu_count, int* processorIds)
     sched_setaffinity(0, sizeof(cpu_set_t), &cpuset);
 }
 
-
 const AffinityDomain*
 affinity_getDomain(bstring domain)
 {
