@@ -6,8 +6,8 @@
  *
  *      Description:  A monitoring daemon for hardware performance counters.
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
@@ -268,7 +268,6 @@ local function logger(results)
             f:close()
         end
     end
-    
 end
 
 local function check_gmetric()
