@@ -8,8 +8,8 @@
  *                    that is used by likwid to avoid reading the systems architecture at
  *                    each start.
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
@@ -116,7 +116,6 @@ for field, value in pairs(cputopo) do
             for k,v in pairs(threadPool_order) do
                 file:write(str..tostring(v).." = "..tostring(tab[v]).."\n")
             end
-            
         end
     elseif (field == "cacheLevels") then
         for id, tab in pairs(cputopo["cacheLevels"]) do
@@ -124,7 +123,6 @@ for field, value in pairs(cputopo) do
             for k,v in pairs(cacheLevels_order) do
                 file:write(str..tostring(v).." = "..tostring(tab[v]).."\n")
             end
-            
         end
     end
 end
@@ -135,8 +133,8 @@ for field, value in pairs(numainfo["nodes"]) do
         if id ~= "processors" and id ~= "distances" then
             file:write("numa_info nodes "..tostring(field).." "..tostring(id).." = "..tostring(tab).."\n")
         elseif id == "processors" then
-            for k,v in pairs(tab) do 
-                str = str..","..tostring(v) 
+            for k,v in pairs(tab) do
+                str = str..","..tostring(v)
                 file:write("numa_info nodes "..tostring(field).." "..tostring(id).." "..tostring(k).." = "..tostring(v).."\n")
             end
         elseif id == "distances" then
