@@ -4,11 +4,11 @@
  *
  *      Filename:  likwid-mpirun.lua
  *
- *      Description: A wrapper script to pin threads spawned by MPI processes and 
+ *      Description: A wrapper script to pin threads spawned by MPI processes and
  *                   measure hardware performance counters
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
@@ -83,6 +83,10 @@ local function usage()
     print_stdout("")
     print_stdout("Processes are pinned to physical CPU cores first. For syntax questions see likwid-pin")
     print_stdout("")
+    print_stdout("For CPU selection and which MPI rank measures Uncore counters the system topology")
+    print_stdout("of the current system is used. There is currently no possibility to overcome this")
+    print_stdout("limitation by providing a topology file or similar.")
+    print_stdout("")
     examples()
 end
 
@@ -215,7 +219,7 @@ local function executeOpenMPI(wrapperscript, hostfile, env, nrNodes)
         local input = f:read("*a")
         ver1,ver2,ver3 = input:match("(%d+)%.(%d+)%.(%d+)")
         if ver1 == "1" then
-            if ver2 == "7" then
+            if tonumber(ver2) >= 7 then
                 bindstr = "--bind-to none"
             elseif ver2 == "6" then
                 bindstr = "--bind-to-none"
@@ -511,7 +515,6 @@ function write_hostlist_to_file(hostlist, nperhost)
             table.insert(outlist, item)
         else
             prefixzeros = 0
-            
             host, start, ende,remain = item:match("(%a+)%[(%d+)-(%d+)%]([%w%d%[%]-]*)")
             if host and start and ende then
                 if tonumber(start) ~= 0 then
@@ -548,7 +551,6 @@ end
 
 local function writeHostfileSlurm(hostlist, filename)
     l = {}
-    
     for i, h in pairs(hostlist) do
         table.insert(l, h["hostname"])
     end
@@ -563,7 +565,6 @@ local function executeSlurm(wrapperscript, hostfile, env, nrNodes)
     if wrapperscript.sub(1,1) ~= "/" then
         wrapperscript = os.getenv("PWD").."/"..wrapperscript
     end
-    
     local exec = string.format("srun -N %d --ntasks-per-node=%d --cpu_bind=none %s %s",
                                 nrNodes, ppn, table.concat(mpiopts, ' '), wrapperscript)
     if debug then
@@ -686,7 +687,6 @@ local function getMpiExec(mpitype)
         writeHostfile = writeHostfileSlurm
         getEnvironment = getEnvironmentSlurm
     end
-    
     for i, exec in pairs(testing) do
         f = io.popen(string.format("which %s 2>/dev/null", exec), 'r')
         if f ~= nil then
@@ -811,7 +811,6 @@ local function assignHosts(hosts, np, ppn)
                                             maxslots=host["maxslots"],
                                             interface=host["interface"]})
                         current = ppn
-                        
                     end
                 else
                     print_stderr(string.format("WARN: Oversubscription for host %s.", host["hostname"]))
@@ -995,7 +994,6 @@ local function setPerfStrings(perflist, cpuexprs)
                     table.insert(coreevents, e)
                 end
             end
-            
             local tmpSocketFlags = {}
             for _,e in pairs(socketListFlags) do
                 table.insert(tmpSocketFlags, e)
@@ -1196,7 +1194,6 @@ local function writeWrapperScript(scriptname, execStr, hosts, outputname)
     f:write("else\n")
     f:write("\techo \"Unknown local rank $LOCALRANK\"\n")
     f:write("fi\n")
-    
     f:close()
     os.execute("chmod +x "..scriptname)
 end
@@ -1311,7 +1308,6 @@ local function parseMarkerOutputFile(filename)
     local eventlist = {}
     local counterlist = {}
     local idx = 1
-    
     local results = {}
     local f = io.open(filename, "r")
     if f == nil then
@@ -1595,8 +1591,6 @@ function printMpiOutput(group_list, all_results, regionname)
     end
 end
 
-
-
 function cpuCount()
     cputopo = likwid.getCpuTopology()
     local cpus = cputopo["activeHWThreads"]
@@ -1688,20 +1682,34 @@ for i=1,#arg do
         table.insert(mpiopts, arg[i])
     end
 end
+
 if #executable == 0 then
     print_stderr("ERROR: No executable given on commandline")
     os.exit(1)
-elseif os.execute(string.format("ls %s 1>/dev/null 2>&1", executable[1])) == 0 then
-    print_stderr("ERROR: Cannot find executable given on commandline")
-    os.exit(1)
 else
-    local f = io.popen(string.format("which %s 2>/dev/null", executable[1]))
-    if f ~= nil then
-        executable[1] = f:read("*line")
-        f:close()
+    local do_which = false
+    local found = false
+    if likwid.access(executable[1], "x") == -1 then
+        do_which = true
+    else
+        found = true
     end
-    if debug then
-        print_stdout("DEBUG: Executable given on commandline: "..table.concat(executable, " "))
+    if not found then
+        if do_which then
+            local f = io.popen(string.format("which %s 2>/dev/null", executable[1]))
+            if f ~= nil then
+                executable[1] = f:read("*line")
+                f:close()
+                found = true
+            end
+            if debug then
+                print_stdout("DEBUG: Executable given on commandline: "..table.concat(executable, " "))
+            end
+        end
+    end
+    if not found then
+        print_stderr("ERROR: Cannot find executable given on commandline")
+        os.exit(1)
     end
 end
 if #mpiopts > 0 and debug then
@@ -1854,7 +1862,6 @@ elseif ppn == 0 and np > 0 then
             maxppn = host["slots"]
         end
     end
-    
     if ppn == 0 then
         ppn = 1
     end
