@@ -7,8 +7,8 @@
  *      Description:  An application to read out performance counter registers
  *                    on x86 processors
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
@@ -315,23 +315,6 @@ io.stdout:setvbuf("no")
 cpuinfo = likwid.getCpuInfo()
 cputopo = likwid.getCpuTopology()
 
-if not likwid.msr_available(access_flags) then
-    if access_mode == 1 then
-        print_stderr("MSR device files not available")
-        print_stderr("Please load msr kernel module before retrying")
-        if likwid.access(outfile..".tmp", "e") == 0 then
-            os.remove(outfile..".tmp")
-        end
-        os.exit(1)
-    else
-        print_stderr("MSR device files not readable and writeable")
-        print_stderr("Be sure that you have enough permissions to access the MSR files directly")
-        if likwid.access(outfile..".tmp", "e") == 0 then
-            os.remove(outfile..".tmp")
-        end
-        os.exit(1)
-    end
-end
 
 if num_cpus == 0 and
    not gotC and
@@ -342,7 +325,7 @@ if num_cpus == 0 and
    not print_info then
     print_stderr("Option -c <list> or -C <list> must be given on commandline")
     usage()
-    if likwid.access(outfile..".tmp", "e") == 0 then
+    if outfile and likwid.access(outfile..".tmp", "e") == 0 then
         os.remove(outfile..".tmp")
     end
     os.exit(1)
@@ -354,7 +337,7 @@ elseif num_cpus == 0 and
        not print_group_help and
        not print_info then
     print_stderr("CPUs given on commandline are not valid in current environment, maybe it's limited by a cpuset.")
-    if likwid.access(outfile..".tmp", "e") == 0 then
+    if outfile and likwid.access(outfile..".tmp", "e") == 0 then
         os.remove(outfile..".tmp")
     end
     os.exit(1)
@@ -643,6 +626,7 @@ if set_access_modes then
         os.exit(1)
     end
 end
+
 if likwid.init(num_cpus, cpulist) < 0 then
     likwid.putTopology()
     likwid.putConfiguration()
@@ -693,7 +677,7 @@ if verbose == true then
     print_stdout(string.format("Executing: %s",execString))
 end
 local ldpath = os.getenv("LD_LIBRARY_PATH")
-local libpath = likwid.pinlibpath:match("([/%g]+)/%g+.so")
+local libpath = string.match(likwid.pinlibpath, "([/%a%d]+)/[%a%s%d]*")
 if ldpath == nil then
     likwid.setenv("LD_LIBRARY_PATH", libpath)
 elseif not ldpath:match(libpath) then
@@ -734,7 +718,7 @@ if use_wrapper or use_timeline then
     local nr_events = likwid.getNumberOfEvents(activeGroup)
     local nr_threads = likwid.getNumberOfThreads()
     local firstrun = true
-    
+
     if use_wrapper and #group_ids == 1 then
         duration = 30.E06
     end
@@ -746,10 +730,14 @@ if use_wrapper or use_timeline then
     end
 
     local pid = nil
-    if pin_cpus then
-        pid = likwid.startProgram(execString, #cpulist, cpulist)
+    if execString:len() > 0 then
+        if pin_cpus then
+            pid = likwid.startProgram(execString, #cpulist, cpulist)
+        else
+            pid = likwid.startProgram(execString, 0, cpulist)
+        end
     else
-        pid = likwid.startProgram(execString, 0, cpulist)
+        pid = likwid.getpid()
     end
 
     if not pid then
@@ -758,52 +746,55 @@ if use_wrapper or use_timeline then
         likwid.putNumaInfo()
         likwid.putConfiguration()
         os.exit(1)
-    end
-    start = likwid.startClock()
-    groupTime[activeGroup] = 0
-    while true do
-        if likwid.getSignalState() ~= 0 then
-            likwid.killProgram()
-            break
-        end
-        local remain = likwid.sleep(duration)
-        exitvalue = likwid.checkProgram(pid)
-        if remain > 0 or exitvalue >= 0 then
-            io.stdout:flush()
-            break
-        end
-        if use_timeline == true then
-            stop = likwid.stopClock()
-            likwid.readCounters()
-            
-            local time = likwid.getClock(start, stop)
-            if likwid.getNumberOfMetrics(activeGroup) == 0 then
-                results = likwid.getLastResults()
-            else
-                results = likwid.getLastMetrics()
+    else
+        start = likwid.startClock()
+        groupTime[activeGroup] = 0
+        while true do
+            if likwid.getSignalState() ~= 0 then
+                if execString:len() > 0 then
+                    likwid.killProgram()
+                end
+                break
             end
-            str = tostring(math.tointeger(activeGroup)) .. " "..tostring(#results[activeGroup]).." "..tostring(#cpulist).." "..tostring(time)
-            for i,l1 in pairs(results[activeGroup]) do
-                for j, value in pairs(l1) do
-                    str = str .. " " .. tostring(value)
+            local remain = likwid.sleep(duration)
+            exitvalue = likwid.checkProgram(pid)
+            if remain > 0 or exitvalue >= 0 then
+                io.stdout:flush()
+                if execString:len() > 0 then
+                    break
                 end
             end
-            io.stderr:write(str.."\n")
-            groupTime[activeGroup] = time
-        else
-            likwid.readCounters()
-        end
-        if #group_ids > 1 then
-            likwid.switchGroup(activeGroup + 1)
-            activeGroup = likwid.getIdOfActiveGroup()
-            if groupTime[activeGroup] == nil then
-                groupTime[activeGroup] = 0
+            if use_timeline == true then
+                stop = likwid.stopClock()
+                likwid.readCounters()
+                local time = likwid.getClock(start, stop)
+                if likwid.getNumberOfMetrics(activeGroup) == 0 then
+                    results = likwid.getLastResults()
+                else
+                    results = likwid.getLastMetrics()
+                end
+                str = tostring(math.tointeger(activeGroup)) .. " "..tostring(#results[activeGroup]).." "..tostring(#cpulist).." "..tostring(time)
+                for i,l1 in pairs(results[activeGroup]) do
+                    for j, value in pairs(l1) do
+                        str = str .. " " .. tostring(value)
+                    end
+                end
+                io.stderr:write(str.."\n")
+                groupTime[activeGroup] = time
+            else
+                likwid.readCounters()
+            end
+            if #group_ids > 1 then
+                likwid.switchGroup(activeGroup + 1)
+                activeGroup = likwid.getIdOfActiveGroup()
+                if groupTime[activeGroup] == nil then
+                    groupTime[activeGroup] = 0
+                end
+                nr_events = likwid.getNumberOfEvents(activeGroup)
             end
-            nr_events = likwid.getNumberOfEvents(activeGroup)
         end
-        
+        stop = likwid.stopClock()
     end
-    stop = likwid.stopClock()
 elseif use_stethoscope then
     local ret = likwid.startCounters()
     if ret < 0 then
@@ -839,15 +830,21 @@ end
 
 
 if use_marker == true then
-    results, metrics = likwid.getMarkerResults(markerFile, cpulist)
-    if #results == 0 then
-        print_stderr("No regions could be found in Marker API result file")
-    else
-        for r=1, #results do
-            likwid.printOutput(results[r], metrics[r], cpulist, r, print_stats)
+    if likwid.access(markerFile, "e") >= 0 then
+        results, metrics = likwid.getMarkerResults(markerFile, cpulist)
+        if not results then
+            print_stderr("Failure reading Marker API result file.")
+        elseif #results == 0 then
+            print_stderr("No regions could be found in Marker API result file.")
+        else
+            for r=1, #results do
+                likwid.printOutput(results[r], metrics[r], cpulist, r, print_stats)
+            end
         end
+        os.remove(markerFile)
+    else
+        print_stderr("Marker API result file does not exist. This may happen if the application has not called LIKWID_MARKER_CLOSE.")
     end
-    os.remove(markerFile)
 elseif use_timeline == false then
     results = likwid.getResults()
     metrics = likwid.getMetrics()
@@ -856,24 +853,29 @@ end
 
 if outfile then
     local suffix = ""
-    if string.match(outfile,"%.") then
+    if string.match(outfile,".-[^\\/]-%.?([^%.\\/]*)$") then
         suffix = string.match(outfile, ".-[^\\/]-%.?([^%.\\/]*)$")
     end
     local command = "<INSTALLED_PREFIX>/share/likwid/filter/" .. suffix
     local tmpfile = outfile..".tmp"
     if suffix == "" then
         os.rename(tmpfile, outfile)
-    elseif suffix ~= "txt" and suffix ~= "csv" and likwid.access(command, "x") then
+    elseif suffix ~= "txt" and suffix ~= "csv" and not likwid.access(command, "x") then
         print_stderr("Cannot find filter script, save output in CSV format to file "..outfile)
         os.rename(tmpfile, outfile)
     else
         if suffix ~= "txt" and suffix ~= "csv" then
             command = command .." ".. tmpfile .. " perfctr"
-            local f = assert(io.popen(command))
+            local f = assert(io.popen(command), "r")
             if f ~= nil then
                 local o = f:read("*a")
                 if o:len() > 0 then
                     print_stderr(string.format("Failed to executed filter script %s.",command))
+                else
+                    os.rename(outfile.."."..suffix, outfile)
+                    if not likwid.access(tmpfile, "e") then
+                        os.remove(tmpfile)
+                    end
                 end
             else
                 print_stderr("Failed to call filter script, save output in CSV format to file "..outfile)
