@@ -7,8 +7,8 @@
  *      Description:  An application to use the timeline mode of likwid-perfctr to generate
  *                    realtime plots using feedGnuplot
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
@@ -541,7 +541,7 @@ while true do
                 end
             end
         end
-        
+
         group_list[group]["output"]:write(str.."\n")
         group_list[group]["output"]:flush()
         if dump then
