@@ -4,11 +4,11 @@
  *
  *      Filename:  likwid-powermeter.lua
  *
- *      Description:  An application to get information about power 
+ *      Description:  An application to get information about power
  *      consumption on architectures implementing the RAPL interface.
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
@@ -122,7 +122,6 @@ for opt,arg in likwid.getopt(arg, {"V:", "c:", "h", "i", "M:", "p", "s:", "v", "
             usage()
             os.exit(1)
         end
-        
     elseif opt == "i" or opt == "info" then
         print_info = true
     elseif (opt == "p") then
@@ -236,6 +235,16 @@ if (print_info) then
             print_stdout()
         end
     end
+    if power["minUncoreFreq"] > 0 and power["maxUncoreFreq"] > 0 then
+        print_stdout("Info about Uncore:")
+        print_stdout(string.format("Minimal Uncore frequency: %g MHz", power["minUncoreFreq"]))
+        print_stdout(string.format("Maximal Uncore frequency: %g MHz", power["maxUncoreFreq"]))
+        print_stdout()
+    end
+    if power["perfBias"] then
+        print_stdout(string.format("Performance energy bias: %.0f (0=highest performance, 15 = lowest energy)", power["perfBias"]))
+        print_stdout()
+    end
     print_stdout(likwid.hline)
 end
 
