@@ -7,8 +7,8 @@
  *      Description:  A application to determine the thread and cache topology
  *                    on x86 processors.
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
@@ -160,10 +160,10 @@ end
 table.insert(output_csv, likwid.hline)
 
 table.insert(output_csv, "STRUCT,Sockets,"..tostring(cputopo["numSockets"]))
-for socket=0,cputopo["numSockets"]-1 do
+for socket=0, #cputopo["topologyTree"] do
     csv_str = string.format("Socket %d:\t\t( ",cputopo["topologyTree"][socket]["ID"])
-    for core=0,cputopo["numCoresPerSocket"]-1 do
-        for thread=0, cputopo["numThreadsPerCore"]-1 do
+    for core = 0, #cputopo["topologyTree"][socket]["Childs"] do
+        for thread = 0, #cputopo["topologyTree"][socket]["Childs"][core]["Childs"] do
             csv_str = csv_str ..tostring(cputopo["topologyTree"][socket]["Childs"][core]["Childs"][thread]).. ","
         end
     end
@@ -188,7 +188,7 @@ for level=1,cputopo["numCacheLevels"] do
         else
             table.insert(output_csv, string.format("Size:\t\t\t%.0f MB",cputopo["cacheLevels"][level]["size"]/1048576))
         end
-        
+
         if (print_caches) then
             if (cputopo["cacheLevels"][level]["type"] == "DATACACHE") then
                 table.insert(output_csv, "Type:\t\t\tData cache")
@@ -199,7 +199,7 @@ for level=1,cputopo["numCacheLevels"] do
             table.insert(output_csv, string.format("Associativity:\t\t%d",cputopo["cacheLevels"][level]["associativity"]))
             table.insert(output_csv, string.format("Number of sets:\t\t%d",cputopo["cacheLevels"][level]["sets"]))
             table.insert(output_csv, string.format("Cache line size:\t%d",cputopo["cacheLevels"][level]["lineSize"]))
-            
+
             if (cputopo["cacheLevels"][level]["inclusive"] == 0) then
                 table.insert(output_csv, "Cache type:\t\tNon Inclusive")
             else
@@ -209,9 +209,9 @@ for level=1,cputopo["numCacheLevels"] do
         end
         local threads = cputopo["cacheLevels"][level]["threads"]
         str = "Cache groups:\t\t( "
-        for socket=0,cputopo["numSockets"]-1 do
-            for core=0,cputopo["numCoresPerSocket"]-1 do
-                for cpu=0,cputopo["numThreadsPerCore"]-1 do
+        for socket=0, #cputopo["topologyTree"] do
+            for core = 0, #cputopo["topologyTree"][socket]["Childs"] do
+                for cpu = 0, #cputopo["topologyTree"][socket]["Childs"][core]["Childs"] do
                     if (threads ~= 0) then
                         str = str .. cputopo["topologyTree"][socket]["Childs"][core]["Childs"][cpu] .. " "
                         threads = threads - 1
@@ -301,12 +301,12 @@ if print_graphical and not print_csv then
     print_stdout(likwid.sline)
     print_stdout("Graphical Topology")
     print_stdout(likwid.sline)
-    for socket=0,cputopo["numSockets"]-1 do
+    for socket=0, #cputopo["topologyTree"] do
         print_stdout(string.format("Socket %d:",cputopo["topologyTree"][socket]["ID"]))
         container = {}
-        for core=0,cputopo["numCoresPerSocket"]-1 do
+        for core = 0, #cputopo["topologyTree"][socket]["Childs"] do
             local tmpString = ""
-            for thread=0,cputopo["numThreadsPerCore"]-1 do
+            for thread = 0, #cputopo["topologyTree"][socket]["Childs"][core]["Childs"] do
                 if thread == 0 then
                     tmpString = tmpString .. tostring(cputopo["topologyTree"][socket]["Childs"][core]["Childs"][thread])
                 else
@@ -315,7 +315,7 @@ if print_graphical and not print_csv then
             end
             likwid.addSimpleAsciiBox(container, 1, core+1, tmpString)
         end
-        
+
         local columnCursor = 1
         local lineCursor = 2
         for cache=1,cputopo["numCacheLevels"] do
@@ -358,27 +358,32 @@ end
 
 if outfile then
     local suffix = ""
-    if string.match(outfile, "%.") then
+    if string.match(outfile, ".-[^\\/]-%.?([^%.\\/]*)$") then
         suffix = string.match(outfile, ".-[^\\/]-%.?([^%.\\/]*)$")
     end
     local command = "<INSTALLED_PREFIX>/share/likwid/filter/" .. suffix
     local tmpfile = outfile..".tmp"
     if suffix == "" then
         os.rename(tmpfile, outfile)
-    elseif suffix ~= "txt" and suffix ~= "csv" and likwid.access(command,"x") then
-        stdout_print("Cannot find filter script, save output in CSV format to file "..outfile)
+    elseif suffix ~= "txt" and suffix ~= "csv" and not likwid.access(command,"x") then
+        print_stderr("Cannot find filter script, save output in CSV format to file "..outfile)
         os.rename(tmpfile, outfile)
     else
         if suffix ~= "txt" and suffix ~= "csv" then
             command = command .." ".. tmpfile .. " topology"
-            local f = assert(io.popen(command))
+            local f = assert(io.popen(command), "r")
             if f ~= nil then
                 local o = f:read("*a")
                 if o:len() > 0 then
-                    stdout_print(string.format("Failed to executed filter script %s.",command))
+                    print_stderr(string.format("Failed to executed filter script %s.",command))
+                else
+                    os.rename(outfile.."."..suffix, outfile)
+                    if not likwid.access(tmpfile, "e") then
+                        os.remove(tmpfile)
+                    end
                 end
             else
-                stdout_print("Failed to call filter script, save output in CSV format to file "..outfile)
+                print_stderr("Failed to call filter script, save output in CSV format to file "..outfile)
                 os.rename(tmpfile, outfile)
                 os.remove(tmpfile)
             end
