@@ -5,8 +5,8 @@
  *
  *      Description:  Lua LIKWID interface library
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
@@ -33,6 +33,8 @@ package.cpath = '<INSTALLED_LIBPREFIX>/?.so;' .. package.cpath
 require("liblikwid")
 require("math")
 
+if not math.tointeger then math.tointeger = function(x) return math.floor(tonumber(x)) or nil end end
+
 likwid.groupfolder = "<LIKWIDGROUPPATH>"
 
 likwid.version = <VERSION>
@@ -142,6 +144,21 @@ likwid.markerRegionTime = likwid_markerRegionTime
 likwid.markerRegionCount = likwid_markerRegionCount
 likwid.markerRegionResult = likwid_markerRegionResult
 likwid.markerRegionMetric = likwid_markerRegionMetric
+likwid.getCpuClockCurrent = likwid_getCpuClockCurrent
+likwid.setCpuClockCurrent = likwid_setCpuClockCurrent
+likwid.getCpuClockMin = likwid_getCpuClockMin
+likwid.setCpuClockMin = likwid_setCpuClockMin
+likwid.getCpuClockMax = likwid_getCpuClockMax
+likwid.setCpuClockMax = likwid_setCpuClockMax
+likwid.getGovernor = likwid_getGovernor
+likwid.setGovernor = likwid_setGovernor
+likwid.getDriver = likwid_getDriver
+likwid.getuid = likwid_getuid
+likwid.geteuid = likwid_geteuid
+likwid.setuid = likwid_setuid
+likwid.seteuid = likwid_seteuid
+likwid.setresuid = likwid_setresuid
+likwid.setresuser = likwid_setresuser
 
 likwid.cpuFeatures = { [0]="HW_PREFETCHER", [1]="CL_PREFETCHER", [2]="DCU_PREFETCHER", [3]="IP_PREFETCHER",
                         [4]="FAST_STRINGS", [5]="THERMAL_CONTROL", [6]="PERF_MON", [7]="FERR_MULTIPLEX",
@@ -276,9 +293,12 @@ local function get_spaces(str, min_space, max_space)
     local length = str:len()
     local back = 0
     local front = 0
-    back = math.ceil((max_space-str:len()) /2)
+    if tonumber(str) == nil then
+        back = math.ceil((max_space-str:len()) /2)
+    else
+        back = 0
+    end
     front = max_space - back - str:len()
-
     if (front < back) then
         local tmp = front
         front = back
@@ -361,10 +381,9 @@ local function printtable(tab)
     end
     hline = hline .. "+"
     print(hline)
-    
     str = "| "
     for i=1,nr_columns do
-        front, back = get_spaces(tostring(tab[i][1]), min_lengths[i],max_lengths[i])
+        front, back = get_spaces(tostring(tab[i][1]), min_lengths[i], max_lengths[i])
         str = str .. front.. tostring(tab[i][1]) ..back
         if i<nr_columns then
             str = str .. " | "
@@ -374,7 +393,6 @@ local function printtable(tab)
     end
     print(str)
     print(hline)
-    
     for j=2,nr_lines do
         str = "| "
         for i=1,nr_columns do
@@ -417,7 +435,6 @@ local function printcsv(tab, linelength)
         end
         print(str)
     end
-    
 end
 
 likwid.printcsv = printcsv
@@ -498,8 +515,6 @@ local function new_groupdata(eventString, fix_ctrs)
         if not eventString:match("FIXC2") and fix_ctrs == 3 then
             eventString = eventString..",CPU_CLK_UNHALTED_REF:FIXC2"
         end
-        
-        
     end
     gdata["EventString"] = eventString
     gdata["GroupString"] = eventString
@@ -531,7 +546,6 @@ local function get_groupdata(group)
         if (a == group) then group_exist = 1 end
     end
     if (group_exist == 0) then return new_groupdata(group, cpuinfo["perf_num_fixed_ctr"]) end
-    
     local f = io.open(likwid.groupfolder .. "/" .. cpuinfo["short_name"] .. "/" .. group .. ".txt", "r")
     if f == nil then
         f = io.open(os.getenv("HOME") .. "/.likwid/groups/" .. cpuinfo["short_name"] .."/" .. group .. ".txt", "r")
@@ -556,7 +570,6 @@ local function get_groupdata(group)
     nr_events = 1
     nr_metrics = 1
     for i, line in pairs(stringsplit(t,"\n")) do
-        
         if (parse_eventset or parse_metrics or parse_long) and line:len() == 0 then
             parse_eventset = false
             parse_metrics = false
@@ -566,7 +579,7 @@ local function get_groupdata(group)
         if line:match("^SHORT%a*") ~= nil then
             linelist = stringsplit(line, "%s+", nil, "%s+")
             table.remove(linelist, 1)
-            groupdata["ShortDescription"] = table.concat(linelist, " ")  
+            groupdata["ShortDescription"] = table.concat(linelist, " ")
         end
 
         if line:match("^EVENTSET$") ~= nil then
@@ -595,7 +608,6 @@ local function get_groupdata(group)
             groupdata["Events"][nr_events]["Counter"] = linelist[1]:gsub("^%s*(.-)%s*$", "%1")
             nr_events = nr_events + 1
         end
-        
         if parse_metrics and line:match("^METRICS$") == nil then
             linelist = stringsplit(line:gsub("^%s*(.-)%s*$", "%1"), "%s+", nil, "%s+")
             formula = linelist[#linelist]
@@ -605,23 +617,17 @@ local function get_groupdata(group)
             groupdata["Metrics"][nr_metrics]["formula"] = formula
             nr_metrics = nr_metrics + 1
         end
-        
         if parse_long and line:match("^LONG$") == nil then
             groupdata["LongDescription"] = groupdata["LongDescription"] .. "\n" .. line
         end
     end
     groupdata["LongDescription"] = groupdata["LongDescription"]:sub(2)
     groupdata["EventString"] = groupdata["EventString"]:sub(2)
-    
     return groupdata
-    
 end
 
 likwid.get_groupdata = get_groupdata
 
-
-
-
 local function parse_time(timestr)
     local duration = 0
     local s1,e1 = timestr:find("ms")
@@ -795,7 +801,6 @@ local function printOutput(results, metrics, cpulist, region, stats)
                     table.insert(tmpList, string.format("%e", likwid.markerRegionTime(region, c)))
                 end
             end
-            
             for e, event in pairs(group) do
                 local tmp = tostring(likwid.num2str(event[c]))
                 table.insert(tmpList, tmp)
@@ -979,7 +984,12 @@ likwid.getLastMetrics = getLastMetrics
 
 local function getMarkerResults(filename, cpulist)
     local cpuinfo = likwid.getCpuInfo()
-    likwid.readMarkerFile(filename)
+    local ret = likwid.readMarkerFile(filename)
+    if ret < 0 then
+        return nil, nil
+    elseif ret == 0 then
+        return {}, {}
+    end
     results = {}
     metrics = {}
     for i=1, likwid.markerNumRegions() do
@@ -999,9 +1009,9 @@ local function getMarkerResults(filename, cpulist)
             end
         end
         if likwid.getNumberOfMetrics(groupID) > 0 then
-            for k=1, likwid.getNumberOfMetrics(likwid.markerRegionGroup(i)) do
+            for k=1, likwid.getNumberOfMetrics(groupID) do
                 local metricName = likwid.getNameOfMetric(groupID, k)
-                metrics[i][groupID][k] = {}
+                metrics[i][likwid.markerRegionGroup(i)][k] = {}
                 for j=1, regionThreads do
                     metrics[i][groupID][k][j] = likwid.markerRegionMetric(i,k,j)
                 end
@@ -1080,7 +1090,6 @@ local function printAsciiBox(container)
             end
             innerboxline = innerboxline .. "+ "
         end
-        
         boxlabelline = "| "
         for j=1,numColumns do
             local offset = 0
@@ -1140,4 +1149,36 @@ end
 
 likwid.getMPIrank = getMPIrank
 
+
+local function llikwid_getAvailFreq(cpu)
+    local freq_str = likwid_getAvailFreq(cpu)
+    local freqs = {}
+    if not freq_str then
+        return freqs, 0
+    end
+    for item in freq_str:gmatch("[%d%.]+") do
+        table.insert(freqs, item)
+    end
+    local turbo = freqs[1]
+    table.remove(freqs, 1)
+    return freqs, turbo
+end
+
+likwid.getAvailFreq = llikwid_getAvailFreq
+
+local function llikwid_getAvailGovs(cpu)
+    local gov_str = likwid_getAvailGovs(cpu)
+    local govs = {}
+    if not gov_str then
+        return govs
+    end
+    for item in gov_str:gmatch("%a+") do
+        table.insert(govs, item)
+    end
+    return govs
+end
+
+likwid.getAvailGovs = llikwid_getAvailGovs
+
+
 return likwid
