@@ -5,9 +5,13 @@
  *
  *      Description:  Infix calculator
  *
- *      Author:   Brandon Mills (bm), mills.brandont@gmail.com
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
- *      Copyright (C) 2016 Brandon Mills
+ *      Author:   Brandon Mills (bm), mills.brandont@gmail.com
+ *      Project:  likwid
+ *
+ *      Copyright (C) Brandon Mills
  *
  *      Permission is hereby granted, free of charge, to any person obtaining a copy of this
  *      software and associated documentation files (the "Software"), to deal in the
@@ -32,8 +36,8 @@
  *
  *      Some changes done for the integration in LIKWID, see inline comments
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@gmail.com
@@ -56,6 +60,8 @@
  * =======================================================================================
  */
 
+/* #####   HEADER FILE INCLUDES   ######################################### */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -63,16 +69,22 @@
 #include <getopt.h>
 #include <calculator_stack.h>
 
+/* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
+
 #define bool char
 #define true 1
 #define false 0
-
 #define PI 3.141592653589793
 
 /* Added by Thomas Roehl (Thomas.Roehl@fau.de) to reduce reallocs by allocating a temporary
  * token for parsing as well as for transforming a number to a string.
  */
 #define MAXTOKENLENGTH 512
+#define MAXPRECISION 20
+#define DEFAULTPRECISION 5
+#define AUTOPRECISION -1
+#define FUNCTIONSEPARATOR "|"
+/* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
 typedef enum
 {
@@ -103,24 +115,24 @@ struct Preferences
     {
         bool degrees;
     } mode;
+    int precision;
+    int maxtokenlength;
 } prefs;
 
 typedef enum
 {
     divZero,
     overflow,
-    parenMismatch
+    parenMismatch,
+    inputMissing,
 } Error;
 
 typedef char* token;
-/* Added by Thomas Roehl (Thomas.Roehl@fau.de) to keep track of the
- * intermediate calculation results to free them in the end
- */
-token* calcTokens = NULL;
-int nrCalcTokens = 0;
 
 typedef double number;
 
+/* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
+
 void raise(Error err)
 {
     char* msg;
@@ -135,11 +147,15 @@ void raise(Error err)
         case parenMismatch:
             msg = "Mismatched parentheses";
             break;
+        case inputMissing:
+            msg = "Function input missing";
+            break;
     }
     printf("\tError: %s\n", msg);
 }
 
-inline unsigned int toDigit(char ch)
+inline unsigned int
+toDigit(char ch)
 {
     return ch - '0';
 }
@@ -147,74 +163,222 @@ inline unsigned int toDigit(char ch)
 number buildNumber(token str)
 {
     number result = 0;
+    /*while(*str && *str != '.')
+    {
+        result = result * 10 + toDigit(*str++);
+    }*/
     result = strtod(str, NULL);
     return result;
 }
 
 token num2Str(number num)
 {
-    /* Increased precision by Thomas Roehl (Thomas.Roehl@fau.de) as required for LIKWID */
-    token str = (token)malloc((MAXTOKENLENGTH+1)*sizeof(char));
-    snprintf(str, 39, "%.20f", num);
+    int len = 0;
+    int precision = MAXPRECISION;
+    if (prefs.precision >= 0 && prefs.precision < precision)
+        precision = prefs.precision;
+    token str = (token)malloc(prefs.maxtokenlength*sizeof(char));
+    len = snprintf(str, prefs.maxtokenlength-1, "%.*f", precision, num);
+    if (prefs.precision == AUTOPRECISION)
+    {
+        while (str[len-1] == '0')
+        {
+            len = snprintf(str, prefs.maxtokenlength-1, "%.*f", --precision, num);
+        }
+    }
+
     return str;
 }
 
-
-
-inline number toRadians(number degrees)
+number
+toRadians(number degrees)
 {
     return degrees * PI / 180.0;
 }
 
-inline number toDegrees(number radians)
+number
+toDegrees(number radians)
 {
     return radians * 180.0 / PI;
 }
 
-token doFunc(token input, token function)
+token doFunc(Stack *s, token function)
 {
+    if (stackSize(s) == 0)
+    {
+        raise(inputMissing);
+        return "NaN";
+    }
+    else if (stackSize(s) == 1 && strcmp(stackTop(s), FUNCTIONSEPARATOR) == 0)
+    {
+        stackPop(s);
+        raise(inputMissing);
+        return "NaN";
+    }
+    token input = (token)stackPop(s);
     number num = buildNumber(input);
     number result = num;
+    number counter = 0;
 
-    if(strcmp(function, "abs") == 0)
+    if(strncmp(function, "abs", 3) == 0)
         result = fabs(num);
-    else if(strcmp(function, "floor") == 0)
+    else if(strncmp(function, "floor", 5) == 0)
         result = floor(num);
-    else if(strcmp(function, "ceil") == 0)
+    else if(strncmp(function, "ceil", 4) == 0)
         result = ceil(num);
-    else if(strcmp(function, "sin") == 0)
+    else if(strncmp(function, "sin", 3) == 0)
         result = !prefs.mode.degrees ? sin(num) : sin(toRadians(num));
-    else if(strcmp(function, "cos") == 0)
+    else if(strncmp(function, "cos", 3) == 0)
         result = !prefs.mode.degrees ? cos(num) : cos(toRadians(num));
-    else if(strcmp(function, "tan") == 0)
+    else if(strncmp(function, "tan", 3) == 0)
         result = !prefs.mode.degrees ? tan(num) : tan(toRadians(num));
-    else if(strcmp(function, "arcsin") == 0
-         || strcmp(function, "asin") == 0)
+    else if(strncmp(function, "arcsin", 6) == 0
+         || strncmp(function, "asin", 4) == 0)
         result = !prefs.mode.degrees ? asin(num) : toDegrees(asin(num));
-    else if(strcmp(function, "arccos") == 0
-         || strcmp(function, "acos") == 0)
+    else if(strncmp(function, "arccos", 6) == 0
+         || strncmp(function, "acos", 4) == 0)
         result = !prefs.mode.degrees ? acos(num) : toDegrees(acos(num));
-    else if(strcmp(function, "arctan") == 0
-         || strcmp(function, "atan") == 0)
+    else if(strncmp(function, "arctan", 6) == 0
+         || strncmp(function, "atan", 4) == 0)
         result = !prefs.mode.degrees ? atan(num) : toDegrees(atan(num));
-    else if(strcmp(function, "sqrt") == 0)
+    else if(strncmp(function, "sqrt", 4) == 0)
         result = sqrt(num);
-    else if(strcmp(function, "cbrt") == 0)
+    else if(strncmp(function, "cbrt", 4) == 0)
         result = cbrt(num);
-    else if(strcmp(function, "log") == 0)
+    else if(strncmp(function, "log", 3) == 0)
         result = log(num);
-    else if(strcmp(function, "exp") == 0)
+    else if(strncmp(function, "exp", 3) == 0)
         result = exp(num);
-    printf("Free %s\n", function);
-    free(function);
-    return num2Str(result);
+    else if(strncmp(function, "min", 3) == 0)
+    {
+        while (stackSize(s) > 0 && strcmp(stackTop(s), FUNCTIONSEPARATOR) != 0)
+        {
+            input = (token)stackPop(s);
+            num = buildNumber(input);
+            if (num < result)
+                result = num;
+        }
+    }
+    else if(strncmp(function, "max", 3) == 0)
+    {
+        while (stackSize(s) > 0 && strcmp(stackTop(s), FUNCTIONSEPARATOR) != 0)
+        {
+            input = (token)stackPop(s);
+            num = buildNumber(input);
+            if (num > result)
+                result = num;
+        }
+    }
+    else if(strncmp(function, "sum", 3) == 0)
+    {
+        while (stackSize(s) > 0  && strcmp(stackTop(s), FUNCTIONSEPARATOR) != 0)
+        {
+            input = (token)stackPop(s);
+            num = buildNumber(input);
+            result += num;
+        }
+    }
+    else if(strncmp(function, "avg", 3) == 0 ||
+            strncmp(function, "mean", 4) == 0)
+    {
+        // Result already initialized with first number
+        counter = 1;
+        while (stackSize(s) > 0  && strcmp(stackTop(s), FUNCTIONSEPARATOR) != 0)
+        {
+            input = (token)stackPop(s);
+            num = buildNumber(input);
+            result += num;
+            counter++;
+        }
+        result /= counter;
+    }
+    else if(strncmp(function, "median", 6) == 0)
+    {
+        // needed for sorting
+        Stack tmp, safe;
+        // Result already initialized with first number
+        counter = 1;
+        stackInit(&tmp, (stackSize(s) > 0 ? stackSize(s) : 1));
+        stackInit(&safe, (stackSize(s) > 0 ? stackSize(s) : 1));
+        // add first value to the later sorted stack
+        stackPush(&tmp, input);
+        while (stackSize(s) > 0  && strcmp(stackTop(s), FUNCTIONSEPARATOR) != 0)
+        {
+            input = (token)stackPop(s);
+            num = buildNumber(input);
+            // save all numbers larger as the stack value
+            while (stackSize(&tmp) > 0 && buildNumber(stackTop(&tmp)) < num)
+            {
+                stackPush(&safe, stackPop(&tmp));
+            }
+            // push value on the sorted stack
+            stackPush(&tmp, input);
+            // push all saved numbers back on the sorted stack
+            while (stackSize(&safe) > 0)
+            {
+                stackPush(&tmp, stackPop(&safe));
+            }
+            counter++;
+        }
+        stackFree(&safe);
+        // calculate the median index
+        counter = (number)(((int)counter+1)/2);
+        // pop all numbers until median index
+        while (counter > 1)
+        {
+            stackPop(&tmp);
+            counter--;
+        }
+        result = buildNumber(stackPop(&tmp));
+        // pop the remaining sorted stack
+        while (stackSize(&tmp) > 0)
+        {
+            stackPop(&tmp);
+        }
+        stackFree(&tmp);
+    }
+    else if(strncmp(function, "var", 3) == 0)
+    {
+        Stack tmp;
+        counter = 1;
+        // second stack to store values during calculation of mean
+        stackInit(&tmp, (stackSize(s) > 0 ? stackSize(s) : 1));
+        // push first value to temporary stack
+        stackPush(&tmp, input);
+        number mean = result;
+        while (stackSize(s) > 0  && strcmp(stackTop(s), FUNCTIONSEPARATOR) != 0)
+        {
+            input = (token)stackPop(s);
+            // push value to temporary stack
+            stackPush(&tmp, input);
+            num = buildNumber(input);
+            mean += num;
+            counter++;
+        }
+        // calculate mean
+        mean /= counter;
+        result = 0;
+        // calculate sum of squared differences
+        while (stackSize(&tmp) > 0)
+        {
+            input = (token)stackPop(&tmp);
+            num = buildNumber(input)-mean;
+            result += pow(num,2);
+        }
+        // determine variance
+        result /= counter;
+        stackFree(&tmp);
+    }
+    if (strcmp(stackTop(s), FUNCTIONSEPARATOR) == 0)
+        stackPop(s);
+    stackPush(s, num2Str(result));
+    return 0;
 }
 
-int doOp(token loperand, token op, token roperand, token *result)
+int doOp(Stack *s, token op)
 {
-    /* Added by Thomas Roehl (Thomas.Roehl@fau.de) to return
-     * errors from calculation like devide-by-zero, ... */
-    int err = 0;
+    token roperand = (token)stackPop(s);
+    token loperand = (token)stackPop(s);
     number lside = buildNumber(loperand);
     number rside = buildNumber(roperand);
     number ret;
@@ -234,9 +398,8 @@ int doOp(token loperand, token op, token roperand, token *result)
             {
                 if(rside == 0)
                 {
-                    /* Changed by Thomas Roehl */
-                    //raise(divZero);
-                    err = -1;
+                    raise(divZero);
+                    return -1;
                 }
                 else
                     ret = lside / rside;
@@ -246,9 +409,8 @@ int doOp(token loperand, token op, token roperand, token *result)
             {
                 if(rside == 0)
                 {
-                    /* Changed by Thomas Roehl */
-                    //raise(divZero);
-                    err = -1;
+                    raise(divZero);
+                    return -1;
                 }
                 else
                 {
@@ -268,11 +430,10 @@ int doOp(token loperand, token op, token roperand, token *result)
             }
             break;
     }
-    *result = num2Str(ret);
-    return err;
+    stackPush(s, num2Str(ret));
+    return 0;
 }
 
-
 Symbol type(char ch)
 {
     Symbol result;
@@ -380,26 +541,35 @@ Symbol type(char ch)
 
 bool isFunction(token tk)
 {
-    return (strcmp(tk, "abs") == 0
-        || strcmp(tk, "floor") == 0
-        || strcmp(tk, "ceil") == 0
-        || strcmp(tk, "sin") == 0
-        || strcmp(tk, "cos") == 0
-        || strcmp(tk, "tan") == 0
-        || strcmp(tk, "arcsin") == 0
-        || strcmp(tk, "arccos") == 0
-        || strcmp(tk, "arctan") == 0
-        || strcmp(tk, "asin") == 0
-        || strcmp(tk, "acos") == 0
-        || strcmp(tk, "atan") == 0
-        || strcmp(tk, "sqrt") == 0
-        || strcmp(tk, "cbrt") == 0
-        || strcmp(tk, "log") == 0
-        || strcmp(tk, "exp") == 0);
+    return (strncmp(tk, "abs", 3) == 0
+        || strncmp(tk, "floor", 5) == 0
+        || strncmp(tk, "ceil", 4) == 0
+        || strncmp(tk, "sin", 3) == 0
+        || strncmp(tk, "cos", 3) == 0
+        || strncmp(tk, "tan", 3) == 0
+        || strncmp(tk, "arcsin", 6) == 0
+        || strncmp(tk, "arccos", 6) == 0
+        || strncmp(tk, "arctan", 6) == 0
+        || strncmp(tk, "asin", 4) == 0
+        || strncmp(tk, "acos", 4) == 0
+        || strncmp(tk, "atan", 4) == 0
+        || strncmp(tk, "sqrt", 4) == 0
+        || strncmp(tk, "cbrt", 4) == 0
+        || strncmp(tk, "log", 3) == 0
+        || strncmp(tk, "min", 3) == 0
+        || strncmp(tk, "max", 3) == 0
+        || strncmp(tk, "sum", 3) == 0
+        || strncmp(tk, "avg", 3) == 0
+        || strncmp(tk, "mean", 4) == 0
+        || strncmp(tk, "median", 6) == 0
+        || strncmp(tk, "var", 3) == 0
+        || strncmp(tk, "exp", 3) == 0);
 }
 
 Symbol tokenType(token tk)
 {
+    if (!tk)
+        return invalid;
     Symbol ret = type(*tk);
     switch(ret)
     {
@@ -417,37 +587,33 @@ Symbol tokenType(token tk)
         case digit:
             ret = value;
             break;
+        default:
+            break;
     }
     return ret;
 }
 
 int tokenize(char *str, char *(**tokensRef))
 {
+    int i = 0;
     char** tokens = NULL;
     char** tmp = NULL;
     char* ptr = str;
     char ch = '\0';
     int numTokens = 0;
-    /* Added by Thomas Roehl (Thomas.Roehl@fau.de) to parse string
-     * in a temporary token to reduce frequent reallocs. newToken
-     * is replaced by tmpToken during parsing. Removed all reallocs
-     * and not required mallocs from the original code.
-     */
-    char* tmpToken = malloc((MAXTOKENLENGTH+1) * sizeof(char));
+    char* tmpToken = malloc((prefs.maxtokenlength+1) * sizeof(char));
     if (!tmpToken)
     {
         fprintf(stderr, "Malloc of temporary buffer failed\n");
         return 0;
     }
-    while(ch = *ptr++)
+    while((ch = *ptr++))
     {
         if(type(ch) == invalid) // Stop tokenizing when we encounter an invalid character
             break;
 
         token newToken = NULL;
-        /* Added by Thomas Roehl (Thomas.Roehl@fau.de)
-         * Prepare temporary token for next parsing step */
-        memset(tmpToken, '\0', MAXTOKENLENGTH+1);
+        tmpToken[0] = '\0';
         switch(type(ch))
         {
             case addop:
@@ -458,7 +624,8 @@ int tokenize(char *str, char *(**tokensRef))
                             || (tokenType(tokens[numTokens-1]) == addop
                                 || tokenType(tokens[numTokens-1]) == multop
                                 || tokenType(tokens[numTokens-1]) == expop
-                                || tokenType(tokens[numTokens-1]) == lparen)))
+                                || tokenType(tokens[numTokens-1]) == lparen
+                                || tokenType(tokens[numTokens-1]) == argsep)))
                     {
                         // Assemble an n-character (plus null-terminator) number token
                         {
@@ -471,36 +638,30 @@ int tokenize(char *str, char *(**tokensRef))
                                 //printf("Decimal\n");
                                 hasDecimal = true;
                                 len++;
-                                //newToken = (char*)malloc((len + 1) * sizeof(char));
                                 tmpToken[0] = '0';
                                 tmpToken[1] = '.';
                             }
                             else // Numbers that do not start with decimal
                             {
-                                //newToken = (char*)malloc((len + 1) * sizeof(char)); // Leave room for '\0'
                                 tmpToken[len-1] = ch;
                             }
 
                             // Assemble rest of number
                             for(; // Don't change len
                                 *ptr // There is a next character and it is not null
-                                && len <= MAXTOKENLENGTH
+                                && len <= prefs.maxtokenlength
                                 && (type(*ptr) == digit // The next character is a digit
                                      || ((type(*ptr) == decimal // Or the next character is a decimal
                                          && hasDecimal == 0)) // But we have not added a decimal
                                      || ((*ptr == 'E' || *ptr == 'e') // Or the next character is an exponent
                                          && hasExponent == false) // But we have not added an exponent yet
-                                     /* Added by Thomas Roehl (Thomas.Roehl@fau.de) to parse scientific notation
-                                      * with signed exponent correctly
-                                      */
-                                     || ((*ptr == '+' || *ptr == '-') && hasExponent == true)); // Exponent with sign
+                                || ((*ptr == '+' || *ptr == '-') && hasExponent == true)); // Exponent with sign
                                 ++len)
                             {
                                 if(type(*ptr) == decimal)
                                     hasDecimal = true;
                                 else if(*ptr == 'E' || *ptr == 'e')
                                     hasExponent = true;
-                                //newToken = (char*)realloc(newToken, (len + 1) * sizeof(char)); // Leave room for '\0'
                                 tmpToken[len] = *ptr++;
                             }
 
@@ -518,7 +679,6 @@ int tokenize(char *str, char *(**tokensRef))
             case argsep:
                 // Assemble a single-character (plus null-terminator) operation token
                 {
-                    //newToken = (char*)malloc(2 * sizeof(char)); // Leave room for '\0'
                     tmpToken[0] = ch;
                     tmpToken[1] = '\0';
                 }
@@ -536,42 +696,30 @@ int tokenize(char *str, char *(**tokensRef))
                         //printf("Decimal\n");
                         hasDecimal = true;
                         len++;
-                        //newToken = (char*)malloc((len + 1) * sizeof(char));
                         tmpToken[0] = '0';
                         tmpToken[1] = '.';
                     }
                     else // Numbers that do not start with decimal
                     {
-                        //newToken = (char*)malloc((len + 1) * sizeof(char)); // Leave room for '\0'
                         tmpToken[len-1] = ch;
                     }
 
                     // Assemble rest of number
-                    /* Added support for signed exponents in scientific notation
-                     * by Thomas Roehl (Thomas.Roehl@fau.de) as required for LIKWID */
                     for(; // Don't change len
                         *ptr // There is a next character and it is not null
-                        && len <= MAXTOKENLENGTH
+                        && len <= prefs.maxtokenlength
                         && (type(*ptr) == digit // The next character is a digit
                              || ((type(*ptr) == decimal // Or the next character is a decimal
-                                 && hasDecimal == false)) // But we have not added a decimal
+                                 && hasDecimal == 0)) // But we have not added a decimal
                              || ((*ptr == 'E' || *ptr == 'e') // Or the next character is an exponent
                                  && hasExponent == false) // But we have not added an exponent yet
-                             /* Added by Thomas Roehl (Thomas.Roehl@fau.de) to parse scientific notation
-                              * with signed exponent correctly
-                              */
                              || ((*ptr == '+' || *ptr == '-') && hasExponent == true)); // Exponent with sign
                         ++len)
                     {
                         if(type(*ptr) == decimal)
-                        {
                             hasDecimal = true;
-                        }
                         else if(*ptr == 'E' || *ptr == 'e')
-                        {
                             hasExponent = true;
-                        }
-                        //newToken = (char*)realloc(newToken, (len + 1) * sizeof(char)); // Leave room for '\0'
                         tmpToken[len] = *ptr++;
                     }
 
@@ -583,34 +731,48 @@ int tokenize(char *str, char *(**tokensRef))
                 // Assemble an n-character (plus null-terminator) text token
                 {
                     int len = 1;
-                    //newToken = (char*)malloc((len + 1) * sizeof(char)); // Leave room for '\0'
                     tmpToken[0] = ch;
-                    for(len = 1; *ptr && type(*ptr) == text && len <= MAXTOKENLENGTH; ++len)
+                    for(len = 1; *ptr && type(*ptr) == text && len <= prefs.maxtokenlength; ++len)
                     {
-                        //newToken = (char*)realloc(newToken, (len + 1) * sizeof(char)); // Leave room for '\0'
                         tmpToken[len] = *ptr++;
                     }
                     tmpToken[len] = '\0';
                 }
                 break;
+            default:
+                break;
         }
         // Add to list of tokens
-        if(tmpToken[0] != '\0')
+        if(tmpToken[0] != '\0' && strlen(tmpToken) > 0)
         {
             numTokens++;
             /*if(tokens == NULL) // First allocation
                 tokens = (char**)malloc(numTokens * sizeof(char*));
             else*/
-            /* Added by Thomas Roehl (Thomas.Roehl@fau.de)
-             * Allocate new output token and copy temporary token
-             */
+            
             newToken = malloc((strlen(tmpToken)+1) * sizeof(char));
+            if (!newToken)
+            {
+                numTokens--;
+                break;
+            }
             strcpy(newToken, tmpToken);
             newToken[strlen(tmpToken)] = '\0';
             tmp = (char**)realloc(tokens, numTokens * sizeof(char*));
             if (tmp == NULL)
             {
+                free(newToken);
+                if (tokens != NULL)
+                {
+                    for(i=0;i<numTokens-1;i++)
+                    {
+                        if (tokens[i] != NULL)
+                            free(tokens[i]);
+                    }
+                    free(tokens);
+                }
                 *tokensRef = NULL;
+                free(newToken);
                 free(tmpToken);
                 return 0;
             }
@@ -620,32 +782,38 @@ int tokenize(char *str, char *(**tokensRef))
         }
     }
     *tokensRef = tokens; // Send back out
-    /* Added by Thomas Roehl (Thomas.Roehl@fau.de) */
     free(tmpToken);
+    tmpToken = NULL;
     return numTokens;
 }
 
 bool leftAssoc(token op)
 {
-    bool ret;
+    bool ret = false;
     switch(tokenType(op))
     {
         case addop:
         case multop:
+        
             ret = true;
             break;
+        case function:
         case expop:
             ret = false;
             break;
+        default:
+            break;
     }
     return ret;
 }
 
 int precedence(token op1, token op2)
 {
-    int ret;
+    int ret = 0;
 
-    if(tokenType(op1) == tokenType(op2)) // Equal precedence
+    if (op2 == NULL)
+        ret = 1;
+    else if(tokenType(op1) == tokenType(op2)) // Equal precedence
         ret = 0;
     else if(tokenType(op1) == addop
             && (tokenType(op2) == multop || tokenType(op2) == expop)) // op1 has lower precedence
@@ -659,16 +827,17 @@ int precedence(token op1, token op2)
     else if(tokenType(op1) == expop
             && tokenType(op2) == multop) // op1 has higher precedence
         ret = 1;
-
+    else if (tokenType(op1) == function 
+            && (tokenType(op2) == addop || tokenType(op2) == multop || tokenType(op2) == expop || tokenType(op2) == lparen))
+        ret = 1;
+    else if ((tokenType(op1) == addop || tokenType(op1) == multop || tokenType(op1) == expop)
+            && tokenType(op2) == function)
+        ret = -1;
     return ret;
 }
 
-int evalStackPush(Stack *s, token val)
+void evalStackPush(Stack *s, token val)
 {
-    /* Added by Thomas Roehl (Thomas.Roehl@fau.de) to return
-     * calculation errors. Function now returns an int.
-     */
-    int ret = 0;
     if(prefs.display.postfix)
         printf("\t%s\n", val);
 
@@ -676,11 +845,11 @@ int evalStackPush(Stack *s, token val)
     {
         case function:
             {
-                token operand, res;
-                operand = (token)stackPop(s);
-                res = doFunc(operand, val);
-                //free(operand);
-                stackPush(s, res);
+                //token res;
+                //operand = (token)stackPop(s);
+                if (doFunc(s, val) < 0)
+                    return;
+                //stackPush(s, res);
             }
             break;
         case expop:
@@ -690,20 +859,13 @@ int evalStackPush(Stack *s, token val)
                 if(stackSize(s) >= 2)
                 {
                     // Pop two operands
-                    token l, r, res;
-                    r = (token)stackPop(s);
-                    l = (token)stackPop(s);
 
                     // Evaluate
-                    /* Added return value by Thomas Roehl (Thomas.Roehl@fau.de) */
-                    ret = doOp(l, val, r, &res);
+                    if (doOp(s, val) < 0)
+                        return;
+
                     // Push result
-                    stackPush(s, res);
-                    /* Added by Thomas Roehl (Thomas.Roehl@fau.de)
-                     * Keeping track of the intermediate results
-                     */
-                    calcTokens[nrCalcTokens] = res;
-                    nrCalcTokens++;
+                    //stackPush(s, res);
                 }
                 else
                 {
@@ -716,17 +878,18 @@ int evalStackPush(Stack *s, token val)
                 stackPush(s, val);
             }
             break;
+        default:
+            break;
     }
-    /* Return value by Thomas Roehl (Thomas.Roehl@fau.de) */
-    return ret;
 }
 
-int postfix(token *tokens, int numTokens, Stack *output)
+bool postfix(token *tokens, int numTokens, Stack *output)
 {
-    Stack operators;
+    Stack operators, intermediate;
     int i;
-    int err = 0;
-    stackInit(&operators, 2*numTokens);
+    bool err = false;
+    stackInit(&operators, numTokens);
+    stackInit(&intermediate, numTokens);
     for(i = 0; i < numTokens; i++)
     {
         // From Wikipedia/Shunting-yard_algorithm:
@@ -735,13 +898,23 @@ int postfix(token *tokens, int numTokens, Stack *output)
             case value:
                 {
                     // If the token is a number, then add it to the output queue.
-                    //printf("Adding number to output stack\n");
-                    err = evalStackPush(output, tokens[i]);
+                    //printf("Adding number %s to output stack\n", tokens[i]);
+                    evalStackPush(output, tokens[i]);
                 }
                 break;
             case function:
                 {
+                    while(stackSize(&operators) > 0
+                        && (tokenType(tokens[i]) != lparen)
+                        && ((precedence(tokens[i], (char*)stackTop(&operators)) <= 0)))
+                    {
+                        //printf("Moving operator %s from operator stack to output stack\n", (char*)stackTop(&operators));
+                        evalStackPush(output, stackPop(&operators));
+                        stackPush(&intermediate, stackTop(output));
+                    }
+
                     // If the token is a function token, then push it onto the stack.
+                    //printf("Adding operator %s to operator stack\n", tokens[i]);
                     stackPush(&operators, tokens[i]);
                 }
                 break;
@@ -756,24 +929,12 @@ int postfix(token *tokens, int numTokens, Stack *output)
                      */
                     while(stackSize(&operators) > 0
                         && tokenType((token)stackTop(&operators)) != lparen
-                        && stackSize(&operators) > 1
-                        && err == 0)
+                        && stackSize(&operators) > 1)
                     {
                         //printf("Moving operator from operator stack to output stack\n");
-                        token t = (token)stackPop(&operators);
-                        err = evalStackPush(output, t);
-                        //free(t);
+                        evalStackPush(output, stackPop(&operators));
+                        stackPush(&intermediate, stackTop(output));
                     }
-                    if(stackSize(&operators) > 0
-                        && tokenType((token)stackTop(&operators)) != lparen)
-                    {
-                        err = -1;
-                        /* Changed by Thomas Roehl */
-                        //raise(parenMismatch);
-                    }
-                    //printf("Removing left paren from operator stack\n");
-                    token t = stackPop(&operators); // Discard lparen
-                    //free(t);
                 }
                 break;
             case addop:
@@ -791,15 +952,13 @@ int postfix(token *tokens, int numTokens, Stack *output)
                     while(stackSize(&operators) > 0
                         && (tokenType((char*)stackTop(&operators)) == addop || tokenType((char*)stackTop(&operators)) == multop || tokenType((char*)stackTop(&operators)) == expop)
                         && ((leftAssoc(tokens[i]) && precedence(tokens[i], (char*)stackTop(&operators)) <= 0)
-                            || (!leftAssoc(tokens[i]) && precedence(tokens[i], (char*)stackTop(&operators)) < 0))
-                        && err == 0)
+                            || (!leftAssoc(tokens[i]) && precedence(tokens[i], (char*)stackTop(&operators)) < 0)))
                     {
-                        //printf("Moving operator from operator stack to output stack\n");
-                        token t = (token)stackPop(&operators);
-                        err = evalStackPush(output, t);
-                        //free(t);
+                        //printf("Moving operator %s from operator stack to output stack\n", (char*)stackTop(&operators));
+                        evalStackPush(output, stackPop(&operators));
+                        stackPush(&intermediate, stackTop(output));
                     }
-                    //printf("Adding operator to operator stack\n");
+                    //printf("Adding operator %s to operator stack\n", tokens[i]);
                     stackPush(&operators, tokens[i]);
                 }
                 break;
@@ -807,6 +966,8 @@ int postfix(token *tokens, int numTokens, Stack *output)
                 {
                     // If the token is a left paren, then push it onto the stack
                     //printf("Adding left paren to operator stack\n");
+                    if (tokenType(stackTop(&operators)) == function)
+                        stackPush(output, FUNCTIONSEPARATOR);
                     stackPush(&operators, tokens[i]);
                 }
                 break;
@@ -820,30 +981,31 @@ int postfix(token *tokens, int numTokens, Stack *output)
                      */
                     while(stackSize(&operators) > 0
                         && tokenType((token)stackTop(&operators)) != lparen
-                        && stackSize(&operators) > 1
-                        && err == 0)
+                        && stackSize(&operators) > 1)
                     {
-                        //printf("Moving operator from operator stack to output stack\n");
-                        token t = (token)stackPop(&operators);
-                        err = evalStackPush(output, t);
-                        //free(t);
+                        //printf("Moving operator %s from operator stack to output stack\n", (char*)stackTop(&operators));
+                        evalStackPush(output, stackPop(&operators));
+                        stackPush(&intermediate, stackTop(output));
                     }
                     if(stackSize(&operators) > 0
                         && tokenType((token)stackTop(&operators)) != lparen)
                     {
-                        err = -1;
-                        /* Changed by Thomas Roehl */
-                        //raise(parenMismatch);
+                        err = true;
+                        raise(parenMismatch);
                     }
                     //printf("Removing left paren from operator stack\n");
-                    token t = (token)stackPop(&operators);
-                    //stackPop(&operators); // Discard lparen
-                    //free(t);
+                    stackPop(&operators); // Discard lparen
+                    while (stackSize(&operators) > 0 && tokenType((token)stackTop(&operators)) == function)
+                    {
+                        //printf("Removing function from operator stack to output stack\n");
+                        evalStackPush(output, stackPop(&operators));
+                        stackPush(&intermediate, stackTop(output));
+                    }
                 }
                 break;
+            default:
+                break;
         }
-        if (err)
-            break;
     }
     /*
      * When there are no more tokens to read:
@@ -855,60 +1017,63 @@ int postfix(token *tokens, int numTokens, Stack *output)
     {
         if(tokenType((token)stackTop(&operators)) == lparen)
         {
-            /* Changed by Thomas Roehl */
-            //raise(parenMismatch);
-            err = -1;
+            raise(parenMismatch);
+            err = true;
         }
         //printf("Moving operator from operator stack to output stack\n");
-        token t = (token)stackPop(&operators);
-        err = evalStackPush(output, t);
-        //free(t);
+        evalStackPush(output, stackPop(&operators));
+        stackPush(&intermediate, stackTop(output));
     }
+    // pop result from intermediate stack
+    stackPop(&intermediate);
+    // free remaining intermediate results
+    while (stackSize(&intermediate) > 0)
+    {
+        stackPop(&intermediate);
+    }
+    if (err == true)
+    {
+        while (stackSize(&operators) > 0)
+        {
+            token s = stackPop(&operators);
+            //printf("Freeing %s from operators stack\n", s);
+            free(s);
+        }
+    }
+    stackFree(&intermediate);
     stackFree(&operators);
     return err;
 }
-
-
-
 /* Added by Thomas Roehl (Thomas.Roehl@fau.de) as interface for LIKWID */
-int calculate_infix(char* finfix, double *result)
+int
+calculate_infix(char* finfix, double *result)
 {
     int i;
     int ret = 0;
     *result = 0;
     token* tokens = NULL;
     Stack expr;
-    nrCalcTokens = 0;
+    prefs.maxtokenlength = MAXTOKENLENGTH;
+    prefs.precision = MAXPRECISION;
     int numTokens = tokenize(finfix, &tokens);
-    calcTokens = (token*)malloc(2 * numTokens * sizeof(token));
-    if (calcTokens == NULL)
-    {
-        ret = -1;
-        *result = NAN;
-    }
-    memset(calcTokens, 0, 2 * numTokens * sizeof(token));
-    stackInit(&expr, 2*numTokens);
+    stackInit(&expr, numTokens);
     ret = postfix(tokens, numTokens, &expr);
-    if ((stackSize(&expr) != 1) || (ret < 0))
+    if ((stackSize(&expr) != 1) || (ret == true))
     {
         *result = NAN;
         goto calcerror;
     }
     else
     {
+        for (i=0; i< numTokens; i++)
+        {
+            if (tokens[i] == stackTop(&expr))
+                tokens[i] = NULL;
+        }
         *result = strtod((char*)stackTop(&expr), NULL);
     }
     ret = 0;
 calcerror:
-    for (i=0;i<nrCalcTokens; i++)
-    {
-        if (calcTokens[i] != NULL)
-            free(calcTokens[i]);
-    }
-    if (calcTokens)
-        free(calcTokens);
-    calcTokens = NULL;
-    nrCalcTokens = 0;
     for (i=0;i<numTokens;i++)
     {
         if (tokens[i])
@@ -926,4 +1091,3 @@ calcerror:
     return ret;
 }
 
-
