@@ -5,8 +5,8 @@
  *
  *      Description:  Configuration file module.
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
@@ -28,6 +28,8 @@
  * =======================================================================================
  */
 
+/* #####   HEADER FILE INCLUDES   ######################################### */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
@@ -37,23 +39,34 @@
 #include <sys/stat.h>
 #include <errno.h>
 
-
-
 #include <configuration.h>
 
+/* #####   EXPORTED VARIABLES   ########################################### */
+
 Configuration config = {NULL,NULL,NULL,NULL,-1,MAX_NUM_THREADS,MAX_NUM_NODES};
 int init_config = 0;
 
+/* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
+
 static int daemonPath_len = 0;
 static int groupPath_len = 0;
 
-static int default_configuration(void)
+/* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
+
+static int
+default_configuration(void)
 {
     int ret = 0;
     char filename[1024] = { [0 ... 1023] = '\0' };
     char *fptr = NULL;
     size_t len = 0;
     filename[0] = '\0';
+
+    groupPath_len = strlen(TOSTRING(GROUPPATH))+10;
+    config.groupPath = malloc(groupPath_len+1);
+    ret = snprintf(config.groupPath, groupPath_len, "%s", TOSTRING(GROUPPATH));
+    config.groupPath[ret] = '\0';
+#ifndef LIKWID_USE_PERFEVENT
     if (ACCESSMODE == 0)
     {
         config.daemonMode = ACCESSMODE_DIRECT;
@@ -61,13 +74,7 @@ static int default_configuration(void)
         return 0;
     }
     config.daemonMode = ACCESSMODE_DAEMON;
-    
-    groupPath_len = strlen(TOSTRING(GROUPPATH))+10;
-    config.groupPath = malloc(groupPath_len+1);
-    ret = snprintf(config.groupPath, groupPath_len, "%s", TOSTRING(GROUPPATH));
-    config.groupPath[ret] = '\0';
-    
-    
+
     FILE* fp = popen("which likwid-accessD 2>/dev/null | tr -d '\n'","r");
     if (fp == NULL)
     {
@@ -97,8 +104,9 @@ static int default_configuration(void)
             free(fptr);
         goto use_hardcoded;
     }
-    init_config = 1;
     fclose(fp);
+#endif
+    init_config = 1;
     return 0;
 use_hardcoded:
     ret = sprintf(filename,"%s", TOSTRING(ACCESSDAEMON));
@@ -111,13 +119,19 @@ use_hardcoded:
     }
     else
     {
-        ERROR_PLAIN_PRINT(Unable to get path to access daemon. Maybe your PATH environment variable does not contain the folder where you installed it or the file was moved away / not copied to that location?);
-        exit(EXIT_FAILURE);
+        if (getenv("LIKWID_NO_ACCESS") == NULL)
+        {
+            ERROR_PLAIN_PRINT(Unable to get path to access daemon. Maybe your PATH environment variable does not contain the folder where you installed it or the file was moved away / not copied to that location?);
+            exit(EXIT_FAILURE);
+        }
     }
     return 0;
 }
 
-int init_configuration(void)
+/* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
+
+int
+init_configuration(void)
 {
     int i;
     FILE* fp;
@@ -152,7 +166,7 @@ int init_configuration(void)
     {
         sprintf(filename, "%s",preconfigured);
     }
-    
+
     if ((config.topologyCfgFileName == NULL) && (strlen(filename) == 0))
     {
         if (!access(TOSTRING(TOPOFILE), R_OK))
@@ -259,14 +273,14 @@ int init_configuration(void)
         }
     }
 
-
     init_config = 1;
 
     fclose(fp);
     return 0;
 }
 
-Configuration_t get_configuration(void)
+Configuration_t
+get_configuration(void)
 {
     if (init_config == 1)
     {
@@ -275,7 +289,8 @@ Configuration_t get_configuration(void)
     return NULL;
 }
 
-int destroy_configuration(void)
+int
+destroy_configuration(void)
 {
     if (init_config == 0)
     {
@@ -304,7 +319,8 @@ int destroy_configuration(void)
     return 0;
 }
 
-int config_setGroupPath(char* path)
+int
+config_setGroupPath(const char* path)
 {
     int ret = 0;
     struct stat st;
@@ -337,3 +353,4 @@ int config_setGroupPath(char* path)
     printf("Given path is no directory\n");
     return -ENOTDIR;
 }
+
