@@ -5,8 +5,8 @@
  *
  *      Description:  Parser for CPU selection strings
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
@@ -28,14 +28,18 @@
  * =======================================================================================
  */
 
+/* #####   HEADER FILE INCLUDES   ######################################### */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
 
 #include <likwid.h>
 
+/* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 
-static int cpulist_sort(int* incpus, int* outcpus, int length)
+static int
+cpulist_sort(int* incpus, int* outcpus, int length)
 {
     int insert = 0;
     topology_init();
@@ -45,9 +49,9 @@ static int cpulist_sort(int* incpus, int* outcpus, int length)
         return -1;
     }
     int inner_loop = ceil((double)length/cpuid_topology->numThreadsPerCore);
-    for (int off=0;off < cpuid_topology->numThreadsPerCore;off++)
+    for (int off = 0; off < cpuid_topology->numThreadsPerCore; off++)
     {
-        for (int i=0; i<inner_loop;i++)
+        for (int i = 0; i < inner_loop; i++)
         {
             outcpus[insert] = incpus[(i*cpuid_topology->numThreadsPerCore)+off];
             insert++;
@@ -58,7 +62,8 @@ static int cpulist_sort(int* incpus, int* outcpus, int length)
     return insert;
 }
 
-static int cpulist_concat(int* cpulist, int startidx, int* addlist, int addlength)
+static int
+cpulist_concat(int* cpulist, int startidx, int* addlist, int addlength)
 {
     int count = 0;
     if (addlength <= 0)
@@ -73,7 +78,8 @@ static int cpulist_concat(int* cpulist, int startidx, int* addlist, int addlengt
     return count;
 }
 
-static int cpu_in_domain(int domainidx, int cpu)
+static int
+cpu_in_domain(int domainidx, int cpu)
 {
     affinity_init();
     AffinityDomains_t affinity = get_affinityDomains();
@@ -87,7 +93,8 @@ static int cpu_in_domain(int domainidx, int cpu)
     return 0;
 }
 
-static int cpuexpr_to_list(bstring bcpustr, bstring prefix, int* list, int length)
+static int
+cpuexpr_to_list(bstring bcpustr, bstring prefix, int* list, int length)
 {
     topology_init();
     CpuTopology_t cpuid_topology = get_cpuTopology();
@@ -124,8 +131,10 @@ list_done:
     return insert;
 }
 
-static int cpustr_to_cpulist_scatter(bstring bcpustr, int* cpulist, int length)
+static int
+cpustr_to_cpulist_scatter(bstring bcpustr, int* cpulist, int length)
 {
+    int max_procs = 0;
     topology_init();
     CpuTopology_t cpuid_topology = get_cpuTopology();
     affinity_init();
@@ -143,25 +152,42 @@ static int cpustr_to_cpulist_scatter(bstring bcpustr, int* cpulist, int length)
         }
         for (int i=0; i<affinity->numberOfAffinityDomains; i++)
         {
-            if (bstrchrp(affinity->domains[i].tag, cpustring[0], 0) != BSTR_ERR)
+            if (bstrchrp(affinity->domains[i].tag, cpustring[0], 0) != BSTR_ERR &&
+                affinity->domains[i].numberOfProcessors > 0)
             {
                 suitable[suitidx] = i;
                 suitidx++;
+                if (affinity->domains[i].numberOfProcessors > max_procs)
+                    max_procs = affinity->domains[i].numberOfProcessors;
             }
         }
-        int* sortedList = (int*) malloc(affinity->domains[suitable[0]].numberOfProcessors * sizeof(int));
-        if (!sortedList)
+        int** sLists = (int**) malloc(suitidx * sizeof(int*));
+        if (!sLists)
         {
             free(suitable);
             bcstrfree(cpustring);
             return -ENOMEM;
         }
-        for (int off=0;off<affinity->domains[suitable[0]].numberOfProcessors;off++)
+        for (int i = 0; i< suitidx; i++)
+        {
+            sLists[i] = (int*) malloc(max_procs * sizeof(int));
+            if (!sLists[i])
+            {
+                free(suitable);
+                for (int j=0; i<i; j++)
+                {
+                    free(sLists[j]);
+                }
+                bcstrfree(cpustring);
+                return -ENOMEM;
+            }
+            cpulist_sort(affinity->domains[suitable[i]].processorList, sLists[i], affinity->domains[suitable[i]].numberOfProcessors);
+        }
+        for (int off=0;off<max_procs;off++)
         {
             for(int i=0;i < suitidx; i++)
             {
-                cpulist_sort(affinity->domains[suitable[i]].processorList, sortedList, affinity->domains[suitable[i]].numberOfProcessors);
-                cpulist[insert] = sortedList[off];
+                cpulist[insert] = sLists[i][off];
                 insert++;
                 if (insert == length)
                     goto scatter_done;
@@ -169,7 +195,11 @@ static int cpustr_to_cpulist_scatter(bstring bcpustr, int* cpulist, int length)
         }
 scatter_done:
         bcstrfree(cpustring);
-        free(sortedList);
+        for (int i = 0; i< suitidx; i++)
+        {
+            free(sLists[i]);
+        }
+        free(sLists);
         free(suitable);
         return insert;
     }
@@ -177,7 +207,8 @@ scatter_done:
     return 0;
 }
 
-static int cpustr_to_cpulist_expression(bstring bcpustr, int* cpulist, int length)
+static int
+cpustr_to_cpulist_expression(bstring bcpustr, int* cpulist, int length)
 {
     topology_init();
     CpuTopology_t cpuid_topology = get_cpuTopology();
@@ -228,11 +259,11 @@ static int cpustr_to_cpulist_expression(bstring bcpustr, int* cpulist, int lengt
     int insert = 0;
     for (int i=0;i<count;i++)
     {
-        for (int j=0;j<chunk && offset+j<affinity->domains[domainidx].numberOfProcessors;j++)
+        for (int j=0; j<chunk && offset+j<affinity->domains[domainidx].numberOfProcessors;j++)
         {
             cpulist[insert] = affinity->domains[domainidx].processorList[offset + j];
             insert++;
-            if (insert == length)
+            if (insert == length || insert == count)
                 goto expression_done;
         }
         offset += stride;
@@ -252,7 +283,8 @@ expression_done:
     return insert;
 }
 
-static int cpustr_to_cpulist_logical(bstring bcpustr, int* cpulist, int length)
+static int
+cpustr_to_cpulist_logical(bstring bcpustr, int* cpulist, int length)
 {
     topology_init();
     CpuTopology_t cpuid_topology = get_cpuTopology();
@@ -264,7 +296,7 @@ static int cpustr_to_cpulist_logical(bstring bcpustr, int* cpulist, int length)
     struct bstrList* strlist;
     if (bstrchrp(bcpustr, 'L', 0) != 0)
     {
-        fprintf(stderr, "Not a valid CPU expression\n");
+        fprintf(stderr, "ERROR: Not a valid CPU expression\n");
         return 0;
     }
 
@@ -288,11 +320,12 @@ static int cpustr_to_cpulist_logical(bstring bcpustr, int* cpulist, int length)
     }
     if (domainidx < 0)
     {
-        fprintf(stderr, "Cannot find domain %s\n", bdata(bdomain));
+        fprintf(stderr, "ERROR: Cannot find domain %s\n", bdata(bdomain));
         bdestroy(bdomain);
         bdestroy(blist);
         return 0;
     }
+
     int *inlist = malloc(affinity->domains[domainidx].numberOfProcessors * sizeof(int));
     if (inlist == NULL)
     {
@@ -300,10 +333,16 @@ static int cpustr_to_cpulist_logical(bstring bcpustr, int* cpulist, int length)
         bdestroy(blist);
         return -ENOMEM;
     }
-    int ret = cpulist_sort(affinity->domains[domainidx].processorList, inlist, affinity->domains[domainidx].numberOfProcessors);
+
+    int ret = cpulist_sort(affinity->domains[domainidx].processorList,
+            inlist, affinity->domains[domainidx].numberOfProcessors);
 
     strlist = bsplit(blist, ',');
     int insert = 0;
+    int insert_offset = 0;
+    int inlist_offset = 0;
+    int inlist_idx = 0;
+    int require = 0;
     for (int i=0; i< strlist->qty; i++)
     {
         if (bstrchrp(strlist->entry[i], '-', 0) != BSTR_ERR)
@@ -312,27 +351,64 @@ static int cpustr_to_cpulist_logical(bstring bcpustr, int* cpulist, int length)
             indexlist = bsplit(strlist->entry[i], '-');
             if (atoi(bdata(indexlist->entry[0])) <= atoi(bdata(indexlist->entry[1])))
             {
-                for (int j=atoi(bdata(indexlist->entry[0])); j<=atoi(bdata(indexlist->entry[1]));j++)
+                require += atoi(bdata(indexlist->entry[1])) - atoi(bdata(indexlist->entry[0])) + 1;
+            }
+            else
+            {
+                require += atoi(bdata(indexlist->entry[0])) - atoi(bdata(indexlist->entry[1])) + 1;
+            }
+        }
+        else
+        {
+            require++;
+        }
+    }
+    if (require > ret && getenv("LIKWID_SILENT") == NULL)
+    {
+        fprintf(stderr,
+                "WARN: Selected affinity domain %s has only %d hardware threads, but selection string evaluates to %d threads.\n",
+                bdata(affinity->domains[domainidx].tag), ret, require);
+        fprintf(stderr, "      This results in multiple threads on the same hardware thread.\n");
+    }
+logical_redo:
+    for (int i=0; i< strlist->qty; i++)
+    {
+        if (bstrchrp(strlist->entry[i], '-', 0) != BSTR_ERR)
+        {
+            struct bstrList* indexlist;
+            indexlist = bsplit(strlist->entry[i], '-');
+            if (atoi(bdata(indexlist->entry[0])) <= atoi(bdata(indexlist->entry[1])))
+            {
+                for (int j=atoi(bdata(indexlist->entry[0])); j<=atoi(bdata(indexlist->entry[1])) && (insert_offset+insert < require);j++)
                 {
-                    cpulist[insert] = inlist[j];
+                    cpulist[insert_offset + insert] = inlist[inlist_idx % ret];
                     insert++;
-                    if (insert == length)
+                    inlist_idx++;
+                    if (insert == ret)
                     {
                         bstrListDestroy(indexlist);
-                        goto logical_done;
+                        if (insert == require)
+                            goto logical_done;
+                        else
+                            goto logical_redo;
                     }
                 }
             }
             else
             {
-                for (int j=atoi(bdata(indexlist->entry[0])); j>=atoi(bdata(indexlist->entry[1]));j--)
+                for (int j=atoi(bdata(indexlist->entry[0]));
+                        j>=atoi(bdata(indexlist->entry[1])) && (insert_offset+insert < require); j--)
                 {
-                    cpulist[insert] = inlist[j];
+                    cpulist[insert_offset + insert] = inlist[inlist_idx % ret];
                     insert++;
-                    if (insert == length)
+                    inlist_idx++;
+                    if (insert == ret)
                     {
                         bstrListDestroy(indexlist);
-                        goto logical_done;
+                        if (insert == require)
+                            goto logical_done;
+                        else
+                            goto logical_redo;
                     }
                 }
             }
@@ -340,11 +416,14 @@ static int cpustr_to_cpulist_logical(bstring bcpustr, int* cpulist, int length)
         }
         else
         {
-            cpulist[insert] = inlist[atoi(bdata(strlist->entry[i])) % ret];
+            cpulist[insert_offset + insert] = inlist[atoi(bdata(strlist->entry[i])) % ret];
             insert++;
-            if (insert == length)
+            if (insert == ret)
             {
-                goto logical_done;
+                if (insert == require)
+                    goto logical_done;
+                else
+                    goto logical_redo;
             }
         }
     }
@@ -353,12 +432,11 @@ logical_done:
     bdestroy(blist);
     bstrListDestroy(strlist);
     free(inlist);
-    return insert;
+    return require;
 }
 
-
-
-static int cpustr_to_cpulist_physical(bstring bcpustr, int* cpulist, int length)
+static int
+cpustr_to_cpulist_physical(bstring bcpustr, int* cpulist, int length)
 {
     topology_init();
     CpuTopology_t cpuid_topology = get_cpuTopology();
@@ -471,7 +549,10 @@ physical_done:
     return insert;
 }
 
-int cpustr_to_cpulist(char* cpustring, int* cpulist, int length)
+/* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
+
+int
+cpustr_to_cpulist(const char* cpustring, int* cpulist, int length)
 {
     int insert = 0;
     int len = 0;
@@ -510,7 +591,9 @@ int cpustr_to_cpulist(char* cpustring, int* cpulist, int length)
         }
         else if (cpuid_topology->activeHWThreads < cpuid_topology->numHWThreads)
         {
-            fprintf(stdout, "INFO: You are running LIKWID in a cpuset with %d CPUs, only logical numbering allowed\n", cpuid_topology->activeHWThreads);
+            fprintf(stdout,
+                    "INFO: You are running LIKWID in a cpuset with %d CPUs, only logical numbering allowed\n",
+                    cpuid_topology->activeHWThreads);
             if (((bstrchrp(strlist->entry[i], 'N', 0) == 0) ||
                 (bstrchrp(strlist->entry[i], 'S', 0) == 0) ||
                 (bstrchrp(strlist->entry[i], 'C', 0) == 0) ||
@@ -558,7 +641,8 @@ int cpustr_to_cpulist(char* cpustring, int* cpulist, int length)
     return insert;
 }
 
-int nodestr_to_nodelist(char* nodestr, int* nodes, int length)
+int
+nodestr_to_nodelist(const char* nodestr, int* nodes, int length)
 {
     int ret = 0;
     bstring prefix = bformat("M");
@@ -569,7 +653,8 @@ int nodestr_to_nodelist(char* nodestr, int* nodes, int length)
     return ret;
 }
 
-int sockstr_to_socklist(char* sockstr, int* sockets, int length)
+int
+sockstr_to_socklist(const char* sockstr, int* sockets, int length)
 {
     int ret = 0;
     bstring prefix = bformat("S");
@@ -579,3 +664,4 @@ int sockstr_to_socklist(char* sockstr, int* sockets, int length)
     bdestroy(prefix);
     return ret;
 }
+
