@@ -5,8 +5,8 @@
  *
  *      Description:  Header File HPM access Module
  *
- *      Version:   4.1
- *      Released:  13.6.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
@@ -27,7 +27,6 @@
  *
  * =======================================================================================
  */
-
 #ifndef ACCESS_H
 #define ACCESS_H
 
@@ -40,5 +39,4 @@ int HPMread(int cpu_id, PciDeviceIndex dev, uint32_t reg, uint64_t* data);
 int HPMwrite(int cpu_id, PciDeviceIndex dev, uint32_t reg, uint64_t data);
 int HPMcheck(PciDeviceIndex dev, int cpu_id);
 
-
-#endif
+#endif /* ACCESS_H */
