@@ -5,8 +5,8 @@
  *
  *      Description:  Header file for the interface to x86 related functions for the access module.
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
@@ -27,8 +27,8 @@
  *
  * =======================================================================================
  */
-#ifndef LIKWID_ACCESS_X86_H
-#define LIKWID_ACCESS_X86_H
+#ifndef ACCESS_X86_H
+#define ACCESS_X86_H
 
 #include <types.h>
 
@@ -38,5 +38,4 @@ int access_x86_write(PciDeviceIndex dev, const int cpu_id, uint32_t reg, uint64_
 void access_x86_finalize(int cpu_id);
 int access_x86_check(PciDeviceIndex dev, int cpu_id);
 
-
-#endif
+#endif /* ACCESS_X86_H */
