@@ -5,8 +5,8 @@
  *
  *      Description:  Header File affinity Module
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@gmail.com
@@ -28,7 +28,6 @@
  *
  * =======================================================================================
  */
-
 #ifndef AFFINITY_H
 #define AFFINITY_H
 
@@ -36,15 +35,14 @@
 #include <likwid.h>
 
 int socket_lock[MAX_NUM_NODES];
+int core_lock[MAX_NUM_THREADS];
 int tile_lock[MAX_NUM_THREADS];
 extern AffinityDomains affinityDomains;
 
 extern int affinity_core2node_lookup[MAX_NUM_THREADS];
-
+extern int affinity_thread2core_lookup[MAX_NUM_THREADS];
 extern int affinity_processGetProcessorId();
 extern int affinity_threadGetProcessorId();
 extern const AffinityDomain* affinity_getDomain(bstring domain);
 
-
 #endif /*AFFINITY_H*/
-
