@@ -5,8 +5,8 @@
  *
  *      Description:  Types file for libperfctr module.
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
@@ -27,7 +27,6 @@
  *
  * =======================================================================================
  */
-
 #ifndef LIBPERFCTR_H
 #define LIBPERFCTR_H
 
@@ -35,6 +34,7 @@
 
 typedef struct LikwidThreadResults{
     bstring  label;
+    uint32_t index;
     double time;
     TimerData startTime;
     int groupID;
