@@ -5,8 +5,8 @@
  *
  *      Description:  Header File procfs/sysfs NUMA backend
  *
- *      Version:   4.1
- *      Released:  13.6.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
@@ -27,13 +27,11 @@
  *
  * =======================================================================================
  */
-
 #ifndef LIKWID_NUMA_PROC
 #define LIKWID_NUMA_PROC
 
 extern int proc_numa_init(void);
 extern void proc_numa_membind(void* ptr, size_t size, int domainId);
-extern void proc_numa_setInterleaved(int* processorList, int numberOfProcessors);
-
+extern void proc_numa_setInterleaved(const int* processorList, int numberOfProcessors);
 
 #endif
