@@ -5,8 +5,8 @@
  *
  *      Description:  Header File of performance group and event set handler
  *
- *      Version:   4.1
- *      Released:  13.6.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@gmail.com
  *      Project:  likwid
@@ -27,11 +27,9 @@
  *
  * =======================================================================================
  */
-
 #ifndef PERFGROUP_H
 #define PERFGROUP_H
 
-
  /*! \brief The groupInfo data structure describes a performance group
 
 Groups can be either be read in from file or be a group with custom event set. For
@@ -73,10 +71,10 @@ static char* groupFileSectionNames[5] = {
     "LONG"
 };
 
-extern int get_groups(char* grouppath, char* architecture, char*** groupnames, char*** groupshort, char*** grouplong);
+extern int get_groups(const char* grouppath, const char* architecture, char*** groupnames, char*** groupshort, char*** grouplong);
 extern void return_groups(int groups, char** groupnames, char** groupshort, char** grouplong);
-extern int read_group(char* grouppath, char* architecture, char* groupname, GroupInfo* ginfo);
-extern int custom_group(char* eventStr, GroupInfo* ginfo);
+extern int read_group(const char* grouppath, const char* architecture, const char* groupname, GroupInfo* ginfo);
+extern int custom_group(const char* eventStr, GroupInfo* ginfo);
 extern char* get_eventStr(GroupInfo* ginfo);
 void put_eventStr(char* eventset);
 extern char* get_shortInfo(GroupInfo* ginfo);
@@ -87,8 +85,9 @@ extern void return_group(GroupInfo* ginfo);
 
 extern void init_clist(CounterList* clist);
 extern int add_to_clist(CounterList* clist, char* counter, double result);
+extern int update_clist(CounterList* clist, char* counter, double result);
 extern void destroy_clist(CounterList* clist);
 
 extern int calc_metric(char* formula, CounterList* clist, double *result);
 
-#endif
+#endif /* PERFGROUP_H */
