@@ -7,14 +7,14 @@
  *                    Configures and reads out performance counters
  *                    on x86 based architectures. Supports multi threading.
  *
- *      Version:   4.1
- *      Released:  13.6.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
  *
- *      Copyright (C) 2016 RRZE, University Erlangen-Nuremberg
+ *      Copyright (C) 2015 RRZE, University Erlangen-Nuremberg
  *
  *      This program is free software: you can redistribute it and/or modify it under
  *      the terms of the GNU General Public License as published by the Free Software
@@ -107,7 +107,7 @@ extern char* eventOptionTypeName[NUM_EVENT_OPTIONS];
         (1ULL<<type))
 
 
-/** @cond */ 
+/** @cond */
 #define EVENT_OPTION_OPCODE_MASK (1ULL<<EVENT_OPTION_OPCODE)
 #define EVENT_OPTION_MATCH0_MASK (1ULL<<EVENT_OPTION_MATCH0)
 #define EVENT_OPTION_MATCH1_MASK (1ULL<<EVENT_OPTION_MATCH1)
@@ -135,7 +135,7 @@ extern char* eventOptionTypeName[NUM_EVENT_OPTIONS];
 
 /*! \brief Structure specifying thread to CPU relation
 
-Threads are always numbered incrementally. This structure is used in order to 
+Threads are always numbered incrementally. This structure is used in order to
 resolve the real HW thread ID.
 \extends PerfmonGroupSet
 */
@@ -214,11 +214,10 @@ typedef struct {
     TimerData             timer; /*!< \brief Time information how long the counters were running */
     double                rdtscTime; /*!< \brief Evaluation of the Time information in seconds */
     double                runTime; /*!< \brief Sum of all time information in seconds that the group was running */
-#ifdef __x86_64
-    __uint128_t           regTypeMask; /*!< \brief Bitmask for easy checks which types are included in the eventSet */
-#else
-    uint64_t              regTypeMask; /*!< \brief Bitmask for easy checks which types are included in the eventSet */
-#endif
+    uint64_t              regTypeMask1; /*!< \brief Bitmask1 for easy checks which types are included in the eventSet */
+    uint64_t              regTypeMask2; /*!< \brief Bitmask2 for easy checks which types are included in the eventSet */
+    uint64_t              regTypeMask3; /*!< \brief Bitmask3 for easy checks which types are included in the eventSet */
+    uint64_t              regTypeMask4; /*!< \brief Bitmask4 for easy checks which types are included in the eventSet */
     GroupState            state; /*!< \brief Current state of the event group (configured, started, none) */
     GroupInfo             group; /*!< \brief Structure holding the performance group information */
 } PerfmonEventSet;
