@@ -6,8 +6,8 @@
  *      Description:  Header File Power Module
  *                    Implements Intel RAPL Interface.
  *
- *      Version:   4.1
- *      Released:  13.6.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
@@ -29,7 +29,6 @@
  *
  * =======================================================================================
  */
-
 #ifndef POWER_H
 #define POWER_H
 
@@ -68,7 +67,7 @@ uint32_t info_regs[NUM_POWER_DOMAINS] = {MSR_PKG_POWER_INFO,
 
 
 double
-power_printEnergy(PowerData* data)
+power_printEnergy(const PowerData* data)
 {
     return  (double) ((data->after - data->before) * power_info.domains[data->domain].energyUnit);
 }
