@@ -5,8 +5,8 @@
  *
  *      Description:  Register Defines for the perfmon module
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
@@ -28,7 +28,6 @@
  *
  * =======================================================================================
  */
-
 #ifndef REGISTERS_H
 #define REGISTERS_H
 
@@ -48,10 +47,10 @@
 #define MSR_PERFEVTSEL1           0x187
 #define MSR_PERFEVTSEL2           0x188
 #define MSR_PERFEVTSEL3           0x189
-#define MSR_PERFEVTSEL4           0x190
-#define MSR_PERFEVTSEL5           0x191
-#define MSR_PERFEVTSEL6           0x192
-#define MSR_PERFEVTSEL7           0x193
+#define MSR_PERFEVTSEL4           0x18A
+#define MSR_PERFEVTSEL5           0x18B
+#define MSR_PERFEVTSEL6           0x18C
+#define MSR_PERFEVTSEL7           0x18D
 #define MSR_PMC0                  0x0C1
 #define MSR_PMC1                  0x0C2
 #define MSR_PMC2                  0x0C3
@@ -91,8 +90,8 @@
 #define MSR_UNCORE_PMC5                0x3B5
 #define MSR_UNCORE_PMC6                0x3B6
 #define MSR_UNCORE_PMC7                0x3B7
-/* 
- * Perfmon V3 (starting with Haswell, according to 
+/*
+ * Perfmon V3 (starting with Haswell, according to
  * Intel software developers guide also for SandyBridge,
  * IvyBridge not mentioned in this section)
  */
@@ -165,10 +164,585 @@
 #define MSR_MIC_PERF_GLOBAL_OVF_CTRL  0x02E
 #define MSR_MIC_PERF_GLOBAL_CTRL      0x02F
 /* Xeon Phi (Knights Landing)*/
-#define MSR_MIC2_PMC0                 0x4C1
-#define MSR_MIC2_PMC1                 0x4C2
+#define MSR_MIC2_PMC0                 0xC1
+#define MSR_MIC2_PMC1                 0xC2
+#define MSR_MIC2_PERFEVTSEL0          0x186
+#define MSR_MIC2_PERFEVTSEL1          0x187
 #define MSR_MIC2_TURBO_RATIO_LIMIT    0x1AD
-
+#define MSR_MIC2_SPFLT_CONTROL	      0x02C
+#define MSR_MIC2_PERF_GLOBAL_STATUS    0x02D
+#define MSR_MIC2_PERF_GLOBAL_OVF_CTRL  0x02E
+#define MSR_MIC2_PERF_GLOBAL_CTRL      0x02F
+/* Xeon Phi (Knights Landing) UBOX*/
+#define MSR_MIC2_U_GLOBAL_CTRL      0x700
+#define MSR_MIC2_U_GLOBAL_STATUS    0x701
+#define MSR_MIC2_U_CONFIG           0x702
+#define MSR_MIC2_U_FIXED_CTRL	      0x703
+#define MSR_MIC2_U_FIXED_CTR	      0x704
+#define MSR_MIC2_U_CTRL0	      0x705
+#define MSR_MIC2_U_CTRL1	      0x706
+#define MSR_MIC2_U_OVFL	      0x708
+#define MSR_MIC2_U_CTR0	      0x709
+#define MSR_MIC2_U_CTR1	      0x70A
+/* Xeon Phi (Knights Landing) WBOX*/
+#define MSR_MIC2_PCU_GLOBAL_CTRL    0x710
+#define MSR_MIC2_PCU_CTRL0   0x711
+#define MSR_MIC2_PCU_CTRL1   0x712
+#define MSR_MIC2_PCU_CTRL2   0x713
+#define MSR_MIC2_PCU_CTRL3   0x714
+#define MSR_MIC2_PCU_CTR0    0x717
+#define MSR_MIC2_PCU_CTR1    0x718
+#define MSR_MIC2_PCU_CTR2    0x719
+#define MSR_MIC2_PCU_CTR3    0x71A
+/* Xeon Phi (Knights Landing) Cache boxes*/
+#define MSR_MIC2_C0_GLOBAL_CTRL       0xE00
+#define MSR_MIC2_C0_CTRL0             0xE01
+#define MSR_MIC2_C0_CTRL1             0xE02
+#define MSR_MIC2_C0_CTRL2             0xE03
+#define MSR_MIC2_C0_CTRL3             0xE04
+#define MSR_MIC2_C0_CTR0             0xE08
+#define MSR_MIC2_C0_CTR1             0xE09
+#define MSR_MIC2_C0_CTR2             0xE0A
+#define MSR_MIC2_C0_CTR3             0xE0B
+#define MSR_MIC2_C0_FILTER0          0xE05
+#define MSR_MIC2_C0_FILTER1          0xE06
+#define MSR_MIC2_C0_STATUS           0xE07
+#define MSR_MIC2_C1_GLOBAL_CTRL       0xE0C
+#define MSR_MIC2_C1_CTRL0             0xE0D
+#define MSR_MIC2_C1_CTRL1             0xE0E
+#define MSR_MIC2_C1_CTRL2             0xE0F
+#define MSR_MIC2_C1_CTRL3             0xE10
+#define MSR_MIC2_C1_CTR0             0xE14
+#define MSR_MIC2_C1_CTR1             0xE15
+#define MSR_MIC2_C1_CTR2             0xE16
+#define MSR_MIC2_C1_CTR3             0xE17
+#define MSR_MIC2_C1_FILTER0          0xE11
+#define MSR_MIC2_C1_FILTER1          0xE12
+#define MSR_MIC2_C1_STATUS           0xE13
+#define MSR_MIC2_C2_GLOBAL_CTRL       0xE18
+#define MSR_MIC2_C2_CTRL0             0xE19
+#define MSR_MIC2_C2_CTRL1             0xE1A
+#define MSR_MIC2_C2_CTRL2             0xE1B
+#define MSR_MIC2_C2_CTRL3             0xE1C
+#define MSR_MIC2_C2_CTR0             0xE20
+#define MSR_MIC2_C2_CTR1             0xE21
+#define MSR_MIC2_C2_CTR2             0xE22
+#define MSR_MIC2_C2_CTR3             0xE23
+#define MSR_MIC2_C2_FILTER0          0xE1D
+#define MSR_MIC2_C2_FILTER1          0xE1E
+#define MSR_MIC2_C2_STATUS           0xE1F
+#define MSR_MIC2_C3_GLOBAL_CTRL       0xE24
+#define MSR_MIC2_C3_CTRL0             0xE25
+#define MSR_MIC2_C3_CTRL1             0xE26
+#define MSR_MIC2_C3_CTRL2             0xE27
+#define MSR_MIC2_C3_CTRL3             0xE28
+#define MSR_MIC2_C3_CTR0             0xE2C
+#define MSR_MIC2_C3_CTR1             0xE2D
+#define MSR_MIC2_C3_CTR2             0xE2E
+#define MSR_MIC2_C3_CTR3             0xE2F
+#define MSR_MIC2_C3_FILTER0          0xE29
+#define MSR_MIC2_C3_FILTER1          0xE2A
+#define MSR_MIC2_C3_STATUS           0xE2B
+#define MSR_MIC2_C4_GLOBAL_CTRL       0xE30
+#define MSR_MIC2_C4_CTRL0             0xE31
+#define MSR_MIC2_C4_CTRL1             0xE32
+#define MSR_MIC2_C4_CTRL2             0xE33
+#define MSR_MIC2_C4_CTRL3             0xE34
+#define MSR_MIC2_C4_CTR0             0xE38
+#define MSR_MIC2_C4_CTR1             0xE39
+#define MSR_MIC2_C4_CTR2             0xE3A
+#define MSR_MIC2_C4_CTR3             0xE3B
+#define MSR_MIC2_C4_FILTER0          0xE35
+#define MSR_MIC2_C4_FILTER1          0xE36
+#define MSR_MIC2_C4_STATUS           0xE37
+#define MSR_MIC2_C5_GLOBAL_CTRL       0xE3C
+#define MSR_MIC2_C5_CTRL0             0xE3D
+#define MSR_MIC2_C5_CTRL1             0xE3E
+#define MSR_MIC2_C5_CTRL2             0xE3F
+#define MSR_MIC2_C5_CTRL3             0xE40
+#define MSR_MIC2_C5_CTR0             0xE44
+#define MSR_MIC2_C5_CTR1             0xE45
+#define MSR_MIC2_C5_CTR2             0xE46
+#define MSR_MIC2_C5_CTR3             0xE47
+#define MSR_MIC2_C5_FILTER0          0xE41
+#define MSR_MIC2_C5_FILTER1          0xE42
+#define MSR_MIC2_C5_STATUS           0xE43
+#define MSR_MIC2_C6_GLOBAL_CTRL       0xE48
+#define MSR_MIC2_C6_CTRL0             0xE49
+#define MSR_MIC2_C6_CTRL1             0xE4A
+#define MSR_MIC2_C6_CTRL2             0xE4B
+#define MSR_MIC2_C6_CTRL3             0xE4C
+#define MSR_MIC2_C6_CTR0             0xE50
+#define MSR_MIC2_C6_CTR1             0xE51
+#define MSR_MIC2_C6_CTR2             0xE52
+#define MSR_MIC2_C6_CTR3             0xE53
+#define MSR_MIC2_C6_FILTER0          0xE4D
+#define MSR_MIC2_C6_FILTER1          0xE4E
+#define MSR_MIC2_C6_STATUS           0xE4F
+#define MSR_MIC2_C7_GLOBAL_CTRL       0xE54
+#define MSR_MIC2_C7_CTRL0             0xE55
+#define MSR_MIC2_C7_CTRL1             0xE56
+#define MSR_MIC2_C7_CTRL2             0xE57
+#define MSR_MIC2_C7_CTRL3             0xE58
+#define MSR_MIC2_C7_CTR0             0xE5C
+#define MSR_MIC2_C7_CTR1             0xE5D
+#define MSR_MIC2_C7_CTR2             0xE5E
+#define MSR_MIC2_C7_CTR3             0xE5F
+#define MSR_MIC2_C7_FILTER0          0xE59
+#define MSR_MIC2_C7_FILTER1          0xE5A
+#define MSR_MIC2_C7_STATUS           0xE5B
+#define MSR_MIC2_C8_GLOBAL_CTRL       0xE60
+#define MSR_MIC2_C8_CTRL0             0xE61
+#define MSR_MIC2_C8_CTRL1             0xE62
+#define MSR_MIC2_C8_CTRL2             0xE63
+#define MSR_MIC2_C8_CTRL3             0xE64
+#define MSR_MIC2_C8_CTR0             0xE68
+#define MSR_MIC2_C8_CTR1             0xE69
+#define MSR_MIC2_C8_CTR2             0xE6A
+#define MSR_MIC2_C8_CTR3             0xE6B
+#define MSR_MIC2_C8_FILTER0          0xE65
+#define MSR_MIC2_C8_FILTER1          0xE66
+#define MSR_MIC2_C8_STATUS           0xE67
+#define MSR_MIC2_C9_GLOBAL_CTRL       0xE6C
+#define MSR_MIC2_C9_CTRL0             0xE6D
+#define MSR_MIC2_C9_CTRL1             0xE6E
+#define MSR_MIC2_C9_CTRL2             0xE6F
+#define MSR_MIC2_C9_CTRL3             0xE70
+#define MSR_MIC2_C9_CTR0             0xE74
+#define MSR_MIC2_C9_CTR1             0xE75
+#define MSR_MIC2_C9_CTR2             0xE76
+#define MSR_MIC2_C9_CTR3             0xE77
+#define MSR_MIC2_C9_FILTER0          0xE71
+#define MSR_MIC2_C9_FILTER1          0xE72
+#define MSR_MIC2_C9_STATUS           0xE73
+#define MSR_MIC2_C10_GLOBAL_CTRL       0xE78
+#define MSR_MIC2_C10_CTRL0             0xE79
+#define MSR_MIC2_C10_CTRL1             0xE7A
+#define MSR_MIC2_C10_CTRL2             0xE7B
+#define MSR_MIC2_C10_CTRL3             0xE7C
+#define MSR_MIC2_C10_CTR0             0xE80
+#define MSR_MIC2_C10_CTR1             0xE81
+#define MSR_MIC2_C10_CTR2             0xE82
+#define MSR_MIC2_C10_CTR3             0xE83
+#define MSR_MIC2_C10_FILTER0          0xE7D
+#define MSR_MIC2_C10_FILTER1          0xE7E
+#define MSR_MIC2_C10_STATUS           0xE7F
+#define MSR_MIC2_C11_GLOBAL_CTRL       0xE84
+#define MSR_MIC2_C11_CTRL0             0xE85
+#define MSR_MIC2_C11_CTRL1             0xE86
+#define MSR_MIC2_C11_CTRL2             0xE87
+#define MSR_MIC2_C11_CTRL3             0xE88
+#define MSR_MIC2_C11_CTR0             0xE8C
+#define MSR_MIC2_C11_CTR1             0xE8D
+#define MSR_MIC2_C11_CTR2             0xE8E
+#define MSR_MIC2_C11_CTR3             0xE8F
+#define MSR_MIC2_C11_FILTER0          0xE89
+#define MSR_MIC2_C11_FILTER1          0xE8A
+#define MSR_MIC2_C11_STATUS           0xE8B
+#define MSR_MIC2_C12_GLOBAL_CTRL       0xE90
+#define MSR_MIC2_C12_CTRL0             0xE91
+#define MSR_MIC2_C12_CTRL1             0xE92
+#define MSR_MIC2_C12_CTRL2             0xE93
+#define MSR_MIC2_C12_CTRL3             0xE94
+#define MSR_MIC2_C12_CTR0             0xE98
+#define MSR_MIC2_C12_CTR1             0xE99
+#define MSR_MIC2_C12_CTR2             0xE9A
+#define MSR_MIC2_C12_CTR3             0xE9B
+#define MSR_MIC2_C12_FILTER0          0xE95
+#define MSR_MIC2_C12_FILTER1          0xE96
+#define MSR_MIC2_C12_STATUS           0xE97
+#define MSR_MIC2_C13_GLOBAL_CTRL       0xE9C
+#define MSR_MIC2_C13_CTRL0             0xE9D
+#define MSR_MIC2_C13_CTRL1             0xE9E
+#define MSR_MIC2_C13_CTRL2             0xE9F
+#define MSR_MIC2_C13_CTRL3             0xEA0
+#define MSR_MIC2_C13_CTR0             0xEA4
+#define MSR_MIC2_C13_CTR1             0xEA5
+#define MSR_MIC2_C13_CTR2             0xEA6
+#define MSR_MIC2_C13_CTR3             0xEA7
+#define MSR_MIC2_C13_FILTER0          0xEA1
+#define MSR_MIC2_C13_FILTER1          0xEA2
+#define MSR_MIC2_C13_STATUS           0xEA3
+#define MSR_MIC2_C14_GLOBAL_CTRL       0xEA8
+#define MSR_MIC2_C14_CTRL0             0xEA9
+#define MSR_MIC2_C14_CTRL1             0xEAA
+#define MSR_MIC2_C14_CTRL2             0xEAB
+#define MSR_MIC2_C14_CTRL3             0xEAC
+#define MSR_MIC2_C14_CTR0             0xEB0
+#define MSR_MIC2_C14_CTR1             0xEB1
+#define MSR_MIC2_C14_CTR2             0xEB2
+#define MSR_MIC2_C14_CTR3             0xEB3
+#define MSR_MIC2_C14_FILTER0          0xEAD
+#define MSR_MIC2_C14_FILTER1          0xEAE
+#define MSR_MIC2_C14_STATUS           0xEAF
+#define MSR_MIC2_C15_GLOBAL_CTRL       0xEB4
+#define MSR_MIC2_C15_CTRL0             0xEB5
+#define MSR_MIC2_C15_CTRL1             0xEB6
+#define MSR_MIC2_C15_CTRL2             0xEB7
+#define MSR_MIC2_C15_CTRL3             0xEB8
+#define MSR_MIC2_C15_CTR0             0xEBC
+#define MSR_MIC2_C15_CTR1             0xEBD
+#define MSR_MIC2_C15_CTR2             0xEBE
+#define MSR_MIC2_C15_CTR3             0xEBF
+#define MSR_MIC2_C15_FILTER0          0xEB9
+#define MSR_MIC2_C15_FILTER1          0xEBA
+#define MSR_MIC2_C15_STATUS           0xEBB
+#define MSR_MIC2_C16_GLOBAL_CTRL       0xEC0
+#define MSR_MIC2_C16_CTRL0             0xEC1
+#define MSR_MIC2_C16_CTRL1             0xEC2
+#define MSR_MIC2_C16_CTRL2             0xEC3
+#define MSR_MIC2_C16_CTRL3             0xEC4
+#define MSR_MIC2_C16_CTR0             0xEC8
+#define MSR_MIC2_C16_CTR1             0xEC9
+#define MSR_MIC2_C16_CTR2             0xECA
+#define MSR_MIC2_C16_CTR3             0xECB
+#define MSR_MIC2_C16_FILTER0          0xEC5
+#define MSR_MIC2_C16_FILTER1          0xEC6
+#define MSR_MIC2_C16_STATUS           0xEC7
+#define MSR_MIC2_C17_GLOBAL_CTRL       0xECC
+#define MSR_MIC2_C17_CTRL0             0xECD
+#define MSR_MIC2_C17_CTRL1             0xECE
+#define MSR_MIC2_C17_CTRL2             0xECF
+#define MSR_MIC2_C17_CTRL3             0xED0
+#define MSR_MIC2_C17_CTR0             0xED4
+#define MSR_MIC2_C17_CTR1             0xED5
+#define MSR_MIC2_C17_CTR2             0xED6
+#define MSR_MIC2_C17_CTR3             0xED7
+#define MSR_MIC2_C17_FILTER0          0xED1
+#define MSR_MIC2_C17_FILTER1          0xED2
+#define MSR_MIC2_C17_STATUS           0xED3
+#define MSR_MIC2_C18_GLOBAL_CTRL       0xED8
+#define MSR_MIC2_C18_CTRL0             0xED9
+#define MSR_MIC2_C18_CTRL1             0xEDA
+#define MSR_MIC2_C18_CTRL2             0xEDB
+#define MSR_MIC2_C18_CTRL3             0xED0
+#define MSR_MIC2_C18_CTR0             0xEDF
+#define MSR_MIC2_C18_CTR1             0xEE0
+#define MSR_MIC2_C18_CTR2             0xEE1
+#define MSR_MIC2_C18_CTR3             0xEE2
+#define MSR_MIC2_C18_FILTER0          0xEE3
+#define MSR_MIC2_C18_FILTER1          0xEDD
+#define MSR_MIC2_C18_STATUS           0xEDE
+#define MSR_MIC2_C19_GLOBAL_CTRL       0xEE4
+#define MSR_MIC2_C19_CTRL0             0xEE5
+#define MSR_MIC2_C19_CTRL1             0xEE6
+#define MSR_MIC2_C19_CTRL2             0xEE7
+#define MSR_MIC2_C19_CTRL3             0xEE8
+#define MSR_MIC2_C19_CTR0             0xEEC
+#define MSR_MIC2_C19_CTR1             0xEED
+#define MSR_MIC2_C19_CTR2             0xEEE
+#define MSR_MIC2_C19_CTR3             0xEEF
+#define MSR_MIC2_C19_FILTER0          0xEE9
+#define MSR_MIC2_C19_FILTER1          0xEEA
+#define MSR_MIC2_C19_STATUS           0xEEB
+#define MSR_MIC2_C20_GLOBAL_CTRL       0xEF0
+#define MSR_MIC2_C20_CTRL0             0xEF1
+#define MSR_MIC2_C20_CTRL1             0xEF2
+#define MSR_MIC2_C20_CTRL2             0xEF3
+#define MSR_MIC2_C20_CTRL3             0xEF4
+#define MSR_MIC2_C20_CTR0             0xEF8
+#define MSR_MIC2_C20_CTR1             0xEF9
+#define MSR_MIC2_C20_CTR2             0xEFA
+#define MSR_MIC2_C20_CTR3             0xEFB
+#define MSR_MIC2_C20_FILTER0          0xEF5
+#define MSR_MIC2_C20_FILTER1          0xEF6
+#define MSR_MIC2_C20_STATUS           0xEF7
+#define MSR_MIC2_C21_GLOBAL_CTRL       0xEFC
+#define MSR_MIC2_C21_CTRL0             0xEFD
+#define MSR_MIC2_C21_CTRL1             0xEFE
+#define MSR_MIC2_C21_CTRL2             0xEFF
+#define MSR_MIC2_C21_CTRL3             0xF00
+#define MSR_MIC2_C21_CTR0             0xF04
+#define MSR_MIC2_C21_CTR1             0xF05
+#define MSR_MIC2_C21_CTR2             0xF06
+#define MSR_MIC2_C21_CTR3             0xF07
+#define MSR_MIC2_C21_FILTER0          0xF01
+#define MSR_MIC2_C21_FILTER1          0xF02
+#define MSR_MIC2_C21_STATUS           0xF03
+#define MSR_MIC2_C22_GLOBAL_CTRL       0xF08
+#define MSR_MIC2_C22_CTRL0             0xF09
+#define MSR_MIC2_C22_CTRL1             0xF0A
+#define MSR_MIC2_C22_CTRL2             0xF0B
+#define MSR_MIC2_C22_CTRL3             0xF0C
+#define MSR_MIC2_C22_CTR0             0xF10
+#define MSR_MIC2_C22_CTR1             0xF11
+#define MSR_MIC2_C22_CTR2             0xF12
+#define MSR_MIC2_C22_CTR3             0xF13
+#define MSR_MIC2_C22_FILTER0          0xF0D
+#define MSR_MIC2_C22_FILTER1          0xF0E
+#define MSR_MIC2_C22_STATUS           0xF0F
+#define MSR_MIC2_C23_GLOBAL_CTRL       0xF14
+#define MSR_MIC2_C23_CTRL0             0xF15
+#define MSR_MIC2_C23_CTRL1             0xF16
+#define MSR_MIC2_C23_CTRL2             0xF17
+#define MSR_MIC2_C23_CTRL3             0xF18
+#define MSR_MIC2_C23_CTR0             0xF1C
+#define MSR_MIC2_C23_CTR1             0xF1D
+#define MSR_MIC2_C23_CTR2             0xF1E
+#define MSR_MIC2_C23_CTR3             0xF1F
+#define MSR_MIC2_C23_FILTER0          0xF19
+#define MSR_MIC2_C23_FILTER1          0xF1A
+#define MSR_MIC2_C23_STATUS           0xF1B
+#define MSR_MIC2_C24_GLOBAL_CTRL       0xF20
+#define MSR_MIC2_C24_CTRL0             0xF21
+#define MSR_MIC2_C24_CTRL1             0xF22
+#define MSR_MIC2_C24_CTRL2             0xF23
+#define MSR_MIC2_C24_CTRL3             0xF24
+#define MSR_MIC2_C24_CTR0             0xF28
+#define MSR_MIC2_C24_CTR1             0xF29
+#define MSR_MIC2_C24_CTR2             0xF2A
+#define MSR_MIC2_C24_CTR3             0xF2B
+#define MSR_MIC2_C24_FILTER0          0xF25
+#define MSR_MIC2_C24_FILTER1          0xF26
+#define MSR_MIC2_C24_STATUS           0xF27
+#define MSR_MIC2_C25_GLOBAL_CTRL       0xF2C
+#define MSR_MIC2_C25_CTRL0             0xF2D
+#define MSR_MIC2_C25_CTRL1             0xF2E
+#define MSR_MIC2_C25_CTRL2             0xF2F
+#define MSR_MIC2_C25_CTRL3             0xF30
+#define MSR_MIC2_C25_CTR0             0xF34
+#define MSR_MIC2_C25_CTR1             0xF35
+#define MSR_MIC2_C25_CTR2             0xF36
+#define MSR_MIC2_C25_CTR3             0xF37
+#define MSR_MIC2_C25_FILTER0          0xF31
+#define MSR_MIC2_C25_FILTER1          0xF32
+#define MSR_MIC2_C25_STATUS           0xF33
+#define MSR_MIC2_C26_GLOBAL_CTRL       0xF38
+#define MSR_MIC2_C26_CTRL0             0xF39
+#define MSR_MIC2_C26_CTRL1             0xF3A
+#define MSR_MIC2_C26_CTRL2             0xF3B
+#define MSR_MIC2_C26_CTRL3             0xF3C
+#define MSR_MIC2_C26_CTR0             0xF40
+#define MSR_MIC2_C26_CTR1             0xF41
+#define MSR_MIC2_C26_CTR2             0xF42
+#define MSR_MIC2_C26_CTR3             0xF43
+#define MSR_MIC2_C26_FILTER0          0xF3D
+#define MSR_MIC2_C26_FILTER1          0xF3E
+#define MSR_MIC2_C26_STATUS           0xF3F
+#define MSR_MIC2_C27_GLOBAL_CTRL       0xF44
+#define MSR_MIC2_C27_CTRL0             0xF45
+#define MSR_MIC2_C27_CTRL1             0xF46
+#define MSR_MIC2_C27_CTRL2             0xF47
+#define MSR_MIC2_C27_CTRL3             0xF48
+#define MSR_MIC2_C27_CTR0             0xF4C
+#define MSR_MIC2_C27_CTR1             0xF4D
+#define MSR_MIC2_C27_CTR2             0xF4E
+#define MSR_MIC2_C27_CTR3             0xF4F
+#define MSR_MIC2_C27_FILTER0          0xF49
+#define MSR_MIC2_C27_FILTER1          0xF4A
+#define MSR_MIC2_C27_STATUS           0xF4B
+#define MSR_MIC2_C28_GLOBAL_CTRL       0xF50
+#define MSR_MIC2_C28_CTRL0             0xF51
+#define MSR_MIC2_C28_CTRL1             0xF52
+#define MSR_MIC2_C28_CTRL2             0xF53
+#define MSR_MIC2_C28_CTRL3             0xF54
+#define MSR_MIC2_C28_CTR0             0xF58
+#define MSR_MIC2_C28_CTR1             0xF59
+#define MSR_MIC2_C28_CTR2             0xF5A
+#define MSR_MIC2_C28_CTR3             0xF5B
+#define MSR_MIC2_C28_FILTER0          0xF55
+#define MSR_MIC2_C28_FILTER1          0xF56
+#define MSR_MIC2_C28_STATUS           0xF57
+#define MSR_MIC2_C29_GLOBAL_CTRL       0xF5C
+#define MSR_MIC2_C29_CTRL0             0xF5D
+#define MSR_MIC2_C29_CTRL1             0xF5E
+#define MSR_MIC2_C29_CTRL2             0xF5F
+#define MSR_MIC2_C29_CTRL3             0xF60
+#define MSR_MIC2_C29_CTR0             0xF64
+#define MSR_MIC2_C29_CTR1             0xF65
+#define MSR_MIC2_C29_CTR2             0xF66
+#define MSR_MIC2_C29_CTR3             0xF67
+#define MSR_MIC2_C29_FILTER0          0xF61
+#define MSR_MIC2_C29_FILTER1          0xF62
+#define MSR_MIC2_C29_STATUS           0xF63
+#define MSR_MIC2_C30_GLOBAL_CTRL       0xF68
+#define MSR_MIC2_C30_CTRL0             0xF69
+#define MSR_MIC2_C30_CTRL1             0xF6A
+#define MSR_MIC2_C30_CTRL2             0xF6B
+#define MSR_MIC2_C30_CTRL3             0xF6C
+#define MSR_MIC2_C30_CTR0             0xF70
+#define MSR_MIC2_C30_CTR1             0xF71
+#define MSR_MIC2_C30_CTR2             0xF72
+#define MSR_MIC2_C30_CTR3             0xF73
+#define MSR_MIC2_C30_FILTER0          0xF6D
+#define MSR_MIC2_C30_FILTER1          0xF6E
+#define MSR_MIC2_C30_STATUS           0xF6F
+#define MSR_MIC2_C31_GLOBAL_CTRL       0xF74
+#define MSR_MIC2_C31_CTRL0             0xF75
+#define MSR_MIC2_C31_CTRL1             0xF76
+#define MSR_MIC2_C31_CTRL2             0xF77
+#define MSR_MIC2_C31_CTRL3             0xF78
+#define MSR_MIC2_C31_CTR0             0xF7C
+#define MSR_MIC2_C31_CTR1             0xF7D
+#define MSR_MIC2_C31_CTR2             0xF7E
+#define MSR_MIC2_C31_CTR3             0xF7F
+#define MSR_MIC2_C31_FILTER0          0xF79
+#define MSR_MIC2_C31_FILTER1          0xF7A
+#define MSR_MIC2_C31_STATUS           0xF7B
+#define MSR_MIC2_C32_GLOBAL_CTRL       0xF80
+#define MSR_MIC2_C32_CTRL0             0xF81
+#define MSR_MIC2_C32_CTRL1             0xF82
+#define MSR_MIC2_C32_CTRL2             0xF83
+#define MSR_MIC2_C32_CTRL3             0xF84
+#define MSR_MIC2_C32_CTR0             0xF88
+#define MSR_MIC2_C32_CTR1             0xF89
+#define MSR_MIC2_C32_CTR2             0xF8A
+#define MSR_MIC2_C32_CTR3             0xF8B
+#define MSR_MIC2_C32_FILTER0          0xF85
+#define MSR_MIC2_C32_FILTER1          0xF86
+#define MSR_MIC2_C32_STATUS           0xF87
+#define MSR_MIC2_C33_GLOBAL_CTRL       0xF8C
+#define MSR_MIC2_C33_CTRL0             0xF8D
+#define MSR_MIC2_C33_CTRL1             0xF8E
+#define MSR_MIC2_C33_CTRL2             0xF8F
+#define MSR_MIC2_C33_CTRL3             0xF90
+#define MSR_MIC2_C33_CTR0             0xF94
+#define MSR_MIC2_C33_CTR1             0xF95
+#define MSR_MIC2_C33_CTR2             0xF96
+#define MSR_MIC2_C33_CTR3             0xF97
+#define MSR_MIC2_C33_FILTER0          0xF91
+#define MSR_MIC2_C33_FILTER1          0xF92
+#define MSR_MIC2_C33_STATUS           0xF93
+#define MSR_MIC2_C34_GLOBAL_CTRL       0xF98
+#define MSR_MIC2_C34_CTRL0             0xF99
+#define MSR_MIC2_C34_CTRL1             0xF9A
+#define MSR_MIC2_C34_CTRL2             0xF9B
+#define MSR_MIC2_C34_CTRL3             0xF9C
+#define MSR_MIC2_C34_CTR0             0xFA0
+#define MSR_MIC2_C34_CTR1             0xFA1
+#define MSR_MIC2_C34_CTR2             0xFA2
+#define MSR_MIC2_C34_CTR3             0xFA3
+#define MSR_MIC2_C34_FILTER0          0xF9D
+#define MSR_MIC2_C34_FILTER1          0xF9E
+#define MSR_MIC2_C34_STATUS           0xF9F
+#define MSR_MIC2_C35_GLOBAL_CTRL       0xFA4
+#define MSR_MIC2_C35_CTRL0             0xFA5
+#define MSR_MIC2_C35_CTRL1             0xFA6
+#define MSR_MIC2_C35_CTRL2             0xFA7
+#define MSR_MIC2_C35_CTRL3             0xFA8
+#define MSR_MIC2_C35_CTR0             0xFAC
+#define MSR_MIC2_C35_CTR1             0xFAD
+#define MSR_MIC2_C35_CTR2             0xFAE
+#define MSR_MIC2_C35_CTR3             0xFAF
+#define MSR_MIC2_C35_FILTER0          0xFA9
+#define MSR_MIC2_C35_FILTER1          0xFAA
+#define MSR_MIC2_C35_STATUS           0xFAB
+#define MSR_MIC2_C36_GLOBAL_CTRL       0xFB0
+#define MSR_MIC2_C36_CTRL0             0xFB1
+#define MSR_MIC2_C36_CTRL1             0xFB2
+#define MSR_MIC2_C36_CTRL2             0xFB3
+#define MSR_MIC2_C36_CTRL3             0xFB4
+#define MSR_MIC2_C36_CTR0             0xFB8
+#define MSR_MIC2_C36_CTR1             0xFB9
+#define MSR_MIC2_C36_CTR2             0xFBA
+#define MSR_MIC2_C36_CTR3             0xFBB
+#define MSR_MIC2_C36_FILTER0          0xFB5
+#define MSR_MIC2_C36_FILTER1          0xFB6
+#define MSR_MIC2_C36_STATUS           0xFB7
+#define MSR_MIC2_C37_GLOBAL_CTRL       0xFBC
+#define MSR_MIC2_C37_CTRL0             0xFBD
+#define MSR_MIC2_C37_CTRL1             0xFBE
+#define MSR_MIC2_C37_CTRL2             0xFBF
+#define MSR_MIC2_C37_CTRL3             0xFC0
+#define MSR_MIC2_C37_CTR0             0xFC4
+#define MSR_MIC2_C37_CTR1             0xFC5
+#define MSR_MIC2_C37_CTR2             0xFC6
+#define MSR_MIC2_C37_CTR3             0xFC7
+#define MSR_MIC2_C37_FILTER0          0xFC1
+#define MSR_MIC2_C37_FILTER1          0xFC2
+#define MSR_MIC2_C37_STATUS           0xFC3
+/* Xeon Phi (Knights Landing) Embedded DRAM controller aka High Bandwidth Memory */
+#define PCI_MIC2_EDC_U_CTR0_A	     0x404
+#define PCI_MIC2_EDC_U_CTR0_B	     0x400
+#define PCI_MIC2_EDC_U_CTR1_A	     0x40C
+#define PCI_MIC2_EDC_U_CTR1_B	     0x408
+#define PCI_MIC2_EDC_U_CTR2_A	     0x414
+#define PCI_MIC2_EDC_U_CTR2_B	     0x410
+#define PCI_MIC2_EDC_U_CTR3_A	     0x41C
+#define PCI_MIC2_EDC_U_CTR3_B	     0x418
+#define PCI_MIC2_EDC_U_CTRL0	     0x420
+#define PCI_MIC2_EDC_U_CTRL1	     0x424
+#define PCI_MIC2_EDC_U_CTRL2	     0x428
+#define PCI_MIC2_EDC_U_CTRL3	     0x42C
+#define PCI_MIC2_EDC_U_BOX_CTRL	     0x430
+#define PCI_MIC2_EDC_U_BOX_STATUS    0x434
+#define PCI_MIC2_EDC_U_FIXED_CTR_A   0x450
+#define PCI_MIC2_EDC_U_FIXED_CTR_B   0x44C
+#define PCI_MIC2_EDC_U_FIXED_CTRL    0x454
+#define PCI_MIC2_EDC_D_CTR0_A	     0xA04
+#define PCI_MIC2_EDC_D_CTR0_B	     0xA00
+#define PCI_MIC2_EDC_D_CTR1_A	     0xA0C
+#define PCI_MIC2_EDC_D_CTR1_B	     0xA08
+#define PCI_MIC2_EDC_D_CTR2_A	     0xA14
+#define PCI_MIC2_EDC_D_CTR2_B	     0xA10
+#define PCI_MIC2_EDC_D_CTR3_A	     0xA1C
+#define PCI_MIC2_EDC_D_CTR3_B	     0xA18
+#define PCI_MIC2_EDC_D_CTRL0	     0xA20
+#define PCI_MIC2_EDC_D_CTRL1	     0xA24
+#define PCI_MIC2_EDC_D_CTRL2	     0xA28
+#define PCI_MIC2_EDC_D_CTRL3	     0xA2C
+#define PCI_MIC2_EDC_D_BOX_CTRL	     0xA30
+#define PCI_MIC2_EDC_D_BOX_STATUS    0xA34
+#define PCI_MIC2_EDC_D_FIXED_CTR_A   0xA40
+#define PCI_MIC2_EDC_D_FIXED_CTR_B   0xA3C
+#define PCI_MIC2_EDC_D_FIXED_CTRL    0xA44
+/* Xeon Phi (Knights Landing) Memory controller*/
+#define PCI_MIC2_MC_U_CTR0_A	     0x404
+#define PCI_MIC2_MC_U_CTR0_B	     0x400
+#define PCI_MIC2_MC_U_CTR1_A	     0x40C
+#define PCI_MIC2_MC_U_CTR1_B	     0x408
+#define PCI_MIC2_MC_U_CTR2_A	     0x414
+#define PCI_MIC2_MC_U_CTR2_B	     0x410
+#define PCI_MIC2_MC_U_CTR3_A	     0x41C
+#define PCI_MIC2_MC_U_CTR3_B	     0x418
+#define PCI_MIC2_MC_U_CTRL0	     0x420
+#define PCI_MIC2_MC_U_CTRL1	     0x424
+#define PCI_MIC2_MC_U_CTRL2	     0x428
+#define PCI_MIC2_MC_U_CTRL3	     0x42C
+#define PCI_MIC2_MC_U_BOX_CTRL	     0x430
+#define PCI_MIC2_MC_U_BOX_STATUS    0x434
+#define PCI_MIC2_MC_U_FIXED_CTR_A   0x450
+#define PCI_MIC2_MC_U_FIXED_CTR_B   0x44C
+#define PCI_MIC2_MC_U_FIXED_CTRL    0x454
+#define PCI_MIC2_MC_D_CTR0_A	     0xB04
+#define PCI_MIC2_MC_D_CTR0_B	     0xB00
+#define PCI_MIC2_MC_D_CTR1_A	     0xB0C
+#define PCI_MIC2_MC_D_CTR1_B	     0xB08
+#define PCI_MIC2_MC_D_CTR2_A	     0xB14
+#define PCI_MIC2_MC_D_CTR2_B	     0xB10
+#define PCI_MIC2_MC_D_CTR3_A	     0xB1C
+#define PCI_MIC2_MC_D_CTR3_B	     0xB18
+#define PCI_MIC2_MC_D_CTRL0	     0xB20
+#define PCI_MIC2_MC_D_CTRL1	     0xB24
+#define PCI_MIC2_MC_D_CTRL2	     0xB28
+#define PCI_MIC2_MC_D_CTRL3	     0xB2C
+#define PCI_MIC2_MC_D_BOX_CTRL	     0xB30
+#define PCI_MIC2_MC_D_BOX_STATUS    0xB34
+#define PCI_MIC2_MC_D_FIXED_CTR_A   0xB40
+#define PCI_MIC2_MC_D_FIXED_CTR_B   0xB3C
+#define PCI_MIC2_MC_D_FIXED_CTRL    0xB44
+/* Xeon Phi (Knights Landing) M2PCIE */
+#define PCI_MIC2_M2PCIE_CTR0_A	    0xA4
+#define PCI_MIC2_M2PCIE_CTR0_B	    0xA0
+#define PCI_MIC2_M2PCIE_CTR1_A	    0xAC
+#define PCI_MIC2_M2PCIE_CTR1_B	    0xA8
+#define PCI_MIC2_M2PCIE_CTR2_A	    0xB4
+#define PCI_MIC2_M2PCIE_CTR2_B	    0xB0
+#define PCI_MIC2_M2PCIE_CTR3_A	    0xBC
+#define PCI_MIC2_M2PCIE_CTR3_B	    0xB8
+#define PCI_MIC2_M2PCIE_CTRL0	    0xD8
+#define PCI_MIC2_M2PCIE_CTRL1	    0xDC
+#define PCI_MIC2_M2PCIE_CTRL2	    0xE0
+#define PCI_MIC2_M2PCIE_CTRL3	    0xE4
+#define PCI_MIC2_M2PCIE_BOX_CTRL    0xF4
+#define PCI_MIC2_M2PCIE_BOX_STATUS  0xF8
+/* Xeon Phi (Knights Landing) IRP */
+#define PCI_MIC2_IRP_CTR0	    0xA0
+#define PCI_MIC2_IRP_CTR1	    0xA8
+#define PCI_MIC2_IRP_CTRL0	    0xD8
+#define PCI_MIC2_IRP_CTRL1	    0xDC
+#define PCI_MIC2_IRP_BOX_CTRL	    0xF0
+#define PCI_MIC2_IRP_BOX_STATUS	    0xF4
 
 /* Core v1/v2 type uncore
  * Naming following Intel Uncore Performance Monitoring Guide
@@ -1328,10 +1902,13 @@
 #define MSR_ANY_CORE_C0                 0x659
 #define MSR_ANY_GFXE_C0                 0x65A
 #define MSR_CORE_GFXE_OVERLAP_C0        0x65B
+#define MSR_UNCORE_FREQ                 0x620
+#define MSR_FSB_FREQ                    0xCD
+#define MSR_ENERGY_PERF_BIAS            0x1B0
+#define MSR_ALT_PEBS                    0x39C
 /*
  * AMD
  */
-
 #define MSR_AMD_PERFEVTSEL0           0xC0010000
 #define MSR_AMD_PERFEVTSEL1           0xC0010001
 #define MSR_AMD_PERFEVTSEL2           0xC0010002
@@ -1396,4 +1973,3 @@
 #define MSR_AMD16_NB_PMC3               0xC0010247
 
 #endif /* REGISTERS_H */
-
