@@ -3,12 +3,12 @@
  *
  *      Filename:  textcolor.h
  *
- *      Description:  Header File textcolor Module. 
- *                    Allows toggling of terminal escape sequences for 
+ *      Description:  Header File textcolor Module.
+ *                    Allows toggling of terminal escape sequences for
  *                    colored text.
  *
- *      Version:   4.1
- *      Released:  13.6.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *      Project:  likwid
@@ -29,7 +29,6 @@
  *
  * =======================================================================================
  */
-
 #ifndef TEXTCOLOR_H
 #define TEXTCOLOR_H
 
@@ -53,7 +52,8 @@
 static void color_on(int attr, int fg);
 static void color_reset(void);
 
-static void color_on(int attr, int fg)
+static void
+color_on(int attr, int fg)
 {
     char command[13];
 
@@ -61,7 +61,8 @@ static void color_on(int attr, int fg)
     printf("%s", command);
 }
 
-static void color_reset()
+static void
+color_reset()
 {
     char command[13];
 
