@@ -5,8 +5,8 @@
  *
  *      Description:  Header File of topology backend using cpuid instruction.
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
@@ -28,9 +28,8 @@
  *
  * =======================================================================================
  */
-
-#ifndef LIKWID_TOPOLOGY_CPUID
-#define LIKWID_TOPOLOGY_CPUID
+#ifndef TOPOLOGY_CPUID_H
+#define TOPOLOGY_CPUID_H
 
 #include <sched.h>
 
@@ -39,5 +38,4 @@ void cpuid_init_cpuFeatures(void);
 void cpuid_init_nodeTopology(cpu_set_t cpuSet);
 void cpuid_init_cacheTopology(void);
 
-
-#endif
+#endif /* TOPOLOGY_CPUID_H */
