@@ -5,8 +5,8 @@
  *
  *      Description:  Header File of topology backend using the hwloc library
  *
- *      Version:   4.1
- *      Released:  13.6.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
@@ -27,26 +27,20 @@
  *
  * =======================================================================================
  */
-
-#ifndef LIKWID_TOPOLOGY_HWLOC
-#define LIKWID_TOPOLOGY_HWLOC
-
+#ifndef TOPOLOGY_HWLOC_H
+#define TOPOLOGY_HWLOC_H
 
 #include <hwloc.h>
 #include <sched.h>
 
-
 extern hwloc_topology_t hwloc_topology;
 
 int likwid_hwloc_record_objs_of_type_below_obj(hwloc_topology_t t, hwloc_obj_t obj, hwloc_obj_type_t type, int* index, uint32_t **list);
 
-
-
 void hwloc_init_cpuInfo(cpu_set_t cpuSet);
 void hwloc_init_cpuFeatures(void);
 void hwloc_init_nodeTopology(cpu_set_t cpuSet);
 void hwloc_init_cacheTopology(void);
 void hwloc_close(void);
 
-
-#endif
+#endif /* TOPOLOGY_HWLOC_H */
