@@ -5,8 +5,8 @@
  *
  *      Description:  Header File of topology backend using procfs/sysfs
  *
- *      Version:   4.1
- *      Released:  13.6.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
@@ -27,9 +27,8 @@
  *
  * =======================================================================================
  */
-
-#ifndef LIKWID_TOPOLOGY_PROC
-#define LIKWID_TOPOLOGY_PROC
+#ifndef TOPOLOGY_PROC_H
+#define TOPOLOGY_PROC_H
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -47,5 +46,4 @@ void proc_init_cpuFeatures(void);
 void proc_init_nodeTopology(cpu_set_t cpuSet);
 void proc_init_cacheTopology(void);
 
-
-#endif
+#endif /* TOPOLOGY_PROC_H */
