@@ -5,8 +5,8 @@
  *
  *      Description:  Marker API interface of module perfmon
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
@@ -56,6 +56,7 @@
 /* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
 int socket_lock[MAX_NUM_NODES];
+int core_lock[MAX_NUM_THREADS];
 static int likwid_init = 0;
 static int numberOfGroups = 0;
 static int* groups;
@@ -68,6 +69,7 @@ static pthread_mutex_t globalLock = PTHREAD_MUTEX_INITIALIZER;
 static int use_locks = 0;
 static pthread_mutex_t threadLocks[MAX_NUM_THREADS] = { [ 0 ... (MAX_NUM_THREADS-1)] = PTHREAD_MUTEX_INITIALIZER};
 
+
 /* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
 
 #define gettid() syscall(SYS_gettid)
@@ -75,7 +77,9 @@ static pthread_mutex_t threadLocks[MAX_NUM_THREADS] = { [ 0 ... (MAX_NUM_THREADS
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 
 
-static int getProcessorID(cpu_set_t* cpu_set)
+
+static int
+getProcessorID(cpu_set_t* cpu_set)
 {
     int processorId;
 
@@ -89,7 +93,8 @@ static int getProcessorID(cpu_set_t* cpu_set)
     return processorId;
 }
 
-static int getThreadID(int cpu_id)
+static int
+getThreadID(int cpu_id)
 {
     int i;
     for(i=0;i<groupSet->numberOfThreads;i++)
@@ -130,10 +135,12 @@ calculateMarkerResult(RegisterIndex index, uint64_t start, uint64_t stop, int ov
 
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
-void likwid_markerInit(void)
+void
+likwid_markerInit(void)
 {
     int i;
     int verbosity;
+    int setinit = 0;
     bstring bThreadStr;
     bstring bEventStr;
     struct bstrList* threadTokens;
@@ -146,9 +153,9 @@ void likwid_markerInit(void)
     int (*ownatoi)(const char*);
     ownatoi = &atoi;
 
-    if ((modeStr != NULL) && (filepath != NULL) && (eventStr != NULL) && (cThreadStr != NULL))
+    if ((modeStr != NULL) && (filepath != NULL) && (eventStr != NULL) && (cThreadStr != NULL) && likwid_init == 0)
     {
-        likwid_init = 1;
+        setinit = 1;
     }
     else if (likwid_init == 0)
     {
@@ -172,9 +179,9 @@ void likwid_markerInit(void)
     hashTable_init();
 
     for(int i=0; i<MAX_NUM_NODES; i++) socket_lock[i] = LOCK_INIT;
-
+#ifndef LIKWID_USE_PERFEVENT
     HPMmode(atoi(modeStr));
-
+#endif
     if (getenv("LIKWID_DEBUG") != NULL)
     {
         perfmon_verbosity = atoi(getenv("LIKWID_DEBUG"));
@@ -190,7 +197,7 @@ void likwid_markerInit(void)
     }
     bdestroy(bThreadStr);
     bstrListDestroy(threadTokens);
-    
+
     if (getenv("LIKWID_PIN") != NULL)
     {
         likwid_pinThread(threads2Cpu[0]);
@@ -213,7 +220,7 @@ void likwid_markerInit(void)
     i = perfmon_init(num_cpus, threads2Cpu);
     if (i<0)
     {
-        fprintf(stderr,"Failed to initialize LIKWID perfmon library.\n");
+        //fprintf(stderr,"Failed to initialize LIKWID perfmon library.\n");
         return;
     }
 
@@ -243,18 +250,26 @@ void likwid_markerInit(void)
             groupSet->groups[groups[0]].state = STATE_START;
         }
     }
-
+    if (setinit)
+    {
+        likwid_init = 1;
+    }
     groupSet->activeGroup = 0;
+#ifdef LIKWID_USE_PERFEVENT
+    perfmon_setupCounters(groupSet->activeGroup);
+    perfmon_startCounters();
+#endif
 }
 
-void likwid_markerThreadInit(void)
+void
+likwid_markerThreadInit(void)
 {
     int myID;
     if ( !likwid_init )
     {
         return;
     }
-    
+
     pthread_mutex_lock(&globalLock);
     myID = registered_cpus++;
     pthread_mutex_unlock(&globalLock);
@@ -272,7 +287,8 @@ void likwid_markerThreadInit(void)
     }
 }
 
-void likwid_markerNextGroup(void)
+void
+likwid_markerNextGroup(void)
 {
     int i;
     int next_group;
@@ -297,7 +313,8 @@ void likwid_markerNextGroup(void)
  * 4 regionID threadID countersvalues(space separated)
  * 5 regionID threadID countersvalues
  */
-void likwid_markerClose(void)
+void
+likwid_markerClose(void)
 {
     FILE *file = NULL;
     LikwidResults* results = NULL;
@@ -320,16 +337,20 @@ void likwid_markerClose(void)
     markerfile = getenv("LIKWID_FILEPATH");
     if (markerfile == NULL)
     {
-        fprintf(stderr, "Is the application executed with LIKWID wrapper? No file path for the Marker API output defined.\n");
+        fprintf(stderr,
+                "Is the application executed with LIKWID wrapper? No file path for the Marker API output defined.\n");
         return;
     }
     file = fopen(markerfile,"w");
 
     if (file != NULL)
     {
-        DEBUG_PRINT(DEBUGLEV_DEVELOP, Creating Marker file %s with %d regions %d groups and %d threads, markerfile, numberOfRegions, numberOfGroups, numberOfThreads);
+        DEBUG_PRINT(DEBUGLEV_DEVELOP,
+                Creating Marker file %s with %d regions %d groups and %d threads,
+                markerfile, numberOfRegions, numberOfGroups, numberOfThreads);
         fprintf(file,"%d %d %d\n",numberOfThreads, numberOfRegions, numberOfGroups);
         DEBUG_PRINT(DEBUGLEV_DEVELOP, %d %d %d, numberOfThreads, numberOfRegions, numberOfGroups);
+
         for (int i=0; i<numberOfRegions; i++)
         {
             fprintf(file,"%d:%s\n",i,bdata(results[i].tag));
@@ -345,7 +366,10 @@ void likwid_markerClose(void)
                 fprintf(file,"%u ",results[i].count[j]);
                 fprintf(file,"%e ",results[i].time[j]);
                 fprintf(file,"%d ",groupSet->groups[results[i].groupID].numberOfEvents);
-                lineidx = sprintf(&(line[0]), "%d %d %d %u %e %d ", i, results[i].groupID,results[i].cpulist[j],results[i].count[j],results[i].time[j],groupSet->groups[results[i].groupID].numberOfEvents);
+                lineidx = sprintf(&(line[0]), "%d %d %d %u %e %d ",
+                        i, results[i].groupID,results[i].cpulist[j],results[i].count[j],
+                        results[i].time[j],groupSet->groups[results[i].groupID].numberOfEvents);
+
                 for (int k=0; k<groupSet->groups[results[i].groupID].numberOfEvents; k++)
                 {
                     fprintf(file,"%e ",results[i].counters[j][k]);
@@ -363,6 +387,20 @@ void likwid_markerClose(void)
         fprintf(stderr, "%s", strerror(errno));
     }
 
+}
+
+void __attribute__((destructor (101))) likwid_markerCloseDestruct(void)
+{
+    LikwidResults* results = NULL;
+    int numberOfThreads = 0;
+    int numberOfRegions = 0;
+    if (!likwid_init)
+        return;
+    hashTable_finalize(&numberOfThreads, &numberOfRegions, &results);
+    if ((numberOfThreads == 0)||(numberOfThreads == 0))
+    {
+        return;
+    }
     for (int i=0;i<numberOfRegions; i++)
     {
         for (int j=0;j<numberOfThreads; j++)
@@ -384,7 +422,8 @@ void likwid_markerClose(void)
     HPMfinalize();
 }
 
-int likwid_markerRegisterRegion(const char* regionTag)
+int
+likwid_markerRegisterRegion(const char* regionTag)
 {
     if ( ! likwid_init )
     {
@@ -398,11 +437,15 @@ int likwid_markerRegisterRegion(const char* regionTag)
     bcatcstr(tag, groupSuffix);
     int cpu_id = hashTable_get(tag, &results);
     bdestroy(tag);
+#ifdef LIKWID_USE_PERFEVENT
+    return HPMaddThread(cpu_id);
+#else
     return 0;
+#endif
 }
 
-
-int likwid_markerStartRegion(const char* regionTag)
+int
+likwid_markerStartRegion(const char* regionTag)
 {
     if ( ! likwid_init )
     {
@@ -419,30 +462,30 @@ int likwid_markerStartRegion(const char* regionTag)
     char groupSuffix[10];
     sprintf(groupSuffix, "-%d", groupSet->activeGroup);
     bcatcstr(tag, groupSuffix);
-    
+
     int cpu_id = hashTable_get(tag, &results);
     int thread_id = getThreadID(cpu_id);
     perfmon_readCountersCpu(cpu_id);
     results->cpuID = cpu_id;
     for(int i=0;i<groupSet->groups[groupSet->activeGroup].numberOfEvents;i++)
     {
-        DEBUG_PRINT(DEBUGLEV_DEVELOP, START [%s] READ EVENT [%d=%d] EVENT %d VALUE %llu , regionTag, thread_id, cpu_id, i,
-                        LLU_CAST groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData);
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, START [%s] READ EVENT [%d=%d] EVENT %d VALUE %llu,
+                regionTag, thread_id, cpu_id, i,
+                LLU_CAST groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData);
         //groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].startData =
         //        groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData;
-        
+
         results->StartPMcounters[i] = groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData;
         results->StartOverflows[i] = groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].overflows;
     }
-    
+
     bdestroy(tag);
     timer_start(&(results->startTime));
     return 0;
 }
 
-
-
-int likwid_markerStopRegion(const char* regionTag)
+int
+likwid_markerStopRegion(const char* regionTag)
 {
     if (! likwid_init)
     {
@@ -468,7 +511,7 @@ int likwid_markerStopRegion(const char* regionTag)
     {
         pthread_mutex_lock(&threadLocks[myCPU]);
     }
-    
+
     cpu_id = hashTable_get(tag, &results);
     thread_id = getThreadID(cpu_id);
     results->groupID = groupSet->activeGroup;
@@ -485,7 +528,7 @@ int likwid_markerStopRegion(const char* regionTag)
                         LLU_CAST groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData);
         result = calculateMarkerResult(groupSet->groups[groupSet->activeGroup].events[i].index, results->StartPMcounters[i],
                                         groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].counterData,
-                                        groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].overflows - 
+                                        groupSet->groups[groupSet->activeGroup].events[i].threadCounter[thread_id].overflows -
                                         results->StartOverflows[i]);
         if (counter_map[groupSet->groups[groupSet->activeGroup].events[i].index].type != THERMAL)
         {
@@ -503,8 +546,13 @@ int likwid_markerStopRegion(const char* regionTag)
     return 0;
 }
 
-
-void likwid_markerGetRegion(const char* regionTag, int* nr_events, double* events, double *time, int *count)
+void
+likwid_markerGetRegion(
+        const char* regionTag,
+        int* nr_events,
+        double* events,
+        double *time,
+        int *count)
 {
     if (! likwid_init)
     {
@@ -537,8 +585,8 @@ void likwid_markerGetRegion(const char* regionTag, int* nr_events, double* event
     return;
 }
 
-
-int  likwid_getProcessorId()
+int
+likwid_getProcessorId()
 {
     int i;
     cpu_set_t  cpu_set;
@@ -556,7 +604,8 @@ int  likwid_getProcessorId()
 }
 
 #ifdef HAS_SCHEDAFFINITY
-int  likwid_pinThread(int processorId)
+int
+likwid_pinThread(int processorId)
 {
     int ret;
     cpu_set_t cpuset;
@@ -577,8 +626,8 @@ int  likwid_pinThread(int processorId)
 }
 #endif
 
-
-int  likwid_pinProcess(int processorId)
+int
+likwid_pinProcess(int processorId)
 {
     int ret;
     cpu_set_t cpuset;
@@ -596,4 +645,3 @@ int  likwid_pinProcess(int processorId)
     return TRUE;
 }
 
-
