@@ -5,8 +5,8 @@
  *
  *      Description:  Interface to procfs/sysfs for PCI device lookup
  *
- *      Version:   4.1
- *      Released:  13.6.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Thomas Roehl (tr), thomas.roehl@googlemail.com
  *      Project:  likwid
@@ -28,19 +28,23 @@
  * =======================================================================================
  */
 
+/* #####   HEADER FILE INCLUDES   ######################################### */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdint.h>
 #include <fcntl.h>
 
-
 #include <types.h>
 #include <bstrlib.h>
 #include <affinity.h>
 #include <topology.h>
 #include <error.h>
 
-int getBusFromSocket(const uint32_t socket)
+/* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
+
+int
+getBusFromSocket(const uint32_t socket)
 {
     int cur_bus = 0;
     uint32_t cur_socket = 0;
@@ -84,7 +88,6 @@ proc_pci_init(uint16_t testDevice, char** socket_bus, int* nrSockets)
     uint16_t testVendor = 0x8086;
     uint32_t sbus, sdevfn, svend, sdev;
     int busID;
-    
 
     if ( (fptr = fopen( "/proc/bus/pci/devices", "r")) == NULL )
     {
@@ -112,14 +115,12 @@ proc_pci_init(uint16_t testDevice, char** socket_bus, int* nrSockets)
         }
     }
     fclose(fptr);
-    
     *nrSockets = cntr;
-    
     if ( cntr == 0 )
     {
         //fprintf(stderr, "Uncore not supported on this system\n");
         return -ENODEV;
     }
-    
     return 0;
 }
+
