@@ -5,8 +5,8 @@
  *
  *      Description:  Handler for performance groups and event sets
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Author:   Jan Treibig (jt), jan.treibig@gmail.com
  *                Thomas Roehl (tr), thomas.roehl@gmail.com
@@ -29,6 +29,8 @@
  * =======================================================================================
  */
 
+/* #####   HEADER FILE INCLUDES   ######################################### */
+
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -44,7 +46,12 @@
 #include <calculator.h>
 #include <likwid.h>
 
-int isdir(char* dirname)
+static int totalgroups = 0;
+
+/* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
+
+int
+isdir(char* dirname)
 {
     struct stat st;
     if (NULL == dirname) {
@@ -56,7 +63,54 @@ int isdir(char* dirname)
     return S_ISDIR(st.st_mode) ? 1 : 0;
 }
 
-int get_groups(char* grouppath, char* architecture, char*** groupnames, char*** groupshort, char*** grouplong)
+void
+return_groups(int groups, char** groupnames, char** groupshort, char** grouplong)
+{
+    int i;
+    int freegroups = (totalgroups < groups ? groups : totalgroups);
+    for (i = 0; i <freegroups; i++)
+    {
+        free(groupnames[i]);
+        groupnames[i] = NULL;
+        if (i < groups)
+        {
+            if (groupshort[i] != NULL)
+            {
+                free(groupshort[i]);
+                groupshort[i] = NULL;
+            }
+            if (grouplong[i] != NULL)
+            {
+                free(grouplong[i]);
+                grouplong[i] = NULL;
+            }
+        }
+    }
+    if (groupnames != NULL)
+    {
+        free(groupnames);
+        groupnames = NULL;
+    }
+    if (groupshort != NULL)
+    {
+        free(groupshort);
+        groupshort = NULL;
+    }
+    if (grouplong != NULL)
+    {
+        free(grouplong);
+        grouplong = NULL;
+    }
+}
+
+
+int
+get_groups(
+        const char* grouppath,
+        const char* architecture,
+        char*** groupnames,
+        char*** groupshort,
+        char*** grouplong)
 {
     int i = 0, j = 0, s = 0;
     int fsize = 0, hsize = 0;
@@ -70,6 +124,7 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
     int search_home = 0;
     bstring SHORT = bformat("SHORT");
     bstring LONG = bformat("LONG");
+    bstring REQUIRE = bformat("REQUIRE_NOHT");
     int read_long = 0;
     if ((grouppath == NULL)||(architecture == NULL)||(groupnames == NULL))
         return -EINVAL;
@@ -78,6 +133,7 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
     {
         bdestroy(SHORT);
         bdestroy(LONG);
+        bdestroy(REQUIRE);
         return -ENOMEM;
     }
     char* homepath = malloc((strlen(getenv("HOME"))+strlen(architecture)+50) * sizeof(char));
@@ -86,6 +142,7 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
         free(fullpath);
         bdestroy(SHORT);
         bdestroy(LONG);
+        bdestroy(REQUIRE);
         return -ENOMEM;
     }
     fsize = sprintf(fullpath, "%s/%s", grouppath, architecture);
@@ -99,6 +156,7 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
             free(homepath);
             bdestroy(SHORT);
             bdestroy(LONG);
+            bdestroy(REQUIRE);
             return -EACCES;
         }
     }
@@ -109,6 +167,7 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
         free(homepath);
         bdestroy(SHORT);
         bdestroy(LONG);
+        bdestroy(REQUIRE);
         return -EACCES;
     }
     i = 0;
@@ -117,7 +176,7 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
     {
         if (strncmp(&(ep->d_name[strlen(ep->d_name)-4]), ".txt", 4) == 0)
         {
-            i++;
+            totalgroups++;
             if (strlen(ep->d_name)-4 > s)
                 s = strlen(ep->d_name)-4;
         }
@@ -138,7 +197,7 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
             {
                 if (strncmp(&(ep->d_name[strlen(ep->d_name)-4]), ".txt", 4) == 0)
                 {
-                    i++;
+                    totalgroups++;
                     if (strlen(ep->d_name)-4 > s)
                         s = strlen(ep->d_name)-4;
                 }
@@ -146,17 +205,18 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
             closedir(dp);
         }
     }
-
-    *groupnames = malloc(i * sizeof(char**));
+    *groupnames = malloc(totalgroups * sizeof(char**));
     if (*groupnames == NULL)
     {
         free(fullpath);
         free(homepath);
         bdestroy(SHORT);
         bdestroy(LONG);
+        bdestroy(REQUIRE);
         return -ENOMEM;
     }
-    *groupshort = malloc(i * sizeof(char**));
+    memset(*groupnames, 0, totalgroups * sizeof(char**));
+    *groupshort = malloc(totalgroups * sizeof(char**));
     if (*groupshort == NULL)
     {
         free(*groupnames);
@@ -165,9 +225,11 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
         free(homepath);
         bdestroy(SHORT);
         bdestroy(LONG);
+        bdestroy(REQUIRE);
         return -ENOMEM;
     }
-    *grouplong = malloc(i * sizeof(char**));
+    memset(*groupshort, 0, totalgroups * sizeof(char**));
+    *grouplong = malloc(totalgroups * sizeof(char**));
     if (*grouplong == NULL)
     {
         free(*groupnames);
@@ -178,15 +240,19 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
         free(homepath);
         bdestroy(SHORT);
         bdestroy(LONG);
+        bdestroy(REQUIRE);
         return -ENOMEM;
     }
-    for (j=0; j < i; j++)
+    memset(*grouplong, 0, totalgroups * sizeof(char**));
+    for (j=0; j < totalgroups; j++)
     {
-        (*grouplong)[i] == NULL;
-        (*groupshort)[i] == NULL;
         (*groupnames)[j] = malloc((s+1) * sizeof(char));
         if ((*groupnames)[j] == NULL)
         {
+            for (s=0; s<j; s++)
+            {
+                free((*groupnames)[s]);
+            }
             free(*groupnames);
             *groupnames = NULL;
             free(*groupshort);
@@ -197,18 +263,20 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
             free(homepath);
             bdestroy(SHORT);
             bdestroy(LONG);
+            bdestroy(REQUIRE);
             return -ENOMEM;
         }
     }
     dp = opendir(fullpath);
     i = 0;
-    
+    int skip_group = 0;
+
     while (ep = readdir(dp))
     {
         if (strncmp(&(ep->d_name[strlen(ep->d_name)-4]), ".txt", 4) == 0)
         {
             read_long = 0;
-            bstring long_info = bfromcstr("");;
+            bstring long_info = bfromcstr("");
             sprintf(&(fullpath[fsize]), "/%s", ep->d_name);
             if (!access(fullpath, R_OK))
             {
@@ -216,7 +284,7 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
                 s = sprintf((*groupnames)[i], "%.*s", (int)(strlen(ep->d_name)-4), ep->d_name);
                 (*groupnames)[i][s] = '\0';
                 fp = fopen(fullpath,"r");
-                
+
                 while (fgets (buf, sizeof(buf), fp)) {
                     bstring bbuf = bfromcstr(buf);
                     btrimws(bbuf);
@@ -262,9 +330,12 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
                             bdestroy(LONG);
                             bdestroy(bbuf);
                             bdestroy(sinfo);
+                            bdestroy(REQUIRE);
                             free(homepath);
                             free(fullpath);
+                            bdestroy(long_info);
                             bstrListDestroy(linelist);
+                            return_groups(i, *groupnames, *groupshort, *grouplong);
                             return -ENOMEM;
                         }
                         s = sprintf((*groupshort)[i], "%s", bdata(sinfo));
@@ -272,6 +343,13 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
                         bstrListDestroy(linelist);
                         bdestroy(sinfo);
                     }
+                    else if (bstrncmp(bbuf, REQUIRE, blength(REQUIRE)) == 0)
+                    {
+                        if (cpuid_topology.numThreadsPerCore > 1)
+                        {
+                            skip_group = 1;
+                        }
+                    }
                     else if (bstrncmp(bbuf, LONG, 4) == 0)
                     {
                         read_long = 1;
@@ -286,6 +364,7 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
                 }
                 if (read_long)
                 {
+
                     (*grouplong)[i] = malloc((blength(long_info) + 1) * sizeof(char) );
                     if ((*grouplong)[i] != NULL)
                     {
@@ -294,18 +373,46 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
                     }
                 }
                 fclose(fp);
-                
+                if (skip_group)
+                {
+                    if ((*grouplong)[i] != NULL)
+                    {
+                        free((*grouplong)[i]);
+                        (*grouplong)[i] = NULL;
+                    }
+                    if ((*groupshort)[i] != NULL)
+                    {
+                        free((*groupshort)[i]);
+                        (*groupshort)[i] = NULL;
+                    }
+                    (*groupnames)[i][0] = '\0';
+                    bdestroy(long_info);
+                    goto skip_cur_def_group;
+                }
                 i++;
             }
             bdestroy(long_info);
         }
+skip_cur_def_group:
+        skip_group = 0;
     }
     closedir(dp);
     if (!search_home)
     {
+        if (i==0)
+            return_groups(totalgroups, *groupnames, *groupshort, *grouplong);
+        /*else if (i < totalgroups)
+        {
+            for (s=i;s<totalgroups;s++)
+            {
+                (*grouplong)[i] = NULL;
+                (*groupshort)[i] = NULL;
+            }
+        }*/
         free(homepath);
         free(fullpath);
         bdestroy(SHORT);
+        bdestroy(REQUIRE);
         bdestroy(LONG);
         return i;
     }
@@ -326,7 +433,6 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
                     (*groupnames)[i][s] = '\0';
                     fp = fopen(homepath,"r");
                     while (fgets (buf, sizeof(buf), fp)) {
-                        
                         bstring bbuf = bfromcstr(buf);
                         btrimws(bbuf);
                         if ((blength(bbuf) == 0) || (buf[0] == '#'))
@@ -363,17 +469,19 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
                                 bconcat(sinfo, tmp);
                                 bdestroy(tmp);
                             }
-
                             (*groupshort)[i] = malloc((blength(sinfo)+1) * sizeof(char));
                             if ((*groupshort)[i] == NULL)
                             {
                                 bdestroy(SHORT);
                                 bdestroy(LONG);
+                                bdestroy(REQUIRE);
                                 bdestroy(bbuf);
                                 bdestroy(sinfo);
                                 free(homepath);
                                 free(fullpath);
                                 bstrListDestroy(linelist);
+                                bdestroy(long_info);
+                                return_groups(i, *groupnames, *groupshort, *grouplong);
                                 return -ENOMEM;
                             }
                             s = sprintf((*groupshort)[i], "%s", bdata(sinfo));
@@ -381,6 +489,13 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
                             bstrListDestroy(linelist);
                             bdestroy(sinfo);
                         }
+                        else if (bstrncmp(bbuf, REQUIRE, blength(REQUIRE)) == 0)
+                        {
+                            if (cpuid_topology.numThreadsPerCore > 1)
+                            {
+                                skip_group = 1;
+                            }
+                        }
                         else if (bstrncmp(bbuf, LONG, 4) == 0)
                         {
                             read_long = 1;
@@ -403,43 +518,53 @@ int get_groups(char* grouppath, char* architecture, char*** groupnames, char***
                         }
                     }
                     fclose(fp);
+                    if (skip_group)
+                    {
+                        if ((*groupshort)[i])
+                        {
+                            free((*groupshort)[i]);
+                            (*groupshort)[i] = NULL;
+                        }
+                        if ((*grouplong)[i])
+                        {
+                            free((*grouplong)[i]);
+                            (*grouplong)[i] = NULL;
+                        }
+                        bdestroy(long_info);
+                        goto skip_cur_home_group;
+                    }
                     i++;
                 }
                 bdestroy(long_info);
             }
+skip_cur_home_group:
+        skip_group = 0;
         }
         closedir(dp);
     }
+    if (i==0)
+        return_groups(totalgroups, *groupnames, *groupshort, *grouplong);
+/*    else if (i < totalgroups)
+    {
+        for (s=i;s<totalgroups;s++)
+        {
+            printf("Setting NULL for group %d\n", s);
+            (*groupnames)[i] = NULL;
+            (*grouplong)[i] = NULL;
+            (*groupshort)[i] = NULL;
+        }
+    }*/
     bdestroy(SHORT);
     bdestroy(LONG);
+    bdestroy(REQUIRE);
     free(fullpath);
     free(homepath);
     return i;
 }
 
-void return_groups(int groups, char** groupnames, char** groupshort, char** grouplong)
-{
-    int i;
-    for (i = 0; i <groups; i++)
-    {
-        if (groupnames[i])
-            free(groupnames[i]);
-        if (groupshort[i])
-            free(groupshort[i]);
-        if (grouplong[i])
-            free(grouplong[i]);
-    }
-    if (groupnames)
-        free(groupnames);
-    if (groupshort)
-        free(groupshort);
-    if (grouplong)
-        free(grouplong);
-}
 
 
-
-int custom_group(char* eventStr, GroupInfo* ginfo)
+int custom_group(const char* eventStr, GroupInfo* ginfo)
 {
     int i, j;
     int err = 0;
@@ -462,7 +587,6 @@ int custom_group(char* eventStr, GroupInfo* ginfo)
     bstring fix1 = bformat("FIXC1");
     bstring fix2 = bformat("FIXC2");
     DEBUG_PRINT(DEBUGLEV_INFO, Creating custom group for event string %s, eventStr);
-    
     ginfo->shortinfo = malloc(7 * sizeof(char));
     if (ginfo->shortinfo == NULL)
     {
@@ -484,7 +608,6 @@ int custom_group(char* eventStr, GroupInfo* ginfo)
         goto cleanup;
     }
     sprintf(ginfo->groupname, "%s", "Custom");
-    
     eventBstr = bfromcstr(eventStr);
     eventList = bsplit(eventBstr, delim);
     ginfo->nevents = eventList->qty;
@@ -610,12 +733,18 @@ cleanup:
     return err;
 }
 
-int read_group(char* grouppath, char* architecture, char* groupname, GroupInfo* ginfo)
+int
+read_group(
+        const char* grouppath,
+        const char* architecture,
+        const char* groupname,
+        GroupInfo* ginfo)
 {
     FILE* fp;
     int i, s, e, err = 0;
     char buf[512];
     GroupFileSections sec = GROUP_NONE;
+    bstring REQUIRE = bformat("REQUIRE_NOHT");
     if ((grouppath == NULL)||(architecture == NULL)||(groupname == NULL)||(ginfo == NULL))
         return -EINVAL;
 
@@ -628,6 +757,7 @@ int read_group(char* grouppath, char* architecture, char* groupname, GroupInfo*
         if (access(bdata(homepath), R_OK))
         {
             ERROR_PRINT(Cannot read group file %s.txt. Searched in %s and %s, groupname, bdata(fullpath), bdata(homepath));
+            bdestroy(REQUIRE);
             bdestroy(fullpath);
             bdestroy(homepath);
             return -EACCES;
@@ -684,6 +814,15 @@ int read_group(char* grouppath, char* architecture, char* groupname, GroupInfo*
             sprintf(ginfo->shortinfo, "%.*s", (int)strlen(&(buf[i]))-1, &(buf[i]));
             continue;
         }
+        else if (strncmp(bdata(REQUIRE), buf, blength(REQUIRE)) == 0)
+        {
+            if (cpuid_topology.numThreadsPerCore > 1)
+            {
+                err = -ENODEV;
+                goto cleanup;
+            }
+            continue;
+        }
         else if (strncmp(groupFileSectionNames[GROUP_EVENTSET], buf, strlen(groupFileSectionNames[GROUP_EVENTSET])) == 0)
         {
             sec = GROUP_EVENTSET;
@@ -781,8 +920,6 @@ int read_group(char* grouppath, char* architecture, char* groupname, GroupInfo*
                 }
             }
             bstrListDestroy(linelist);
-            
-
             linelist = bsplit(bbuf, ' ');
             bdestroy(bbuf);
             for (i=0; i<linelist->qty; i++)
@@ -801,7 +938,6 @@ int read_group(char* grouppath, char* architecture, char* groupname, GroupInfo*
             }
             sprintf(ginfo->counters[ginfo->nevents], "%s", bdata(linelist->entry[0]));
             sprintf(ginfo->events[ginfo->nevents], "%s", bdata(linelist->entry[1]));
-            
             ginfo->nevents++;
             bstrListDestroy(linelist);
             continue;
@@ -915,10 +1051,12 @@ int read_group(char* grouppath, char* architecture, char* groupname, GroupInfo*
     }
     //bstrListDestroy(linelist);
     fclose(fp);
+    bdestroy(REQUIRE);
     bdestroy(homepath);
     bdestroy(fullpath);
     return 0;
 cleanup:
+    bdestroy(REQUIRE);
     bdestroy(homepath);
     bdestroy(fullpath);
     if (ginfo->groupname)
@@ -947,10 +1085,20 @@ cleanup:
                 free(ginfo->metricnames[i]);
         }
     }
+    /*ginfo->shortinfo = NULL;
+    ginfo->nevents = 0;
+    ginfo->events = NULL;
+    ginfo->counters = NULL;
+    ginfo->nmetrics = 0;
+    ginfo->metricformulas = NULL;
+    ginfo->metricnames = NULL;
+    ginfo->longinfo = NULL;
+    ginfo->groupname = NULL;*/
     return err;
 }
 
-int new_group(GroupInfo* ginfo)
+int
+new_group(GroupInfo* ginfo)
 {
     if (!ginfo)
         return -EINVAL;
@@ -966,7 +1114,8 @@ int new_group(GroupInfo* ginfo)
     return 0;
 }
 
-char* get_eventStr(GroupInfo* ginfo)
+char*
+get_eventStr(GroupInfo* ginfo)
 {
     int i;
     char* string;
@@ -994,7 +1143,8 @@ char* get_eventStr(GroupInfo* ginfo)
     return string;
 }
 
-void put_eventStr(char* eventset)
+void
+put_eventStr(char* eventset)
 {
     if (eventset != NULL)
     {
@@ -1003,7 +1153,8 @@ void put_eventStr(char* eventset)
     }
 }
 
-int add_event(GroupInfo* ginfo, char* event, char* counter)
+int
+add_event(GroupInfo* ginfo, char* event, char* counter)
 {
     if ((!ginfo) || (!event) || (!counter))
         return -EINVAL;
@@ -1025,7 +1176,8 @@ int add_event(GroupInfo* ginfo, char* event, char* counter)
     return 0;
 }
 
-int add_metric(GroupInfo* ginfo, char* mname, char* mcalc)
+int
+add_metric(GroupInfo* ginfo, char* mname, char* mcalc)
 {
     if ((!ginfo) || (!mname) || (!mcalc))
         return -EINVAL;
@@ -1048,7 +1200,8 @@ int add_metric(GroupInfo* ginfo, char* mname, char* mcalc)
 }
 
 
-char* get_groupName(GroupInfo* ginfo)
+char*
+get_groupName(GroupInfo* ginfo)
 {
     if ((ginfo != NULL) && (ginfo->groupname != NULL))
     {
@@ -1060,7 +1213,8 @@ char* get_groupName(GroupInfo* ginfo)
     return NULL;
 }
 
-int set_groupName(GroupInfo* ginfo, char* groupName)
+int
+set_groupName(GroupInfo* ginfo, char* groupName)
 {
     if ((ginfo == NULL) || (groupName == NULL))
         return -EINVAL;
@@ -1072,7 +1226,8 @@ int set_groupName(GroupInfo* ginfo, char* groupName)
     return 0;
 }
 
-char* get_shortInfo(GroupInfo* ginfo)
+char*
+get_shortInfo(GroupInfo* ginfo)
 {
     if ((ginfo != NULL) && (ginfo->shortinfo != NULL))
     {
@@ -1084,7 +1239,8 @@ char* get_shortInfo(GroupInfo* ginfo)
     return NULL;
 }
 
-void put_shortInfo(char* sinfo)
+void
+put_shortInfo(char* sinfo)
 {
     if (sinfo != NULL)
     {
@@ -1093,7 +1249,8 @@ void put_shortInfo(char* sinfo)
     }
 }
 
-int set_shortInfo(GroupInfo* ginfo, char* shortInfo)
+int
+set_shortInfo(GroupInfo* ginfo, char* shortInfo)
 {
     if ((ginfo == NULL) || (shortInfo == NULL))
         return -EINVAL;
@@ -1105,7 +1262,8 @@ int set_shortInfo(GroupInfo* ginfo, char* shortInfo)
     return 0;
 }
 
-char* get_longInfo(GroupInfo* ginfo)
+char*
+get_longInfo(GroupInfo* ginfo)
 {
     if ((ginfo != NULL) && (ginfo->longinfo != NULL))
     {
@@ -1117,7 +1275,8 @@ char* get_longInfo(GroupInfo* ginfo)
     return NULL;
 }
 
-void put_longInfo(char* linfo)
+void
+put_longInfo(char* linfo)
 {
     if (linfo != NULL)
     {
@@ -1126,7 +1285,8 @@ void put_longInfo(char* linfo)
     }
 }
 
-int set_longInfo(GroupInfo* ginfo, char* longInfo)
+int
+set_longInfo(GroupInfo* ginfo, char* longInfo)
 {
     if ((ginfo == NULL) || (longInfo == NULL))
         return -EINVAL;
@@ -1138,7 +1298,8 @@ int set_longInfo(GroupInfo* ginfo, char* longInfo)
     return 0;
 }
 
-void return_group(GroupInfo* ginfo)
+void
+return_group(GroupInfo* ginfo)
 {
     int i;
     if (ginfo->groupname)
@@ -1182,14 +1343,16 @@ void return_group(GroupInfo* ginfo)
     ginfo->nmetrics = 0;
 }
 
-void init_clist(CounterList* clist)
+void
+init_clist(CounterList* clist)
 {
     clist->counters = 0;
     clist->cnames = NULL;
     clist->cvalues = NULL;
 }
 
-int add_to_clist(CounterList* clist, char* counter, double result)
+int
+add_to_clist(CounterList* clist, char* counter, double result)
 {
     char** tmpnames;
     double* tmpvalues;
@@ -1218,7 +1381,31 @@ int add_to_clist(CounterList* clist, char* counter, double result)
     return 0;
 }
 
-void destroy_clist(CounterList* clist)
+int
+update_clist(CounterList* clist, char* counter, double result)
+{
+    int i;
+    int found = 0;
+    if ((clist == NULL)||(counter == NULL))
+        return -EINVAL;
+    for (i=0; i< clist->counters; i++)
+    {
+        if (strcmp(clist->cnames[i], counter) == 0)
+        {
+            clist->cvalues[i] = result;
+            found = 1;
+            break;
+        }
+    }
+    if (!found)
+    {
+        return -ENOENT;
+    }
+    return 0;
+}
+
+void
+destroy_clist(CounterList* clist)
 {
     int i;
     if (clist != NULL)
@@ -1232,12 +1419,11 @@ void destroy_clist(CounterList* clist)
     }
 }
 
-
-int calc_metric(char* formula, CounterList* clist, double *result)
+int
+calc_metric(char* formula, CounterList* clist, double *result)
 {
     int i=0;
     *result = 0.0;
-    int fail = 0;
     int maxstrlen = 0, minstrlen = 10000;
 
     if ((formula == NULL) || (clist == NULL))
@@ -1268,18 +1454,9 @@ int calc_metric(char* formula, CounterList* clist, double *result)
         }
         maxstrlen--;
     }
-    bstring test = bfromcstr("aAbBcCdDfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVwWxXyYzZ,_:;!'§$&=?°´`#<>");
-    if (binchr(f, 0, test) != BSTR_ERR)
-    {
-        fprintf(stderr, "Not all counter names in formula can be substituted\n");
-        fprintf(stderr, "%s\n", bdata(f));
-        i = -EINVAL;
-        fail = 1;
-    }
-    bdestroy(test);
     // now we can calculate the formula
-    if (!fail)
-        i = calculate_infix(bdata(f), result);
+    i = calculate_infix(bdata(f), result);
     bdestroy(f);
     return i;
 }
+
