@@ -5,8 +5,8 @@
  *
  *      Description:  Module implementing Intel RAPL interface
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
@@ -28,6 +28,9 @@
  *
  * =======================================================================================
  */
+
+/* #####   HEADER FILE INCLUDES   ######################################### */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
@@ -40,9 +43,9 @@
 
 PowerInfo power_info;
 
-/* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
-static int power_initialized = 0;
+/* #####   LOCAL VARIABLES   ############################################## */
 
+static int power_initialized = 0;
 
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
 
@@ -67,6 +70,9 @@ power_init(int cpuId)
     power_info.powerUnit = 0;
     power_info.timeUnit = 0;
     power_info.hasRAPL = 0;
+    power_info.uncoreMinFreq = 0;
+    power_info.uncoreMaxFreq = 0;
+    power_info.perfBias = 0;
 
     switch (cpuid_info.model)
     {
@@ -89,6 +95,9 @@ power_init(int cpuId)
         case HASWELL_M2:
         case SKYLAKE1:
         case SKYLAKE2:
+        case KABYLAKE1:
+        case KABYLAKE2:
+        case XEON_PHI_KNL:
             power_info.hasRAPL = 1;
             break;
         case ATOM_SILVERMONT_C:
@@ -107,8 +116,15 @@ power_init(int cpuId)
     if (!HPMinitialized())
     {
         HPMinit();
-        HPMaddThread(cpuId);
+        err = HPMaddThread(cpuId);
+        if (err != 0)
+        {
+            ERROR_PLAIN_PRINT(Cannot get access to RAPL counters)
+            return err;
+        }
     }
+
+
     if ( power_info.hasRAPL )
     {
         busSpeed = 100.0;
@@ -155,9 +171,42 @@ power_init(int cpuId)
                     }
                 }
             }
-            //TODO: Haswell EP and possibly Broadwell EP support multiple turbo 
-            //      registers besides MSR_TURBO_RATIO_LIMIT:
-            //      MSR_TURBO_RATIO_LIMIT1 and MSR_TURBO_RATIO_LIMIT2
+            if (power_info.turbo.numSteps > 8)
+            {
+                err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT1, &flags);
+                if (!err)
+                {
+                    for (int i=8; i < power_info.turbo.numSteps; i++)
+                    {
+                        if (i < 16)
+                        {
+                            power_info.turbo.steps[i] = busSpeed * (double) field64(flags,i*8, 8);
+                        }
+                        else
+                        {
+                            power_info.turbo.steps[i] = power_info.turbo.steps[15];
+                        }
+                    }
+                }
+            }
+            if (power_info.turbo.numSteps > 16)
+            {
+                err = HPMread(cpuId, MSR_DEV, MSR_TURBO_RATIO_LIMIT2, &flags);
+                if (!err)
+                {
+                    for (int i=16; i < power_info.turbo.numSteps; i++)
+                    {
+                        if (i < 24)
+                        {
+                            power_info.turbo.steps[i] = busSpeed * (double) field64(flags,i*8, 8);
+                        }
+                        else
+                        {
+                            power_info.turbo.steps[i] = power_info.turbo.steps[23];
+                        }
+                    }
+                }
+            }
         }
         else
         {
@@ -182,7 +231,6 @@ power_init(int cpuId)
             {
                 energyUnit = 1.0 * (1 << ((flags >> 8) & 0x1F)) / 1000000;
             }
-            
             for (i = 0; i < NUM_POWER_DOMAINS; i++)
             {
                 power_info.domains[i].energyUnit = energyUnit;
@@ -193,10 +241,12 @@ power_init(int cpuId)
                 power_info.domains[i].maxPower = 0.0;
                 power_info.domains[i].maxTimeWindow = 0.0;
             }
-            
             if ((cpuid_info.model == HASWELL_EP) ||
                 (cpuid_info.model == HASWELL_M1) ||
-                (cpuid_info.model == HASWELL_M2))
+                (cpuid_info.model == HASWELL_M2) ||
+                (cpuid_info.model == BROADWELL_D) ||
+                (cpuid_info.model == BROADWELL_E) ||
+                (cpuid_info.model == XEON_PHI_KNL))
             {
                 power_info.domains[DRAM].energyUnit = 15.3E-6;
             }
@@ -279,6 +329,18 @@ power_init(int cpuId)
             fprintf(stderr,"Cannot gather values from MSR_RAPL_POWER_UNIT, deactivating RAPL support\n");
             power_info.hasRAPL =  0;
         }
+
+        err = HPMread(cpuId, MSR_DEV, MSR_UNCORE_FREQ, &flags);
+        if (err == 0)
+        {
+            power_info.uncoreMinFreq = ((double)((flags >> 8) & 0xFFULL)) * busSpeed;
+            power_info.uncoreMaxFreq = ((double)(flags & 0xFF)) * busSpeed;
+        }
+        err = HPMread(cpuId, MSR_DEV, MSR_ENERGY_PERF_BIAS, &flags);
+        if (err == 0)
+        {
+            power_info.perfBias = flags & 0xF;
+        }
         power_initialized = 1;
         return power_info.hasRAPL;
     }
@@ -290,7 +352,8 @@ power_init(int cpuId)
 }
 
 /* All functions below are experimental and probably don't work */
-int power_perfGet(int cpuId, PowerType domain, uint32_t* status)
+int
+power_perfGet(int cpuId, PowerType domain, uint32_t* status)
 {
     int err = 0;
     *status = 0x0U;
@@ -310,7 +373,8 @@ int power_perfGet(int cpuId, PowerType domain, uint32_t* status)
     return 0;
 }
 
-int power_limitSet(int cpuId, PowerType domain, double power, double time, int doClamping)
+int
+power_limitSet(int cpuId, PowerType domain, double power, double time, int doClamping)
 {
     int err = 0;
     if (domain >= NUM_POWER_DOMAINS)
@@ -340,7 +404,8 @@ int power_limitSet(int cpuId, PowerType domain, double power, double time, int d
     return 0;
 }
 
-int power_limitGet(int cpuId, PowerType domain, double* power, double* time)
+int
+power_limitGet(int cpuId, PowerType domain, double* power, double* time)
 {
     int err = 0;
     *power = 0;
@@ -367,7 +432,8 @@ int power_limitGet(int cpuId, PowerType domain, double* power, double* time)
     return 0;
 }
 
-int power_limitState(int cpuId, PowerType domain)
+int
+power_limitState(int cpuId, PowerType domain)
 {
     int err = 0;
     if (domain >= NUM_POWER_DOMAINS)
@@ -392,7 +458,8 @@ int power_limitState(int cpuId, PowerType domain)
     return 0;
 }
 
-int power_limitActivate(int cpuId, PowerType domain)
+int
+power_limitActivate(int cpuId, PowerType domain)
 {
     int err = 0;
     if (domain >= NUM_POWER_DOMAINS)
@@ -420,7 +487,8 @@ int power_limitActivate(int cpuId, PowerType domain)
     return 0;
 }
 
-int power_limitDectivate(int cpuId, PowerType domain)
+int
+power_limitDectivate(int cpuId, PowerType domain)
 {
     int err = 0;
     uint64_t flags = 0x0ULL;
@@ -444,7 +512,8 @@ int power_limitDectivate(int cpuId, PowerType domain)
     return 0;
 }
 
-int power_policySet(int cpuId, PowerType domain, uint32_t priority)
+int
+power_policySet(int cpuId, PowerType domain, uint32_t priority)
 {
     int err = 0;
     if (domain >= NUM_POWER_DOMAINS)
@@ -464,7 +533,8 @@ int power_policySet(int cpuId, PowerType domain, uint32_t priority)
     return 0;
 }
 
-int power_policyGet(int cpuId, PowerType domain, uint32_t* priority)
+int
+power_policyGet(int cpuId, PowerType domain, uint32_t* priority)
 {
     int err = 0;
     *priority = 0x0U;
@@ -484,8 +554,8 @@ int power_policyGet(int cpuId, PowerType domain, uint32_t* priority)
     return 0;
 }
 
-
-void power_finalize(void)
+void
+power_finalize(void)
 {
     if (power_initialized == 0)
     {
@@ -502,10 +572,14 @@ void power_finalize(void)
     power_info.powerUnit = 0;
     power_info.timeUnit = 0;
     power_info.hasRAPL = 0;
+    power_info.uncoreMinFreq = 0;
+    power_info.uncoreMaxFreq = 0;
     memset(power_info.domains, 0, NUM_POWER_DOMAINS*sizeof(PowerDomain));
 }
 
-PowerInfo_t get_powerInfo(void)
+PowerInfo_t
+get_powerInfo(void)
 {
     return &power_info;
 }
+
