@@ -5,8 +5,8 @@
  *
  *      Description:  Interface to the topology backends
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
@@ -28,6 +28,9 @@
  *
  * =======================================================================================
  */
+
+/* #####   HEADER FILE INCLUDES   ######################################### */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <sys/types.h>
@@ -43,13 +46,9 @@
 //#include <strUtil.h>
 #include <configuration.h>
 
+/* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
 
 static int topology_initialized = 0;
-CpuInfo cpuid_info;
-CpuTopology cpuid_topology;
-
-int affinity_thread2tile_lookup[MAX_NUM_THREADS];
-
 static char* pentium_m_b_str = "Intel Pentium M Banias processor";
 static char* pentium_m_d_str = "Intel Pentium M Dothan processor";
 static char* core_duo_str = "Intel Core Duo processor";
@@ -74,11 +73,12 @@ static char* broadwell_str = "Intel Core Broadwell processor";
 static char* broadwell_d_str = "Intel Xeon D Broadwell processor";
 static char* broadwell_ep_str = "Intel Xeon Broadwell EN/EP/EX processor";
 static char* skylake_str = "Intel Skylake processor";
+static char* kabylake_str = "Intel Kabylake processor";
 static char* nehalem_ex_str = "Intel Nehalem EX processor";
 static char* westmere_ex_str = "Intel Westmere EX processor";
 static char* xeon_mp_string = "Intel Xeon MP processor";
 static char* xeon_phi_string = "Intel Xeon Phi (Knights Corner) Coprocessor";
-static char* xeon_phi2_string = "Intel Xeon Phi (Knights Landing) Coprocessor";
+static char* xeon_phi2_string = "Intel Xeon Phi (Knights Landing) (Co)Processor";
 static char* barcelona_str = "AMD Barcelona processor";
 static char* shanghai_str = "AMD Shanghai processor";
 static char* istanbul_str = "AMD Istanbul processor";
@@ -115,45 +115,25 @@ static char* short_ivybridge_ep = "ivybridgeEP";
 static char* short_sandybridge = "sandybridge";
 static char* short_sandybridge_ep = "sandybridgeEP";
 static char* short_skylake = "skylake";
+static char* short_kabylake = "skylake";
 static char* short_phi = "phi";
-static char* short_phi2 = "phi2";
+static char* short_phi2 = "knl";
 static char* short_k8 = "k8";
 static char* short_k10 = "k10";
 static char* short_k15 = "interlagos";
 static char* short_k16 = "kabini";
 static char* short_unknown = "unknown";
 
+/* #####  EXPORTED VARIABLES  ########################################## */
 
+CpuInfo cpuid_info;
+CpuTopology cpuid_topology;
+int affinity_thread2tile_lookup[MAX_NUM_THREADS];
 
-int cpu_count(cpu_set_t* set)
-{
-    uint32_t i;
-    int s = 0;
-    const __cpu_mask *p = set->__bits;
-    const __cpu_mask *end = &set->__bits[sizeof(cpu_set_t) / sizeof (__cpu_mask)];
+/* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
 
-    while (p < end)
-    {
-        __cpu_mask l = *p++;
-
-        if (l == 0)
-        {
-            continue;
-        }
-
-        for (i=0; i< (sizeof(__cpu_mask)*8); i++)
-        {
-            if (l&(1UL<<i))
-            {
-                s++;
-            }
-        }
-    }
-
-    return s;
-}
-
-static void initTopologyFile(FILE* file)
+static void
+initTopologyFile(FILE* file)
 {
     size_t items;
     HWThread* hwThreadPool;
@@ -198,8 +178,8 @@ static void initTopologyFile(FILE* file)
     }
 }
 
-
-static int readTopologyFile(const char* filename)
+static int
+readTopologyFile(const char* filename)
 {
     FILE* fp;
     char structure[256];
@@ -516,7 +496,39 @@ static int readTopologyFile(const char* filename)
     return 0;
 }
 
-int topology_setName(void)
+/* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
+
+int
+cpu_count(cpu_set_t* set)
+{
+    uint32_t i;
+    int s = 0;
+    const __cpu_mask *p = set->__bits;
+    const __cpu_mask *end = &set->__bits[sizeof(cpu_set_t) / sizeof (__cpu_mask)];
+
+    while (p < end)
+    {
+        __cpu_mask l = *p++;
+
+        if (l == 0)
+        {
+            continue;
+        }
+
+        for (i=0; i< (sizeof(__cpu_mask)*8); i++)
+        {
+            if (l&(1UL<<i))
+            {
+                s++;
+            }
+        }
+    }
+    return s;
+}
+
+
+int
+topology_setName(void)
 {
     switch ( cpuid_info.family )
     {
@@ -604,10 +616,12 @@ int topology_setName(void)
                     cpuid_info.short_name = short_broadwell;
                     break;
                 case BROADWELL_D:
+                    cpuid_info.supportUncore = 1;
                     cpuid_info.name = broadwell_d_str;
                     cpuid_info.short_name = short_broadwell_d;
                     break;
                 case BROADWELL_E:
+                    cpuid_info.supportUncore = 1;
                     cpuid_info.name = broadwell_ep_str;
                     cpuid_info.short_name = short_broadwell_ep;
                     break;
@@ -618,7 +632,14 @@ int topology_setName(void)
                     cpuid_info.short_name = short_skylake;
                     break;
 
-                case XEON_PHI2:
+                case KABYLAKE1:
+                case KABYLAKE2:
+                    cpuid_info.name = kabylake_str;
+                    cpuid_info.short_name = short_skylake;
+                    break;
+
+                case XEON_PHI_KNL:
+                    cpuid_info.supportUncore = 1;
                     cpuid_info.name = xeon_phi2_string;
                     cpuid_info.short_name = short_phi2;
                     break;
@@ -783,7 +804,8 @@ int topology_setName(void)
     return EXIT_SUCCESS;
 }
 
-const struct topology_functions topology_funcs = {
+const struct
+topology_functions topology_funcs = {
 #ifndef LIKWID_USE_HWLOC
     .init_cpuInfo = cpuid_init_cpuInfo,
     .init_cpuFeatures = cpuid_init_cpuFeatures,
@@ -837,15 +859,22 @@ void topology_setupTree(void)
         }
 
     }
-    cpuid_topology.numSockets = tree_countChildren(cpuid_topology.topologyTree);
+    i = tree_countChildren(cpuid_topology.topologyTree);
+    if (cpuid_topology.numSockets == 0)
+        cpuid_topology.numSockets = i;
     currentNode = tree_getChildNode(cpuid_topology.topologyTree);
-    cpuid_topology.numCoresPerSocket = tree_countChildren(currentNode);
+    i = tree_countChildren(currentNode);
+    if (cpuid_topology.numCoresPerSocket == 0)
+        cpuid_topology.numCoresPerSocket = i;
     currentNode = tree_getChildNode(currentNode);
-    cpuid_topology.numThreadsPerCore = tree_countChildren(currentNode);
+    i = tree_countChildren(currentNode);
+    if (cpuid_topology.numThreadsPerCore == 0)
+        cpuid_topology.numThreadsPerCore = i;
     return;
 }
 
-int topology_init(void)
+int
+topology_init(void)
 {
     int ret = 0;
     cpu_set_t cpuSet;
@@ -886,8 +915,8 @@ standard_init:
         topology_setName();
         funcs.init_cpuFeatures();
         funcs.init_nodeTopology(cpuSet);
-        topology_setupTree();
         funcs.init_cacheTopology();
+        topology_setupTree();
         sched_setaffinity(0, sizeof(cpu_set_t), &cpuSet);
     }
     else
@@ -917,7 +946,8 @@ standard_init:
 }
 
 
-void topology_finalize(void)
+void
+topology_finalize(void)
 {
     struct topology_functions funcs = topology_funcs;
     if (!topology_initialized)
@@ -978,11 +1008,8 @@ void topology_finalize(void)
     topology_initialized = 0;
 }
 
-
-
-
-
-void print_supportedCPUs (void)
+void
+print_supportedCPUs (void)
 {
     printf("Supported Intel processors:\n");
     printf("\t%s\n",core_2a_str);
@@ -1024,18 +1051,20 @@ void print_supportedCPUs (void)
     printf("\n");
 }
 
-
-
-CpuTopology_t get_cpuTopology(void)
+CpuTopology_t
+get_cpuTopology(void)
 {
     return &cpuid_topology;
 }
 
-CpuInfo_t get_cpuInfo(void)
+CpuInfo_t
+get_cpuInfo(void)
 {
     return &cpuid_info;
 }
-NumaTopology_t get_numaTopology(void)
+
+NumaTopology_t
+get_numaTopology(void)
 {
     return &numa_info;
 }
