@@ -5,8 +5,8 @@
  *
  *      Description:  Interface to the hwloc based topology backend
  *
- *      Version:   4.1
- *      Released:  8.8.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Authors:  Thomas Roehl (tr), thomas.roehl@googlemail.com
  *
@@ -28,29 +28,34 @@
  *
  * =======================================================================================
  */
- 
+
+/* #####   HEADER FILE INCLUDES   ######################################### */
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <error.h>
 
 #include <topology.h>
+#include <affinity.h>
 #ifdef LIKWID_USE_HWLOC
 #include <hwloc.h>
 #include <topology_hwloc.h>
 #endif
 
+/* #####  EXPORTED VARIABLESE   ########################################### */
+
 hwloc_topology_t hwloc_topology = NULL;
 
-
-/* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
-
-/* #####   VARIABLES  -  LOCAL TO THIS SOURCE FILE   ###################### */
-
-/* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
-
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
+
 #ifdef LIKWID_USE_HWLOC
-int likwid_hwloc_record_objs_of_type_below_obj(hwloc_topology_t t, hwloc_obj_t obj, hwloc_obj_type_t type, int* index, uint32_t **list)
+int
+likwid_hwloc_record_objs_of_type_below_obj(
+        hwloc_topology_t t,
+        hwloc_obj_t obj,
+        hwloc_obj_type_t type,
+        int* index,
+        uint32_t **list)
 {
     int i;
     int count = 0;
@@ -73,7 +78,8 @@ int likwid_hwloc_record_objs_of_type_below_obj(hwloc_topology_t t, hwloc_obj_t o
     return count;
 }
 
-void hwloc_init_cpuInfo(cpu_set_t cpuSet)
+void
+hwloc_init_cpuInfo(cpu_set_t cpuSet)
 {
     int i;
     hwloc_obj_t obj;
@@ -120,15 +126,16 @@ void hwloc_init_cpuInfo(cpu_set_t cpuSet)
     return;
 }
 
-void hwloc_init_nodeTopology(cpu_set_t cpuSet)
+void
+hwloc_init_nodeTopology(cpu_set_t cpuSet)
 {
-    HWThread* hwThreadPool;
-    int maxNumLogicalProcs;
-    int maxNumLogicalProcsPerCore;
-    int maxNumCores;
-    int maxNumSockets;
-    int maxNumCoresPerSocket;
-    hwloc_obj_t obj;
+    HWThread* hwThreadPool = NULL;
+    int maxNumLogicalProcs = 0;
+    int maxNumLogicalProcsPerCore = 0;
+    int maxNumCores = 0;
+    int maxNumSockets = 0;
+    int maxNumCoresPerSocket = 0;
+    hwloc_obj_t obj = NULL;
     int poolsize = 0;
     int nr_sockets = 1;
     int id = 0;
@@ -199,50 +206,70 @@ void hwloc_init_nodeTopology(cpu_set_t cpuSet)
             {
                 hwThreadPool[id].coreId = 0;
                 hwThreadPool[id].packageId = 0;
+                affinity_thread2core_lookup[hwThreadPool[id].apicId] = hwThreadPool[id].coreId;
                 continue;
             }
-            hwThreadPool[id].coreId = obj->os_index;
         }
-        else
+        if (skip)
         {
-            hwThreadPool[id].coreId = hwThreadPool[id].apicId % maxNumCoresPerSocket;
+            hwThreadPool[id].coreId = 0;
+            hwThreadPool[id].packageId = 0;
+            continue;
         }
-        if (maxNumSockets > 1)
+        hwThreadPool[id].coreId = obj->logical_index;
+#if defined(__x86_64) || defined(__i386__)
+        if (maxNumLogicalProcsPerCore == 1 && cpuid_info.isIntel == 0)
         {
-            while (obj->type != socket_type) {
-                obj = obj->parent;
-                if (!obj)
+            if (id == 0)
+            {
+                hwThreadPool[id].coreId = hwThreadPool[id].apicId;
+            }
+            else
+            {
+                if (hwThreadPool[id].apicId == hwThreadPool[id-1].apicId + 1 &&
+                    hwThreadPool[id].packageId == hwThreadPool[id-1].packageId)
                 {
-                    skip = 1;
-                    break;
+                    hwThreadPool[id].coreId = hwThreadPool[id].apicId % maxNumCoresPerSocket;
+                }
+                else
+                {
+                    hwThreadPool[id].coreId = hwThreadPool[id].apicId;
                 }
             }
-            if (skip)
-            {
-                hwThreadPool[id].packageId = 0;
-                continue;
-            }
-            hwThreadPool[id].packageId = obj->os_index;
         }
-        else
+#endif
+        affinity_thread2core_lookup[hwThreadPool[id].apicId] = hwThreadPool[id].coreId;
+        while (obj->type != socket_type) {
+            obj = obj->parent;
+            if (!obj)
+            {
+                skip = 1;
+                break;
+            }
+        }
+        if (skip)
         {
             hwThreadPool[id].packageId = 0;
+            continue;
         }
+        hwThreadPool[id].packageId = obj->os_index;
         DEBUG_PRINT(DEBUGLEV_DEVELOP, HWLOC Thread Pool PU %d Thread %d Core %d Socket %d inCpuSet %d,
-                            hwThreadPool[i].apicId,
-                            hwThreadPool[i].threadId,
-                            hwThreadPool[i].coreId,
-                            hwThreadPool[i].packageId,
-                            hwThreadPool[i].inCpuSet)
+                            hwThreadPool[id].apicId,
+                            hwThreadPool[id].threadId,
+                            hwThreadPool[id].coreId,
+                            hwThreadPool[id].packageId,
+                            hwThreadPool[id].inCpuSet)
     }
 
     cpuid_topology.threadPool = hwThreadPool;
-
+    cpuid_topology.numThreadsPerCore = maxNumLogicalProcsPerCore;
+    cpuid_topology.numCoresPerSocket = maxNumCoresPerSocket;
+    cpuid_topology.numSockets = maxNumSockets;
     return;
 }
 
-
-void hwloc_init_cacheTopology(void)
+void
+hwloc_init_cacheTopology(void)
 {
     int maxNumLevels=0;
     int id=0;
@@ -263,7 +290,6 @@ void hwloc_init_cacheTopology(void)
     /* Start at the bottom of the tree to get all cache levels in order */
     depth = likwid_hwloc_topology_get_depth(hwloc_topology);
     id = 0;
-    
     for(d=depth-1;d >= 0; d--)
     {
         /* We only need caches, so skip other levels */
@@ -271,6 +297,14 @@ void hwloc_init_cacheTopology(void)
         {
             continue;
         }
+        cachePool[id].level = 0;
+        cachePool[id].type = NOCACHE;
+        cachePool[id].associativity = 0;
+        cachePool[id].lineSize = 0;
+        cachePool[id].size = 0;
+        cachePool[id].sets = 0;
+        cachePool[id].inclusive = 0;
+        cachePool[id].threads = 0;
         /* Get the cache object */
         obj = likwid_hwloc_get_obj_by_depth(hwloc_topology, d, 0);
         /* All caches have this attribute, so safe to access */
@@ -304,7 +338,7 @@ void hwloc_init_cacheTopology(void)
         /* Count all HWThreads below the current cache */
         cachePool[id].threads = likwid_hwloc_record_objs_of_type_below_obj(
                         hwloc_topology, obj, HWLOC_OBJ_PU, NULL, NULL);
-
+#if defined(__x86_64) || defined(__i386__)
         while (!(info = likwid_hwloc_obj_get_info_by_name(obj, "inclusiveness")) && obj->next_cousin)
         {
             obj = obj->next_cousin; // If some PU/core are not bindable because of cgroup, hwloc may not know the inclusiveness of some of their cache.
@@ -313,11 +347,18 @@ void hwloc_init_cacheTopology(void)
         {
             cachePool[id].inclusive = info[0]=='t';
         }
-        else
+        else if (cpuid_info.isIntel)
         {
-            ERROR_PLAIN_PRINT(Processor is not supported);
+            DEBUG_PLAIN_PRINT(DEBUGLEV_ONLY_ERROR, Processor is not supported);
             break;
         }
+#endif
+#if defined(_ARCH_PPC)
+        cachePool[id].inclusive = 0;
+#endif
+        DEBUG_PRINT(DEBUGLEV_DEVELOP, HWLOC Cache Pool ID %d Level %d Size %d,
+                                      id, cachePool[id].level,
+                                      cachePool[id].size);
         id++;
     }
 
@@ -326,17 +367,17 @@ void hwloc_init_cacheTopology(void)
     return;
 }
 
-void hwloc_close(void)
+void
+hwloc_close(void)
 {
     if (hwloc_topology)
     {
         hwloc_topology_destroy(hwloc_topology);
     }
 }
-
 #else
-
-void hwloc_init_cpuInfo(void)
+void
+hwloc_init_cpuInfo(void)
 {
     return;
 }
