@@ -5,8 +5,8 @@
  *
  *      Description:  Interface to the procfs/sysfs based topology backend
  *
- *      Version:   4.1
- *      Released:  13.6.2016
+ *      Version:   <VERSION>
+ *      Released:  <DATE>
  *
  *      Authors:  Jan Treibig (jt), jan.treibig@gmail.com,
  *                Thomas Roehl (tr), thomas.roehl@googlemail.com
@@ -29,18 +29,27 @@
  * =======================================================================================
  */
 
+/* #####   HEADER FILE INCLUDES   ######################################### */
+
 #include <topology_proc.h>
+#include <affinity.h>
 #include <cpuid.h>
 
-/* #####   MACROS  -  LOCAL TO THIS SOURCE FILE   ######################### */
 /* #####   FUNCTION DEFINITIONS  -  LOCAL TO THIS SOURCE FILE   ########### */
-static int get_cpu_perf_data(void)
+
+static int
+get_cpu_perf_data(void)
 {
     uint32_t eax = 0x0U, ebx = 0x0U, ecx = 0x0U, edx = 0x0U;
     int largest_function = 0;
     eax = 0x00;
     CPUID(eax, ebx, ecx, edx);
     largest_function = eax;
+    cpuid_info.perf_version = 0;
+    cpuid_info.perf_num_ctr = 0;
+    cpuid_info.perf_width_ctr = 0;
+    cpuid_info.perf_num_fixed_ctr = 0;
+    cpuid_info.turbo = 0;
     if (cpuid_info.family == P6_FAMILY && 0x0A <= largest_function)
     {
         eax = 0x0A;
@@ -64,7 +73,8 @@ static int get_cpu_perf_data(void)
     return 0;
 }
 
-int get_listPosition(int ownid, bstring list)
+static int
+get_listPosition(int ownid, bstring list)
 {
     bstring ownStr = bformat("%d",ownid);
     struct bstrList* tokens = bsplit(list,(char) ',');
@@ -80,7 +90,8 @@ int get_listPosition(int ownid, bstring list)
     return -1;
 }
 
-int fillList(int* outList, int outOffset, bstring list)
+static int
+fillList(int* outList, int outOffset, bstring list)
 {
     int current = 0;
     int (*ownatoi)(const char*);
@@ -108,7 +119,6 @@ int fillList(int* outList, int outOffset, bstring list)
                     {
                         outList[outOffset+current] = j;
                     }
-                    
                     current++;
                 }
             }
@@ -119,7 +129,8 @@ int fillList(int* outList, int outOffset, bstring list)
     return current;
 }
 
-static int readCacheInclusiveIntel(int level)
+static int
+readCacheInclusiveIntel(int level)
 {
     uint32_t eax = 0x0U, ebx = 0x0U, ecx = 0x0U, edx = 0x0U;
     eax = 0x04;
@@ -138,7 +149,9 @@ static int readCacheInclusiveAMD(int level)
 }
 
 /* #####   FUNCTION DEFINITIONS  -  EXPORTED FUNCTIONS   ################## */
-void proc_init_cpuInfo(cpu_set_t cpuSet)
+
+void
+proc_init_cpuInfo(cpu_set_t cpuSet)
 {
     int i = 0;
     int HWthreads = 0;
@@ -164,7 +177,7 @@ void proc_init_cpuInfo(cpu_set_t cpuSet)
     cpuid_topology.numHWThreads = 0;
     cpuid_info.osname = malloc(MAX_MODEL_STRING_LENGTH * sizeof(char));
 
-    if (NULL != (fp = fopen ("/proc/cpuinfo", "r"))) 
+    if (NULL != (fp = fopen ("/proc/cpuinfo", "r")))
     {
         bstring src = bread ((bNread) fread, fp);
         struct bstrList* tokens = bsplit(src,(char) '\n');
@@ -221,7 +234,8 @@ void proc_init_cpuInfo(cpu_set_t cpuSet)
     return;
 }
 
-void proc_init_cpuFeatures(void)
+void
+proc_init_cpuFeatures(void)
 {
     int ret;
     FILE* file;
@@ -391,9 +405,8 @@ void proc_init_cpuFeatures(void)
     return;
 }
 
-
-
-void proc_init_nodeTopology(cpu_set_t cpuSet)
+void
+proc_init_nodeTopology(cpu_set_t cpuSet)
 {
     HWThread* hwThreadPool;
     FILE *fp;
@@ -420,6 +433,7 @@ void proc_init_nodeTopology(cpu_set_t cpuSet)
         {
             bstring src = bread ((bNread) fread, fp);
             hwThreadPool[i].coreId = ownatoi(bdata(src));
+            affinity_thread2core_lookup[hwThreadPool[i].apicId] = hwThreadPool[i].coreId;
             fclose(fp);
         }
         bdestroy(file);
@@ -451,7 +465,8 @@ void proc_init_nodeTopology(cpu_set_t cpuSet)
     return;
 }
 
-void proc_init_cacheTopology(void)
+void
+proc_init_cacheTopology(void)
 {
     FILE *fp;
     CacheLevel* cachePool = NULL;
